// vim: ts=2 sw=2 expandtab ai
//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2007
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

// Ian's second Object RDF Mapping
//   look at cooptest.js

// TODO:
//  * lazy loading of properties (with caching)
//  * too much is done in ctor - it should be more static - things put into
//  the protototype.
//  * real exception class, not strings

const nsIRDFService           = Components.interfaces.nsIRDFService;
const nsIRDFContainerUtils    = Components.interfaces.nsIRDFContainerUtils;
const nsIRDFContainer         = Components.interfaces.nsIRDFContainer;
const nsIRDFResource          = Components.interfaces.nsIRDFResource;
const nsIRDFNode              = Components.interfaces.nsIRDFNode;
const nsIRDFLiteral           = Components.interfaces.nsIRDFLiteral;
const nsIRDFInt               = Components.interfaces.nsIRDFInt;
const nsIRDFDate              = Components.interfaces.nsIRDFDate;
const flockIRDFContainer      = Components.interfaces.flockIRDFContainer;
const flockIRDFService        = Components.interfaces.flockIRDFService;
const RDFSERVICE_CONTRACTID   = '@mozilla.org/rdf/rdf-service;1';
const RDFCONTUTILS_CONTRACTID = '@mozilla.org/rdf/container-utils;1';
const CONTAINER_CONTRACTID    = '@mozilla.org/rdf/container;1';

const RDFS = Components.classes[RDFSERVICE_CONTRACTID].getService (nsIRDFService);
const RDFCU = Components.classes[RDFCONTUTILS_CONTRACTID].getService (nsIRDFContainerUtils);
const RDF_TYPE = RDFS.GetResource ('http://flock.com/rdf#CoopType');

/*
var coop_logger = Components.classes['@flock.com/logger;1'].createInstance(Components.interfaces.flockILogger);
coop_logger.init("coop");
*/
function CoopType (aORM, aType, aFields, aFieldTypes, aFieldDefaults, aFieldMultiple, aContainer, aContainerName, aTightBinding) {
  this.mORM = aORM;
  this.mType = aType;
  this.mTypeResource = RDFS.GetResource (aType);
  this.fields = aFields;
  this.field_types = aFieldTypes;
  this.field_defaults = aFieldDefaults;
  this.field_multiple = aFieldMultiple;
  this.container = aContainer;
  this.container_name = aContainerName;
  this.tight_binding = aTightBinding;
  this.baseClasses = [];
  this.destroyNotifiers = [];

  var ormtype = this;

  function init_instance (type, instance, uri, field_values, resource) {
    if (!uri) {
      var get_id = type.mORM.mTypes[type.mType].ctor.get_id;
      if (get_id && field_values) {
        uri = get_id(field_values);
      }

      if (!uri) {
        uri = RDFS.GetAnonymousResource ().ValueUTF8;
      }
    }
    if (!resource)
      resource = RDFS.GetResource (uri);

    if (type.container) {
      var container = null;

      // initialise the container
      if (type.container &&
          !RDFCU.IsContainer (type.mORM.datasource, resource)) {
        switch (type.container) {
          case 'Seq':
            RDFCU.MakeSeq (type.mORM.datasource, resource);
            break;
          case 'Bag':
            RDFCU.MakeBag (type.mORM.datasource, resource);
            break;
          case 'Alt':
            RDFCU.MakeAlt (type.mORM.datasource, resource);
            break;
          default:
            throw 'invalid container type';
            break;
        }
      }

      // provide an API
      function container_enumerator () { }
      container_enumerator.init_container = function() {
        container = Components.classes['@mozilla.org/rdf/container;1']
          .createInstance(flockIRDFContainer);
        container.Init (type.mORM.datasource, resource);
      }
      container_enumerator.count = function() {
        if (!container) this.init_container();
        return container.GetCount();
      }
      container_enumerator.add = function container_enum_add(child) {
        if (!container) this.init_container();
        if (instance.onBeforeAdd) instance.onBeforeAdd(child);
        try {
          container.AppendElement (/* RDFS.GetResource (child.__Coop_uri) */ child.__Coop_resource);
        } catch (ex) {  
          dump("Coop: "+ex+"\n");
        }
        if (instance.onAdd) instance.onAdd(child);
      }
      container_enumerator.insertAt = function container_enum_insertAt(child, position) {
        if (!container) this.init_container();
        //  /!\ the first element is 1, not 0 /!\
        try {
          // coop_logger.info("Insert "+child.__Coop_uri+" at position "+position);
          container.InsertElementAt( /* RDFS.GetResource (child.__Coop_uri) */ child.__Coop_resource, position, true);
        } catch(ex) {
          // coop_logger.error(ex);
          dump("Coop: "+ex+"\n");
        }
      }
      container_enumerator.remove = function container_enum_remove(child) {
        if (!container) this.init_container();
        if (instance.onBeforeRemove) instance.onBeforeRemove();
        container.RemoveElement (/* RDFS.GetResource (child.__Coop_uri) */ child.__Coop_resource, false);
        if (instance.onRemove) instance.onRemove();
      }
      container_enumerator.enumerate = function container_enum_enumerate() {
        if (!container) this.init_container();
        var enumerator = {
          hasMoreElements: function container_enum_hasMoreElements() {
            return this.elements.hasMoreElements ();
          },
          getNext: function container_enum_getNext() {
            var el = this.elements.getNext ();
            el.QueryInterface (nsIRDFResource);
            return type.mORM.get_from_resource (el);
          }
        }
        enumerator.elements = container.GetElements ();
        return enumerator;
      }
      container_enumerator.enumerateBackwards = function container_enum_enumBackwards() {
        if (!container) this.init_container();
        var enumerator = {
          hasMoreElements: function container_backEnum_hasMoreElements() {
            return this.elements.hasMoreElements ();
          },
          getNext: function container_backEnum_getNext() {
            var el = this.elements.getNext ();
            el.QueryInterface (nsIRDFResource);
            return type.mORM.get_from_resource (el);
          }
        }
        enumerator.elements = container.GetElementsBackwards();
        return enumerator;
      }
      container_enumerator.indexOf = function container_enum_indexOf(child) {
        if (!container) this.init_container();
        return container.IndexOf (/* RDFS.GetResource (child.__Coop_uri) */ child.__Coop_resource);
      }
      container_enumerator.has = function container_enum_has(child) {
        if (!container) this.init_container();
        return container.IndexOf (/* RDFS.GetResource (child.__Coop_uri) */ child.__Coop_resource) >= 0;
      }
      container_enumerator.addOnce = function container_enum_addOnce(child) {
        if (!container) this.init_container();
      	if (this.indexOf(child) < 0) this.add(child);
      }
      container_enumerator.empty = function container_empty() {
        var myenum = this.enumerateBackwards();
        while (myenum.hasMoreElements()) {
          var child = myenum.getNext();
          this.remove(child);
        }
      }

      instance[type.container_name] = container_enumerator;
    }

    // set up the fields

    function set_up_field (f) {
      if (type.field_multiple[f]) {
        var field_type = type.field_types[f] || Coop.string;
        var field_rsrc = type.fields[f];
        var field_enumerator = function field_enum_ctor() { }
        if (!type.read_only) {
          field_enumerator.add = function field_enum_add(child) {
            type.mORM.datasource.Assert (resource, field_rsrc, 
                field_type.toRdf (child, type.mORM), true);
          }
          field_enumerator.remove = function field_enum_remove(child) {
            type.mORM.datasource.Unassert (resource, field_rsrc, 
                field_type.toRdf (child, type.mORM));
          }
          field_enumerator.addOnce = function field_enum_addOnce(child) {
             if (!type.mORM.datasource.HasAssertion( resource, field_rsrc,
                 field_type.toRdf(child, type.mORM), true)) {
                  this.add(child); 
                 } 
          }
          field_enumerator.has = function field_enum_has(child) {
            return type.mORM.datasource.HasAssertion( resource, field_rsrc,
              field_type.toRdf(child, type.mORM), true);
          }
          field_enumerator.removeAll = function field_enum_removeAll() {
            var enum_ = this.enumerate();
            while (enum_.hasMoreElements()) {
              var child = enum_.getNext();
              this.remove(child);
            }
          }
        }
        field_enumerator.enumerate = function field_enum_enumerate() {
          var elements = type.mORM.datasource.GetTargets (resource, 
              field_rsrc, true);
          var enumerator = {
            hasMoreElements: function field_enum_hasMoreElements() {
              return elements.hasMoreElements ();
            },
            getNext: function field_enum_getNext() {
              var el = elements.getNext ();
              return field_type.toJs (el, type.mORM);
            }
          }
          return enumerator;
        }
        instance.__defineGetter__ (f, function () { return field_enumerator; });
        instance.__defineSetter__ (f, function (v) { 
            throw "can't set multiple property: "+f+' to: '+v; 
        });
        // FIXME: handle read-only
      } else {
        if (type.tight_binding) {
          if (!type.read_only) {
            instance.__defineSetter__ (f, function instance_field_setter (v) {
              type.put_field (resource, f, v);
            });
          } else {
            instance.__defineSetter__ (f, function instance_field_setter () {
              throw "can't set property on read-only object";
            });
          }
          instance.__defineGetter__ (f, function instance_field_getter() {
            return type.get_field (resource, f);
          });
        }
      }
    }
    
    
    for (var f in type.fields) {
      set_up_field (f);
    }

    // JMC - Assert the CoopType first, so that observers responding to the other
    // types can perform a coop "get"
    var firstTime = false;
    var type_target = type.mORM.datasource.GetTarget (resource, RDF_TYPE, true);
    if (!type_target) {
       type.mORM.datasource.Assert (resource, RDF_TYPE, type.mTypeResource, true);
       firstTime = true;
    }

    if (!field_values) field_values = {};
    var f;
    for (f in type.fields) {
      if (field_values[f] && type.field_multiple[f]) {
        // FIXME: initializer for a multiple field? eh?
        throw 'multiple field initializer not supported yet';
      }
      if (field_values[f] == null) {
        // Make sure default value ends up in here
        if (!type_target && type.field_defaults[f] != null
            && instance[f] == null) {
          instance[f] = type.field_defaults[f];
        }
      } else {
        instance[f] = field_values[f];
      }
    }

    if (!type.tight_binding) {
      if (!type.read_only) {
        instance.save = function coop_obj_save() { if (instance.onBeforeSave) instance.onBeforeSave(); type.put (uri, instance); if (instance.onSave) instance.onSave(); }
      }
      instance.reload = function coop_obj_reload() { type.get (uri, instance); }
      // FIXME: add a "changed" function
      // FIXME: keep a dirty flag?
    } else {
       instance.save = function coop_obj_save() { } // dump ("Tight Bound coop needs no save\n"); } 
    }

    if (!type.read_only) {
      instance.destroy = function coop_obj_destroy() { type.destroy (uri, instance); }
    }

    instance.isA = function coop_obj_isA(aCoopType) { return type.isA (aCoopType);  }
    instance.isInstanceOf = function coop_obj_isInstanceOf(aCoopType) { return type.isInstanceOf (aCoopType); }
    // fixme: shouldn't this just be a read-only field?
    instance.id = function coop_obj_id() { return uri; }
    instance.resource = function coop_obj_resource() { return resource; }
    instance.getParent = function coop_obj_getParent() {
      var arcs = type.mORM.datasource.ArcLabelsIn(resource);
      while (arcs && arcs.hasMoreElements()) {
        var testresource = arcs.getNext();
        if (testresource && testresource.QueryInterface)
          testresource =  testresource.QueryInterface(nsIRDFResource);
        if (testresource && testresource.QueryInterface && RDFCU.IsOrdinalProperty(testresource)) {
          var source = type.mORM.datasource.GetSource(testresource, resource, true).QueryInterface(nsIRDFResource);
          var finalresource = type.mORM.get_from_resource(source);
          return finalresource;
        }
      }
      return null;
    }
    instance.getParents = function coop_obj_getParents() {
      var result = [];
      var arcs = type.mORM.datasource.ArcLabelsIn(resource);
      while (arcs && arcs.hasMoreElements()) {
        var testresource = arcs.getNext();
        if (testresource && testresource.QueryInterface)
          testresource =  testresource.QueryInterface(nsIRDFResource);
        if (testresource && testresource.QueryInterface && RDFCU.IsOrdinalProperty(testresource)) {
          var sources = type.mORM.datasource.GetSources(testresource, resource, true);
          while (sources && sources.hasMoreElements()) {
            var source = sources.getNext().QueryInterface(nsIRDFResource);
            var finalresource = type.mORM.get_from_resource(source);
            result.push(finalresource);
          }
        }
      }
      return result;
    }

    instance.__Coop_uri = uri; // FIXME can't we use a closure & id()
    instance.__Coop_resource = resource; // JMC - Is this bad, will we leak?
    if (firstTime && instance.onCreate) instance.onCreate();
  }

// JMC - Optimize for ctor getting a resource, instead of just an uri
  this.ctor = function coop_obj_ctor(arg1, arg2, arg3) {
    // FIXME: allow overrides at construct time - the prime cases are for things like tight_binding
    var uri, fields;
    
    if (arg3) {
      uri = arg1;
      init_instance (ormtype, this, uri, fields, arg3);
      return;
    }

    if (arg2) {
      uri = arg1;
      fields = arg2;
    } else {
      if (typeof (arg1) == 'string') {
        uri = arg1;
      } else {
        fields = arg1;
        uri = null;
      }
    }

    // FIXME: for readonly - don't allow initial fields

    init_instance (ormtype, this, uri, fields, arg3);
  }

  this.ctor.all_ids = function all_ids () {
    // FIXME: deal with inheritance
    var sources = ormtype.mORM.datasource.GetSources (RDF_TYPE, ormtype.mTypeResource, true);
    var results = [];
    while (sources.hasMoreElements ()) {
      var source = sources.getNext ().QueryInterface (nsIRDFResource);
      results.push (source);
    }
    return {
      hasMoreElements: function coop_obj_allIds_hasMoreElements() { return results.length > 0; },
      getNext: function coop_obj_allIds_getNext() { return results.shift (); }
    };
  }

  this.ctor.all = function all () {
    // FIXME: deal with inheritance
    var ids = ormtype.ctor.all_ids ();

    return {
      hasMoreElements: function coop_obj_all_hasMoreElements() { return ids.hasMoreElements (); },
      getNext: function coop_obj_all_getNext() { return ormtype.mORM.get_from_resource (ids.getNext ()); }
    };
  }

  this.ctor.find = function find (aQuery) {
    // FIXME: special queries for rdfsql - faster pussycat, search! search!
    // FIXME: we need to be able to order the keys to match in a query

    var realResults = [];
    var firstprop = true;
    for (var propname in aQuery) {
      var results = [];
        // coop_logger.info("Coop Find: looking at property" + propname + " for value of " + aQuery[propname]);
      var type = ormtype.field_types[propname] || Coop.string;
      var sources = ormtype.mORM.datasource.GetSources (ormtype.fields[propname],
        type.toRdf (aQuery[propname], ormtype.mORM), true);
      while (sources.hasMoreElements ()) {
        var source = sources.getNext ().QueryInterface (nsIRDFResource);
        if (!ormtype.mORM.datasource.HasAssertion (source, RDF_TYPE, 
              ormtype.mTypeResource, true)) {
          continue;
        }
        results.push (source.ValueUTF8);
      }
      if (firstprop) {
        realResults = results;
        firstprop = false;
      }
      // JMC - Loop backwards through the realResults, 
      // if they don't exist in results, then pop them
      var interSection = [];
      for (var i=0 ; i < realResults.length; i++)
      {
        if (results.indexOf(realResults[i]) > -1)
        {
          interSection.push(realResults[i]);
        }
      }
      realResults = interSection;
    }
    var finalResults = [];
    for (var i =0; i<realResults.length; i++)
    {
      finalResults.push(new ormtype.ctor (realResults[i]));
    }
    return finalResults;
  }

  this.ctor.exists = function exists (aURN) {
    return ormtype.mORM.datasource.HasAssertion (RDFS.GetResource (aURN), 
        RDF_TYPE, ormtype.mTypeResource, true);
  }

  this.ctor.add_destroy_notifier = function add_destroy_notifier (aNotifyFunc) {
    ormtype.add_destroy_notifier(aNotifyFunc);
  }

  this.ctor.remove_destroy_notifier = function remove_destroy_notifier (aNotifyFunc) {
    ormtype.remove_destroy_notifier(aNotifyFunc);
  }

  this.ctor.mType = this.mType; // JMC - need this later
}
CoopType.prototype.put_field = function CoopType_put_field (aResource, aFieldName, aFieldValue) {
  // find the data type - explicit or string by default
  var type = this.field_types[aFieldName] || Coop.string;

  // does this property already exist?
  var old_target = this.mORM.datasource.GetTarget (aResource, this.fields[aFieldName],
      true);
  if (old_target != null) {
    if (aFieldValue == null) {
      this.mORM.datasource.Unassert (aResource, this.fields[aFieldName], old_target);
      return true;
    } else {
      if (old_target != type.toRdf (aFieldValue, this.mORM)) {
        this.mORM.datasource.Change (aResource, this.fields[aFieldName], 
            old_target, type.toRdf (aFieldValue, this.mORM));
        return true;
      } else {
        return false;
      }
    }
  } else {
    if (aFieldValue != null) {
      this.mORM.datasource.Assert (aResource, this.fields[aFieldName], 
          type.toRdf (aFieldValue, this.mORM), true);
      return true;
    } else {
      return false;
    }
  }
}
CoopType.prototype.put = function CoopType_put (aURI, aObject, aProperties) {
  
  // coop_logger.debug('CoopType.put ('+aURI+')');
  // FIXME: have a way of clearing properties
  var r = RDFS.GetResource (aURI);

  // check and set type
  var type_target = this.mORM.datasource.GetTarget (r, RDF_TYPE, true);
  if (type_target) {
    if (type_target.ValueUTF8 != this.mTypeResource.ValueUTF8) {
      // throw "incorrect type";
    }
    // else the type is right
  } else {
    // add the type
    this.mORM.datasource.Assert (r, RDF_TYPE, this.mTypeResource, true);
  }

  // keep track of what properties changed
  var changed = {};

  // for each field
  for (var propname in this.fields) {
    // check if this field is being included
    if (aProperties != null && aProperties[propname] == null) {
      // a list of properties was specified and this one wasn't included
      continue;
    }

    // check if its a multiple field - that's handled directly
    if (this.field_multiple[propname]) {
      continue;
    }

    // find the right value
    var value = aObject[propname];
    if (value == null) {
      // if its not defined, use a default
      value = this.field_defaults[propname];
    }

    // so, do we have a value to go ahead with?
    if (value == null) {
      // apparently not...
      continue;
    }
    changed[propname] = this.put_field (r, propname, value);
  }
}
CoopType.prototype.get_field = function CoopType_get_field (aResource, aFieldName) {
  // try to get a value from RDF
  var target = this.mORM.datasource.GetTarget (aResource, this.fields[aFieldName], 
      true);
  if (target == null) {
    return null;
  } else {
    // find the data type - explicit or string by default
    var type = this.field_types[aFieldName] || Coop.string;
    return type.toJs (target, this.mORM);
  }
}
CoopType.prototype.get = function CoopType_get (aURI, aObject, aProperties) {
  // coop_logger.debug('CoopType.get ('+aURI+')');
  // dump('CoopType.get ('+aURI+')\n');
  var r = RDFS.GetResource (aURI);

  // check type
  try {
  var type_target = this.mORM.datasource.GetTarget (r, RDF_TYPE, true);
  } catch (ex) {
   debugger; 
  }
  if (type_target && type_target.ValueUTF8 != this.mTypeResource.ValueUTF8) {
    //coop_logger.error('incorrect type');
    dump('Coop: incorrect type\n');
    // throw "incorrect type";
  }
 
  var object = aObject || {};

  // for each field
  for (var propname in this.fields) {
    // check if this field is being included
    if (aProperties != null && !aProperties[propname]) {
      // a list of properties was specified and this one wasn't included
      continue;
    }

    // check if its a multiple field - that's handled directly
    if (this.field_multiple[propname]) {
      continue;
    }

    var value = this.get_field (r, propname);

    if (value != null) {
      object[propname] = value;
    } else {
      if (object[propname] == null &&
          this.field_defaults[propname] != null) {
        object[propname] = this.field_defaults[propname];
      }
    }
  }

  return object;
}

CoopType.prototype.add_destroy_notifier = function CoopType_add_destroy_notifier (aNotifyFunc) {
  function hasFunc(element, index, array) {
    return element == aNotifyFunc;
  }
  if (!this.destroyNotifiers.some(hasFunc))
    this.destroyNotifiers.push(aNotifyFunc);
}

CoopType.prototype.remove_destroy_notifier = function CoopType_remove_destroy_notifier (aNotifyFunc) {
  function keepFunc(element, index, array) {
    return element != aNotifyFunc;
  }
  this.destroyNotifiers = this.destroyNotifiers.filter(keepFunc);
}

CoopType.prototype.destroy = function CoopType_destroy (aURI, aInstance) {
  for each (var func in this.destroyNotifiers) {
    func(aInstance, this.mORM);
  }

  if (aInstance.children) {
    aInstance.children.empty();
  }
  
  for each (var parent in aInstance.getParents()) {
    if (parent.children)
      parent.children.remove(aInstance);
  }

  var r = RDFS.GetResource (aURI);

  // remove the type first, so other coop users can't get at the object anymore
  this.mORM.datasource.Unassert (r, RDF_TYPE, this.mTypeResource);

  var arcs = this.mORM.datasource.ArcLabelsIn (r);
  while (arcs.hasMoreElements()) {
    var arc = arcs.getNext().QueryInterface(nsIRDFResource); 
    var sources = this.mORM.datasource.GetSources( arc, r, true);
    while (sources.hasMoreElements()) {
      var source = sources.getNext().QueryInterface(nsIRDFResource);
      this.mORM.datasource.Unassert (source, arc, r);
    }
  }

  arcs = this.mORM.datasource.ArcLabelsOut(r);
  while (arcs.hasMoreElements()) {
    var arc = arcs.getNext().QueryInterface(nsIRDFResource); 
    var targets = this.mORM.datasource.GetTargets(r, arc, true);
    while (targets.hasMoreElements()) {
      var target = targets.getNext().QueryInterface(nsIRDFNode);
      this.mORM.datasource.Unassert (r, arc, target);
    }
  }
  
  return;
}

// aCoopType is string containing the urn of the type, 
// OR an object that is the ctor of the CoopType
CoopType.prototype.isA = function CoopType_isA (aCoopType) {
 if (aCoopType == this.mType) return true; 
 if (aCoopType == this.ctor) return true;
 return false; 
}

CoopType.prototype.isInstanceOf = function CoopType_isInstanceOf (aCoopType) {
  if (this.isA(aCoopType)) return true;
  if (this.baseClasses.indexOf(aCoopType) > -1) return true;
  if (aCoopType.mType && this.baseClasses.indexOf(aCoopType.mType) > -1) return true; 
  return false;
}

function Coop (aDataSource, aSchema, aDefaultProperties) {
  this.datasource = aDataSource;
  this.mTypes = {};
  if (aDefaultProperties) {
      this.mDefaultType = new CoopType (this, 'urn:coop:internal:default', 
          aDefaultProperties.fields, aDefaultProperties.field_types, 
          aDefaultProperties.field_defaults, aDefaultProperties.field_multiple, 
          aDefaultProperties.container, aDefaultProperties.container_name, 
          aDefaultProperties.tight_binding);
  } else {
    this.mDefaultType = null;
  }

  if (aSchema) {
    var loader = Components.classes['@mozilla.org/moz/jssubscript-loader;1']
        .getService(Components.interfaces.mozIJSSubScriptLoader);
    var context = {Coop: Coop, coop: this};
    loader.loadSubScript(aSchema, context);
  }
}
Coop.prototype.add_type = function Coop_add_type (aType, aProperties) {
  var inherits_from = aProperties['inherits_from'];
  if (inherits_from) {
    inherits_from = this.mTypes[inherits_from];
  } else if (this.mDefaultType) {
    inherits_from = this.mDefaultType;
  }

  var k;

  var fields = {};
  if (inherits_from) {
    for (k in inherits_from.fields) {
      fields[k] = inherits_from.fields[k];
    }
  }
  if (aProperties.fields) {
    for (k in aProperties.fields) {
      fields[k] = RDFS.GetResource (aProperties.fields[k]);
    }
  }

  var field_types = {};
  if (inherits_from) {
    for (k in inherits_from.field_types) {
      field_types[k] = inherits_from.field_types[k];
    }
  }
  if (aProperties.field_types) {
    for (k in aProperties.field_types) {
      field_types[k] = aProperties.field_types[k];
    }
  }

  var field_defaults = {};
  if (inherits_from) {
    for (k in inherits_from.field_defaults) {
      field_defaults[k] = inherits_from.field_defaults[k];
    }
  }
  if (aProperties.field_defaults) {
    for (k in aProperties.field_defaults) {
      field_defaults[k] = aProperties.field_defaults[k];
    }
  }

  var field_multiple = {};
  if (inherits_from) {
    for (k in inherits_from.field_multiple) {
      field_multiple[k] = inherits_from.field_multiple[k];
    }
  }
  if (aProperties.field_multiple) {
    for (k in aProperties.field_multiple) {
      field_multiple[k] = aProperties.field_multiple[k];
    }
  }

  var container = aProperties.container || 
      (inherits_from && inherits_from.container);

  var container_name = null;
  if (aProperties.container_name) {
    container_name = aProperties.container_name;
  } else if (inherits_from && inherits_from.container_name) {
    container_name = inherits_from.container_name;
  } else {
    container_name = 'children';
  }

  var tight_binding = false;
  if (aProperties.tight_binding != null) {
    tight_binding = aProperties.tight_binding;
  } else {
    if (inherits_from && inherits_from.tight_binding != null) {
      tight_binding = inherits_from.tight_binding;
    }
  }

/*
  var constructor = null;
  if (aProperties.constructor) {
    constructor = aProperties.constructor;
  } else if (inherits_from && inherits_from.constructor) {
    constructor = inherits_from.constructor;
  }
  */

  this.mTypes[aType] = new CoopType (this, aType, fields, 
      field_types, field_defaults, field_multiple, 
      container, container_name, tight_binding);
  
  // JMC - Move this into the ctor with everything else
  if (aProperties['inherits_from']) {
    this.mTypes[aType].baseClasses.push(aProperties['inherits_from']);
    for (basetype in inherits_from.baseClasses) {
      this.mTypes[aType].baseClasses.push(inherits_from.baseClasses[basetype]);
    }
  }
  
  return this.mTypes[aType].ctor;
}
Coop.prototype.get_from_resource = function Coop_get_from_resource (aResource) {
  var type_target = this.datasource.GetTarget (aResource, RDF_TYPE, true);
  // coop_logger.debug('get_from_resource: aResource = '+aResource.ValueUTF8);
  if (!type_target) return null;
  // coop_logger.debug('get_from_resource: type_target = '+type_target.ValueUTF8);
  aResource.QueryInterface(Components.interfaces.nsIRDFResource);
  return new this.mTypes[type_target.ValueUTF8].ctor (aResource.ValueUTF8, null, aResource);
}
Coop.prototype.get = function Coop_get (aURI) {
  try{
    if (!aURI) {
      return null;
    }
    return this.get_from_resource (RDFS.GetResource (aURI));
  } catch (ex) {
    dump ("COOP Died trying to get " + aURI + "\n" + ex + "\n");
    return null;
  }
}
Coop.prototype.beginRecoverableUpdates = function Coop_beginRecoverableUpdates () {
  try {
    this.datasource.QueryInterface(flockIRDFService);
    this.datasource.beginRecoverableUpdates();
  } catch (ex) {
  }
}
Coop.prototype.endRecoverableUpdates = function Coop_endRecoverableUpdates () {
  try {
    this.datasource.QueryInterface(flockIRDFService);
    this.datasource.endRecoverableUpdates();
  } catch (ex) {
  }
}


// standard types
INVALID_XML_CHARACTERS = /[^\u0009\u000A\u000D\u0020-\uD7FF\uE000-\uFFFD]/g;
Coop.string = {
  toJs: function string_toJS(node, orm) {
    return node.QueryInterface (nsIRDFLiteral).Value; },
  toRdf: function string_toRdf(value, orm) { 
    try  {
      if (value == null) value = "";
      value = value.toString (); // force it to a string
      return RDFS.GetLiteral (value.replace (INVALID_XML_CHARACTERS, '')); 
    } catch (ex) {
      // coop_logger.error(ex);
      dump("Coop: "+ex+"\n");
      return null;
    }
  }
};
Coop.resource = {
  toJs: function resource_toJS(node, orm) {
    return node.QueryInterface (nsIRDFResource).ValueUTF8; },
  toRdf: function resource_toRdf(value, orm) { 
    return RDFS.GetResource (value); }
};
Coop.date = {
  toJs: function date_toJs(node, orm) {
    return new Date (node.QueryInterface (nsIRDFDate).Value / 1000); },
  // FIXME: I should not need this 1000* here, but for some reason I get the wrong date
  // without it
  toRdf: function date_toRdf(value, orm) { return RDFS.GetDateLiteral (1000 * value.getTime ()); }
};
Coop.boolean = {
  toJs: function boolean_toJs(node, orm) {
    return node.QueryInterface (nsIRDFLiteral).Value=="true"; },
  toRdf: function boolean_toRdf(value, orm) { 
    return RDFS.GetLiteral (value?"true":"false"); }
};
Coop.integer = {
  toJs: function integer_toJs(node, orm) {
    return node.QueryInterface (nsIRDFInt).Value; },
  toRdf: function integer_toRdf(value, orm) { 
    return RDFS.GetIntLiteral (value); }
}
Coop.stringinteger = {
  toJs: function stringinteger_toJs(node, orm) {
    return parseInt(node.QueryInterface (nsIRDFLiteral).Value); },
  toRdf: function stringinteger_toRdf(value, orm) { 
    return RDFS.GetLiteral (value); }
}
Coop.object = {
  toJs: function object_toJs(node, orm) {
    return orm.get_from_resource (node.QueryInterface (nsIRDFResource)); },
  toRdf: function object_toRdf(value, orm) { 
    return /* RDFS.GetResource (value.__Coop_uri) */ value.__Coop_resource; }
}
// TODO: implement compare and toString
