// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2007
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

// the default object definitions for the Favorites datastore
// passed into this environment are Coop (the type) and coop (the instance)

const FLOCK_NS = 'http://flock.com/rdf#';
const NSCP_NS = 'http://home.netscape.com/NC-rdf#';
const RDF_NS = 'http://www.w3.org/1999/02/22-rdf-syntax-ns#';
const WEB_NS = 'http://home.netscape.com/WEB-rdf#';

//////////////////////////////////////////////////////////
// Common Stuff 
//////////////////////////////////////////////////////////

coop.Object = coop.add_type (FLOCK_NS+'Object', {
  fields: {
    name: NSCP_NS+'Name',
    flockType: FLOCK_NS+'flockType',
    rdftype: RDF_NS + 'type',
    isTransient: FLOCK_NS+'isTransient'
  },
  field_types: {
    rdftype: Coop.resource, 
    isTransient: Coop.boolean
  },    
  field_defaults: {
    rdftype: FLOCK_NS+'Object',
    name: ' '
  },
  tight_binding: true,
  target_DS: "data"
});

// A virtual class for common attributes - don't instanciate it!
coop.FlockObject = coop.add_type (FLOCK_NS+'FlockObject', {
  inherits_from: FLOCK_NS+'Object',
  fields: {
    favicon: FLOCK_NS+'favicon',
    isIndexable: FLOCK_NS+'isIndexable'
  },
  field_types: {
    isIndexable: Coop.boolean,
    rdftype: Coop.resource
  },
  field_defaults: {
  }
});

coop.CountedObject = coop.add_type(FLOCK_NS+'CountedObject', {
  inherits_from: FLOCK_NS+'FlockObject',
  fields: {
    unseenItems: FLOCK_NS+'unseenItems',
    count: FLOCK_NS+'count',
    maxItems: FLOCK_NS+'maxItems',
    capItems: FLOCK_NS+'capItems',
    lastItemDate: FLOCK_NS+'lastItemDate',
    hasUnseenItems: FLOCK_NS+'hasUnseenItems'
  },
  field_types: {
    unseenItems: Coop.stringinteger,
    count: Coop.stringinteger,
    maxItems: Coop.stringinteger,
    capItems: Coop.boolean,
    lastItemDate: Coop.date,
    hasUnseenItems: Coop.boolean
  },
  field_defaults: {
    unseenItems: 0,
    isIndexable: false,
    count: 0,
    maxItems: 0,
    capItems: false,
    lastItemDate: new Date(0),
    flockType: 'CountedObject',
    rdftype: FLOCK_NS+'CountedObject',
    hasUnseenItems: false
  }
});

coop.Folder = coop.add_type (FLOCK_NS+'Folder', {
  inherits_from: FLOCK_NS+'CountedObject',
  fields: {
    description: NSCP_NS+'Description'
  },
  field_defaults: {
    flockType: 'folder',
    rdftype: FLOCK_NS+'Folder',
    isIndexable: false
  },
  container: 'Seq'
});

coop.UncountedFolder = coop.add_type (FLOCK_NS+'UncountedFolder', {
  inherits_from: FLOCK_NS+'FlockObject',
  field_defaults: {
    flockType: 'folder',
    rdftype: FLOCK_NS+'Folder'
  },
  container: 'Seq'
});

coop.Pollable = coop.add_type (FLOCK_NS+'Pollable', {
  inherits_from: FLOCK_NS+'CountedObject',
  fields: {
    isPollable: FLOCK_NS+'isPollable',
    lastRefresh: FLOCK_NS+'lastRefresh',
    nextRefresh: FLOCK_NS+'nextRefresh',
    refreshInterval: FLOCK_NS+'refreshInterval'
  },
  field_types: {
    isPollable: Coop.boolean,
    lastRefresh: Coop.date,
    nextRefresh: Coop.date,
    refreshInterval: Coop.integer
  },
  field_defaults: {
    rdftype: FLOCK_NS+'Pollable'
  }
});

coop.WebObject = coop.add_type (FLOCK_NS+'WebObject', {
  inherits_from: FLOCK_NS+'FlockObject',
  fields: {
      URL: NSCP_NS+'URL'
  },
  field_defaults: {
      URL: 'about:blank'
  }
});

//@line 310 "/home/build/tinderbuild_blue/src/flock/mozilla/browser/components/flock/common/content/favorites-coop.js"

coop.Account = coop.add_type (FLOCK_NS+'Account', {
  inherits_from: FLOCK_NS+'Pollable',
  fields: {
    URL: NSCP_NS+'URL',
    service: FLOCK_NS+'service',        
    serviceId: FLOCK_NS+'serviceId',
    accountId: FLOCK_NS+'accountId',
    favicon: FLOCK_NS+'favicon',
    avatar: FLOCK_NS+'avatar',
    status: FLOCK_NS+'status',
    statusMessage: FLOCK_NS+'statusMessage',
    enabledAction: FLOCK_NS+'enabledAction',
    disabledAction: FLOCK_NS+'disabledAction',
    defaultActions: FLOCK_NS+'defaultActions',
    hasNew: FLOCK_NS+'hasNew',
    isAuthenticated: FLOCK_NS+'isAuthenticated',
    authToken: FLOCK_NS+'authToken',
    showInSidebar: FLOCK_NS+'showInSidebar',
    hasUnclearedNotifications: FLOCK_NS+'hasUnclearedNotifications',
    flickr_nsid: FLOCK_NS+'flickr_nsid',
    flickr_id: FLOCK_NS+'flickr_id',
    flickr_username: FLOCK_NS+'flickr_username',
    flickr_fullname: FLOCK_NS+'flickr_fullname'
  },
  field_types: {
    service: Coop.object,
    enabledAction: Coop.object,
    disabledAction: Coop.object,
    defaultActions: Coop.object,
    hasNew: Coop.boolean,
    showInSidebar: Coop.boolean,
    isAuthenticated: Coop.boolean,
    hasUnclearedNotifications: Coop.boolean
  },
  field_defaults: {
    flockType: 'Account',
    rdftype: FLOCK_NS+'Account',
    isAuthenticated: false,
    hasUnclearedNotifications: false
  },
  field_multiple: {
    enabledAction: true,
    disabledAction: true,
    account: true
  },
  container: 'Seq'
});


coop.Defaults = coop.add_type(FLOCK_NS+'Defaults', {
  fields: {
  },
  container: 'Seq'
});


//////////////////////////////////////////////////////////
// Favorites
//////////////////////////////////////////////////////////

coop.Favorite = coop.add_type (FLOCK_NS+'Favorite', {
  inherits_from: FLOCK_NS+'WebObject',
  fields: {
    unseen: FLOCK_NS+'unseen',
    service: FLOCK_NS+'service',
    thumbnail: FLOCK_NS+'thumbnail',
    datevalue: FLOCK_NS+'datevalue',
    description: NSCP_NS+'Description',
    LastCharset: WEB_NS+'LastCharset'
  },
  field_types: {
    unseen: Coop.boolean,
    service: Coop.object,
    datevalue: Coop.date
  },
  field_defaults: {
    flockType: "bookmark",
    rdftype: FLOCK_NS+'Favorite',
    isIndexable: true
  },
  field_multiple: {
    service: true
  },
  target_DS: 'lotsofaves'
});

// ERW: tag (multiple) is duplicated from tags. I'm just trying this model with online bookmarks to make sure it works well, and if it appears to be good I will extend it to local bookmarks too. The goal is to have fast tag->bookmark queries.
coop.Bookmark = coop.add_type (FLOCK_NS+'Bookmark', {
  inherits_from: FLOCK_NS+'Favorite',
  fields: {
    private: FLOCK_NS+'private',
    tags: FLOCK_NS+'tags',
    tag:  FLOCK_NS+'tag', 
    ShortcutURL:  NSCP_NS+'ShortcutURL',
    LastModifiedDate: WEB_NS +'LastModifiedDate',
    LastVisitDate: WEB_NS +'LastVisitDate'
  },
  field_types: {
    private: Coop.boolean
  },
  field_multiple: {
    tag: true
  },
  field_defaults: {
    name: '',
    description: '',
    tags: ''
  }
});

coop.Bookmark.get_id = function Bookmark_get_id(obj) {
  if (obj.URL)
    return obj.URL;
  return null;
}

coop.Separator = coop.add_type (NSCP_NS+'BookmarkSeparator', {
  inherits_from: FLOCK_NS+'Object',
  field_defaults: {
    rdftype: NSCP_NS+'BookmarkSeparator'
  }
});

coop.Notification = coop.add_type (FLOCK_NS+'Notification', {
  inherits_from: FLOCK_NS+'WebObject',
  fields: {
    unseen: FLOCK_NS+'unseen',
    service: FLOCK_NS+'service',
    datevalue: FLOCK_NS+'datevalue',
    type: FLOCK_NS+'type',
    description: FLOCK_NS+'description',
    title: FLOCK_NS+'title',
    onaccept: FLOCK_NS+'onaccept',
    oncancel: FLOCK_NS+'oncancel'
  },
  field_types: {
    unseen: Coop.boolean,
    service: Coop.object,
    datevalue: Coop.date
  },
  field_defaults: {
    unseen: true,
    isIndexable: false,
    isPollable: false,
    datevalue: new Date(0),
    flockType: "notification",
    rdftype: FLOCK_NS+'Favorite'
  }
});

coop.Stream = coop.add_type (FLOCK_NS+'Stream', {
  inherits_from: FLOCK_NS+'Pollable',
  fields: {
    URL: NSCP_NS+'URL',
    apiType: FLOCK_NS+'apiType',
    notify: FLOCK_NS+'notify',
    userid: FLOCK_NS+'userid',
    serviceId: FLOCK_NS+'serviceId',
    lastModification: FLOCK_NS+'lastModification',
    state: FLOCK_NS+'state'
  },
  field_types: {
    notify: Coop.boolean
  },
  field_defaults: {
    URL: 'about:blank',
    notify: false,
    isIndexable: true,
    isPollable: true,
    flockType: 'stream',
    rdftype: NSCP_NS+'Livemark',
    state: 'ok'
  },
  container: 'Seq',
  target_DS: 'feeding'
});

coop.Stream.prototype.addItem = function Stream_add (item) {
  if (this.onBeforeAddItem)
    this.onBeforeAddItem(item);

  if (this.children.indexOf(item) < 0) { // only add once
    // a sorted insert...
    var _enum = this.children.enumerateBackwards();
    var pos = -1; // pos is the position of the previous element
    while (_enum.hasMoreElements ()) {
      var cursor = _enum.getNext ();

      if (cursor.datevalue <= item.datevalue) {
        if (pos < 0)
          this.children.add (item);
        else
          this.children.insertAt (item, pos);

        if (this.onAddItem)
         this.onAddItem(item);

        return;
      }
      if (pos < 0)
        pos = this.children.indexOf(cursor);
      else
        pos--;
    }

    this.children.insertAt (item, 1);
  }

  if (this.onAddItem)
    this.onAddItem(item);
}

coop.OnlineBookmarksStream = coop.add_type (FLOCK_NS+'OnlineBookmarksStream', {
  inherits_from: FLOCK_NS+'Stream',
  fields: {
    last_update_time: FLOCK_NS+'LastUpdateTime', // last value from posts/update
    tag: FLOCK_NS+'tag'
  },
  field_types: {
    last_update_time: Coop.date
  },
  field_multiple: {
    tag: true
  },
  field_defaults: {
    last_update_time: new Date(0),
    rdftype: FLOCK_NS+'OnlineFolder'
  }
});

//////////////////////////////////////////////////////////
// Media
//////////////////////////////////////////////////////////

coop.Photo = coop.add_type (FLOCK_NS+'Photo', {
  inherits_from: FLOCK_NS+'Object',
  fields: {
    datevalue: FLOCK_NS+'datevalue',
    unseen: FLOCK_NS+'unseen',
    URL: NSCP_NS+'URL',
    midSizePhoto: FLOCK_NS+'midSizePhoto'
  },
  field_types: {
    datevalue: Coop.date,
    unseen: Coop.boolean
  },
  field_defaults: {
    datevalue: new Date(0),
    unseen: true
  }
});

coop.RichPhoto = coop.add_type (NSCP_NS+'RichPhoto', {
  inherits_from: FLOCK_NS+'Favorite',
  fields: {
    midSizePhoto: FLOCK_NS+'midSizePhoto',
    largeSizePhoto: FLOCK_NS+'largeSizePhoto',
    username: FLOCK_NS+'username',
    userid: FLOCK_NS+'userid',
    photoid: FLOCK_NS+'photoid',
    is_public: FLOCK_NS+'is_public',
    lastUpdate: FLOCK_NS+'lastUpdate',
    is_video: FLOCK_NS+'is_video'
  },
  field_types: {
    is_public: Coop.boolean,
    is_video: Coop.boolean,
    lastUpdate: Coop.date
  },
  field_defaults: {
    rdftype: NSCP_NS+'Favorite',
    is_video: false,
    lastRefresh: new Date (1),
    isIndexable: false
  }
});


coop.MediaQuery = coop.add_type (NSCP_NS+'MediaQuery', {
  inherits_from: FLOCK_NS+'Pollable',
  fields: {
    service: FLOCK_NS+'svc',
    query: FLOCK_NS+'query',
    favicon: FLOCK_NS+'favicon',
    serviceId: FLOCK_NS+'serviceId',
    latestSeq: FLOCK_NS+'latestSeq',
    firstRefresh: FLOCK_NS+'firstRefresh'
  },
  field_types: {
    latestSeq: Coop.integer,
    firstRefresh: Coop.boolean
  },
  field_defaults: {
    latestSeq: 0,
    isPollable: false,
    isIndexable: true,
    flockType: 'media',
    rdftype: NSCP_NS+'MediaQuery',
    serviceId: '@flock.com/photo-api-manager;1?',
    firstRefresh: true
  },
  container: 'Seq'
});


//////////////////////////////////////////////////////////
// Feeds
//////////////////////////////////////////////////////////

coop.Feed = coop.add_type (FLOCK_NS+'Feed', {
  inherits_from: FLOCK_NS+'Stream',
  fields: {
    link: FLOCK_NS+'link',
    format: FLOCK_NS+'format',
    subtitle: FLOCK_NS+'subtitle',
    image: FLOCK_NS+'image',
    author: FLOCK_NS+'author',
    datevalue: FLOCK_NS+'datevalue',
    finalURL: FLOCK_NS+'finalURL',
    lastFetch: FLOCK_NS+'lastFetch',
    lastFaviconFetch: FLOCK_NS+'lastFaviconFetch',
    metadataOnly: FLOCK_NS+'metadataOnly'
  },
  field_types: {
    datevalue: Coop.date,
    lastFetch: Coop.date,
    lastFaviconFetch: Coop.date,
    metadataOnly: Coop.boolean
  },
  field_defaults: {
    isIndexable: false,
    isPollable: false,
    flockType: 'feed',
    rdftype: NSCP_NS+'Livemark',
    capItems: true,
    lastFetch: new Date(0),
    lastFaviconFetch: new Date(0)
  },
  container: 'Seq'
});

coop.Feed.get_id = function Feed_get_id(obj) {
  if (obj.URL)
    return 'urn:flock:feed:' + obj.URL;
  return null;
}

coop.Feed.prototype.addItem = coop.Stream.prototype.addItem;

coop.FeedItem = coop.add_type (FLOCK_NS+'FeedItem', {
  inherits_from: FLOCK_NS+'Favorite',
  fields: {
    itemid: FLOCK_NS+'itemid',
    subtitle: FLOCK_NS+'subtitle',
    author: FLOCK_NS+'author',
    pubdate: FLOCK_NS+'pubdate',
    flagged: FLOCK_NS+'flagged'
  },
  field_types: {
    pubdate: Coop.date,
    flagged: Coop.boolean
  },
  field_defaults: {
    isPollable: false,
    isIndexable: false,
    flockType: 'feeditem',
    rdftype: FLOCK_NS+'Favorite'
  }
});

coop.FeedItem.get_id = function FeedItem_get_id(obj) {
  if (obj.itemid && obj.parentfeed)
    return 'urn:flock:feeditem:' + obj.parentfeed + ':' + obj.itemid;
  return null;
}

coop.FeedFolder = coop.add_type (FLOCK_NS+'FeedFolder', {
  inherits_from: FLOCK_NS+'Folder',
  field_defaults: {
    flockType: 'folder',
    isIndexable: false,
    rdftype: FLOCK_NS+'Folder'
  },
  container: 'Seq'
});

coop.FeedContext = coop.add_type (FLOCK_NS+'FeedContext', {
  inherits_from: FLOCK_NS+'FeedFolder',
  fields: {
    refreshInterval: FLOCK_NS+'refreshInterval',
    flaggedItems: FLOCK_NS+'flaggedItems'
  },
  field_types: {
    refreshInterval: Coop.integer,
    flaggedItems: Coop.object
  },
  field_defaults: {
    flockType: 'folder',
    isIndexable: false,
    rdftype: FLOCK_NS+'Folder',
    refreshInterval: 3600
  },
  container: 'Seq'
});

coop.FeedContext.get_id = function FeedContext_get_id(obj) {
  if (obj.name)
    return 'urn:flock:feedcontext:' + obj.name;
  return null;
}

coop.FeedFlaggedStream = coop.add_type (FLOCK_NS+'FeedFlaggedStream', {
  inherits_from: FLOCK_NS+'Stream',
  field_defaults: {
    isIndexable: false,
    isPollable: false,
    rdftype: NSCP_NS+'Livemark'
  },
  container: 'Seq'
});

coop.FeedFlaggedStream.get_id = function FeedFlaggedStream_get_id(obj) {
  if (obj.context) {
    obj.name = obj.context.name;
    return obj.context.id() + ':flagged';
  }
  return null;
}

coop.FeedFlaggedStream.prototype.addItem = coop.Stream.prototype.addItem;

//////////////////////////////////////////////////////////
// Blog
//////////////////////////////////////////////////////////

// A blog that belongs to the current user, e.g. for
// publication, NOT for consumption. See "Feed" for consumption.
coop.Blog = coop.add_type (FLOCK_NS+'Blog', {
  inherits_from: FLOCK_NS+'WebObject',
  fields: {
    favicon: FLOCK_NS+'favicon',
    title: FLOCK_NS+'title',
    api: FLOCK_NS+'api',
    supportCategories: FLOCK_NS+'supportCategories',
    apiLink: FLOCK_NS+'apiLink',
    blogid: FLOCK_NS+'blogid',
    username: FLOCK_NS+'username',
    authtoken: FLOCK_NS+'authtoken',
    pwhost: FLOCK_NS+'pwhost',
    categories: FLOCK_NS+'categories'
  },
  field_types: {
    supportCategories: Coop.boolean,
    categories: Coop.object
  },
  field_multiple: {
    categories: true
  },
  field_defaults: {
    rdftype: FLOCK_NS+'Blog',
    isIndexable: false
  }
});

coop.BlogCategory = coop.add_type (FLOCK_NS+'BlogCategory', {
  inherits_from: FLOCK_NS+'Object',
  fields: {
    categoryId: FLOCK_NS+'categoryId'
  },
  field_defaults: {
    rdftype: FLOCK_NS+'BlogCategory'
  }
});

//////////////////////////////////////////////////////////
// Actions
//////////////////////////////////////////////////////////

/*

Method is always a method on the adapter object identified by 
'service' as a short-name, and always takes a single argument 
of the urn of the person.

Flavour is a hint for display of actions within menus, etc. 

Service here is a (potentially-premature) optimisation, since
we can always get the service which contains the action.
*/
coop.Action = coop.add_type (FLOCK_NS+'Action', {
  inherits_from: FLOCK_NS+'WebObject',
  fields: {
    service: FLOCK_NS+'service',
    flavour: FLOCK_NS+'flavour',
    method: FLOCK_NS+'method',
    isPrimary: FLOCK_NS+'isPrimary',
    supportsMultiple: FLOCK_NS+'supportsMultiple',
    supportsSingle: FLOCK_NS+'supportsSingle'
  },
  field_types: {
    isPrimary: Coop.boolean,
    supportsMultiple: Coop.boolean,
    supportsSingle: Coop.boolean
  },
  field_defaults: {
    isIndexable: false,
    isPollable: false,
    isPrimary: false,
    supportsMultiple: false,
    supportsSingle: true,
    rdftype: FLOCK_NS+'Action'
  }
});

coop.DragAction = coop.add_type (FLOCK_NS+'DragAction', {
  inherits_from: FLOCK_NS+'Action',
  fields: {
    dragFlavour: FLOCK_NS+'flavour'
  },
  field_types: {
    dragFlavour: Coop.object
  },
  field_multiple: {
    dragFlavour: true
  },
  field_defaults: {
  }
});

coop.DragFlavour = coop.add_type (FLOCK_NS+'DragFlavour', {
  inherits_from: FLOCK_NS+'Object',
  fields: {
    flavourType: FLOCK_NS+'flavourType'
  }
});

coop.Service = coop.add_type (FLOCK_NS+'Service', {
  inherits_from: FLOCK_NS+'Object',
  fields: {
    serviceId: FLOCK_NS+'serviceId',
    desc: FLOCK_NS+'Description',
    loginURL: FLOCK_NS+'LoginURL',
    logoutOption: FLOCK_NS+'LogoutOption',
    contactLabel: FLOCK_NS+'ContactLabel',
    domains: FLOCK_NS+'Domains'
  },
  field_types: {
    logoutOption: Coop.boolean
  },
  field_defaults: {
    rdftype: FLOCK_NS+'Service',
    logoutOption: true,
    contactLabel: 'Friends'
  },
  container: 'Seq'
});

coop.Toolbar = coop.add_type (FLOCK_NS+'Toolbar', {
  fields: { 
    folder: FLOCK_NS+'folder',
    rdftype: RDF_NS + 'type'
  },
  field_types: { folder: Coop.object,  rdftype: Coop.resource },
  field_defaults: {  rdftype: FLOCK_NS+'Toolbar' },
  tight_binding: true
});

//////////////////////////////////////////////////////////
// Notes
//////////////////////////////////////////////////////////

coop.WebSnippet = coop.add_type (FLOCK_NS+'Snippet', {
  inherits_from: FLOCK_NS+'WebObject',
  fields: {
    content: FLOCK_NS+'Content',
    creationDate: FLOCK_NS+'date',
    tags: FLOCK_NS+'tags',
    snippetType: FLOCK_NS+'snippetType'
  },
  field_types: {
    creationDate: Coop.date
  },
  field_defaults: {
    snippetType: 'document',
    flockType: 'websnippet',
    isIndexable: false,
    rdftype: FLOCK_NS+'Snippet'
  }
});


//////////////////////////////////////////////////////////
// Business logic
//////////////////////////////////////////////////////////


coop.favorites_root = coop.get(FLOCK_NS+'FavoritesRoot');
if (!coop.favorites_root) coop.favorites_root = new coop.Folder (FLOCK_NS+'FavoritesRoot', {name: "master root"});
coop.me_root = new coop.Folder(FLOCK_NS+'MeRoot');
coop.accounts_root = coop.get(FLOCK_NS+'AccountsRoot');
if (!coop.accounts_root) coop.accounts_root = new coop.Folder (FLOCK_NS+'AccountsRoot');
coop.accounts_root.name = 'Me';

coop.my_flock = new coop.Folder(FLOCK_NS+'MyFlock');
coop.my_flock.name = 'My Flock';
coop.my_flock.save();

// coop.accounts_root.children.addOnce(coop.current_items_folder);

coop.favorites_root.children.addOnce(coop.accounts_root);
coop.favorites_root.children.addOnce(coop.my_flock);

coop.me_root.children.addOnce(coop.accounts_root);

coop.accounts_root.save && coop.accounts_root.save();
coop.me_root.save && coop.me_root.save();


// the bookmarks have to be contained with a folder for the UI to work 
// (it will show all of the bookmarks siblings so we are
//  creating a new folder instead of using coop.favorites_root)

coop.bookmarks_root = new coop.Folder (FLOCK_NS+'BookmarksRoot');
coop.bookmarks_root.name = 'Favorites';
coop.onlinebookmarks_root = new coop.Folder (FLOCK_NS+'OnlineBookmarksRoot');
coop.onlinebookmarks_root.name = 'Online Favorites';
coop.favorites_root.children.addOnce(coop.onlinebookmarks_root);
coop.toolbar = new coop.Toolbar ('urn:flock:toolbar'); // ja - is this an orphan???
coop.folders_root = new coop.UncountedFolder(FLOCK_NS+"FoldersRoot", {name: "Bookmarks"});
coop.folders_root.children.addOnce(coop.bookmarks_root);
coop.favorites_root.children.addOnce(coop.folders_root);
var mediaFaves = new coop.Folder("urn:media:favorites");
coop.media_favorites = mediaFaves;
coop.favorites_root.children.addOnce(coop.media_favorites);

/*
coop.current_items_folder = new coop.UncountedFolder (FLOCK_NS+'CurrentItemsFolder', {name: "All New Items"});
coop.current_items = new coop.Stream (FLOCK_NS+'CurrentItems', {
	name: "All New Items", 
	isPollable: false,
	apiType: "manual",
	isIndexable: false,
});
coop.current_items.onAddItem = function() {
	// JMC - Prune it down to the limit
	var count = 0;
	var MAX_CURRENT_ITEMS = 50;
	var enumer = this.children.enumerate();
	while (enumer.hasMoreElements()) {
		var child = enumer.getNext();
		if (!child.unseen || count > MAX_CURRENT_ITEMS) {
			this.children.remove(child);
			continue;
		}
		count++;
	}
}

coop.current_items_folder.children.addOnce(coop.current_items);
coop.me_root.children.addOnce(coop.current_items_folder);
*/


