//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2007
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

// JMC - Notification Overlay

/**
 * notificationOverlay - Object that handles the alert notifications for flock services.
 *
 */
var notificationOverlay = {
  DEBUG : false,    // Change to true to enable dumps

  observerService : null,
  wsMgr : null,
  mBranch: null,
  mSrv: null,
  debug: function (aMessage) {
    if (this.DEBUG) { dump("notificationOverlay.js: " + aMessage + "\n"); }
  },

  // "Constant" for the stringbundle used most in this file.
  SB: "common/notificationOverlay",

  init: function() {
    this.debug("init()");
    this.observerService = Components.classes["@mozilla.org/observer-service;1"]
                                     .getService(Components.interfaces.nsIObserverService);
    this.observerService.addObserver(this, 'notificationCountChanged', false);
    this.observerService.addObserver(this, 'new-stuff-notification', false);
    this.observerService.addObserver(this, 'test-notification', false);

    this.coop = Components.classes["@flock.com/singleton;1"]
                          .getService(Components.interfaces.flockISingleton)
                          .getSingleton("chrome://browser/content/flock/common/load-faves-coop.js")
                          .wrappedJSObject;

    this.observe(null, "notificationCountChanged", this.coop.accounts_root.id());
    this.observe(null, "notificationCountChanged", this.coop.my_flock.id());
    // Register to receive notifications of preference changes
    var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                                .getService(Components.interfaces.nsIPrefService);
    //adding observer on pref branch flock.notification.show.accounts
    this.mBranch = prefService.getBranch("flock.notification.show.accounts");
    this.mBranch.QueryInterface(Components.interfaces.nsIPrefBranch2);
    this.mBranch.addObserver("", this, false);
    this.wsMgr = Components.classes["@flock.com/webservice-manager;1"]
                           .getService(Components.interfaces.flockIWebServiceManager);
    this.wsMgr.addListener(this);
  },

  uninit: function() {
    this.debug("uninit()");
    if (this.observerService != null) {
      this.observerService.removeObserver(this, 'notificationCountChanged');
      this.observerService.removeObserver(this, 'new-stuff-notification');
      this.observerService.removeObserver(this, 'test-notification');
    }
    this.mBranch.removeObserver("", this, false);
    if (this.wsMgr != null) {
      this.wsMgr.removeListener(this);
    }
  },

  observe: function (aSubject, aTopic, aData) {
    this.debug("observe - " + aTopic);
    switch (aTopic) {
      case 'notificationCountChanged':
        switch (aData) {
          case this.coop.my_flock.id():
            document.getElementById('flock_PeopleNotificationBroadcaster').setAttribute('totalcount',  '(' + this.coop.my_flock.count + ')');
            document.getElementById('flock_PeopleNotificationBroadcaster').setAttribute('label', '(' + this.coop.my_flock.unseenItems + ')');
            break;
          case this.coop.accounts_root.id():
            document.getElementById('flock_MeNotificationBroadcaster').setAttribute('totalcount',  '(' + this.coop.accounts_root.count + ')');
            document.getElementById('flock_MeNotificationBroadcaster').setAttribute('label', '(' + this.coop.accounts_root.unseenItems + ')');
            break;
        }
        break;
        case 'nsPref:changed':
          if(this.mSrv)
          {
            var mPrefService = Components.classes["@mozilla.org/preferences-service;1"]
                                  .getService(Components.interfaces.nsIPrefBranch);
            var prefName = "flock.notification.show.accounts.account-setup-info-" + this.mSrv.shortName;
            if (mPrefService.getPrefType(prefName) && !mPrefService.getBoolPref(prefName) && this.mSrv.shortName != null)
              { 
                      // Pop the message bar
                      var notificationBox = gBrowser.getNotificationBox();
                      var links = [{
                         label: flockGetString(this.SB,"account.openSidebar"),
                         callback: function() {  toggleSidebar('flock_AccountsSidebarBroadcaster', true); }
                       }];
                      
                      var message = flockGetString(this.SB,"accounts.checkBoxClicked.label", [this.mSrv.title,this.mSrv.title]);
                      notificationBox.appendUniqueNotification(message, "account-remove-success-" + this.mSrv.shortName, 
                                                        (document.getElementById("page-proxy-favicon"))?document.getElementById("page-proxy-favicon").src:null,
                                                        notificationBox.FLOCK_PRIORITY_MEDIUM, null, links, null, true);
                }else if(mPrefService.getPrefType(prefName) && mPrefService.getBoolPref(prefName) && this.mSrv.shortName != null)
                {
                  var notificationBox = gBrowser.getNotificationBox();
                  notificationBox.removeCurrentNotification( "account-remove-success-" + this.mSrv.shortName);
                }
              }
          break;
      case 'new-stuff-notification':
        var service;
        var alertMessage;
        var alertTitle;
        var alertURL;
        var alertIcon;
        var notificationType;
        var notificationObj = this.coop.get(aData);
        var clickListener = {
          observe: function(subject, topic, data) {
            switch (topic) {  
              case "alertclickcallback":
              openUILinkIn(data, 'tab');
              break;
            }
          }
        }
        if (notificationObj) {
            notificationType = notificationObj.name;
            alertTitle = notificationObj.title;
            alertMessage =  notificationObj.description;
            alertIcon = notificationObj.favicon;
            alertURL = notificationObj.URL;
            this.showNotification(alertIcon, alertTitle, alertMessage, alertURL, clickListener);
        }
        break;

      case 'test-notification':
        var clickListener = {
          observe: function(subject, topic, data) {
            switch (topic) {
              case "alertclickcallback":
              openUILinkIn(data, 'tab');
              break;
            }
          }
        }
        var alertTitle = "Test Message";
        var alertMessage =  "This is a test notification, click to goto http://www.flock.com";
        var alertIcon = "http://x.myspace.com/images/icon_envelope.gif";
        var alertURL = "http://www.flock.com";
        this.showNotification(alertIcon, alertTitle, alertMessage, alertURL, clickListener);
        break;

      }
    },

    QueryInterface : function (aIID) {
      if ( !aIID.equals(Components.interfaces.nsISupports) &&
           !aIID.equals(Components.interfaces.nsIObserver) &&
           !aIID.equals(Components.interfaces.flockIWebServiceAccountListener) )
      {
        throw Components.results.NS_ERROR_NO_INTERFACE;
      }
      return this;
    },

    // BEGIN flockIWebServiceAccountListener
    onAccountCreated : function (aAccount, aDocument) {
      // First check whether this account is transient (ie., whether is has
      // already been "Remembered").  Below, we will suppress the first message
      // bar unless this account is transient.
      var acUtils = Components.classes["@flock.com/account-utils;1"]
                              .getService(Components.interfaces.flockIAccountUtils);
      var accountIsTransient = acUtils.isTransient(aAccount.urn);

      // Find the tab that contains aDocument
      var tabbrowser = getBrowser();
      var browsers = tabbrowser.mPanelContainer.childNodes;
      var window = aDocument.defaultView;
      while (window.parent != window) { window = window.parent; }
      for (var i = 0; i < browsers.length; i++) {
        var browser = tabbrowser.getBrowserAtIndex(i);
        if (browser.contentWindow == window) {
          // Pop the message bar
          var nBox = gBrowser.getNotificationBox(browser);
          var c_acct = this.coop.get(aAccount.urn);
          var svc = Components.classes[c_acct.serviceId]
                              .getService(Components.interfaces.flockIWebService);
          this.mSrv = svc; 
          var bar1Label;
          var bar1Button1;
          var bar1Links;
          var bar2Label;
          var bar2Button1;
          var bar2Links;
          var appName = brandGetString("brand", "brandShortName");

          if (svc instanceof Components.interfaces.flockIBookmarkWebService) {
            bar1Label = flockGetString(this.SB, "favs.bar1.label", [appName, svc.title]);
            bar1Button1 = flockGetString(this.SB, "favs.bar1.button1");
            bar2Links = [{
              label: flockGetString(this.SB, "favs.bar1.link1"),
              callback: function () {
                toggleSidebar("viewBookmarksSidebar", true);
              }
            }];
            bar2Label = flockGetString(this.SB, "favs.bar2.label");

          } else if (svc instanceof Components.interfaces.flockIBlogWebService) {
            bar1Label = flockGetString(this.SB, "blog.bar1.label", [appName, svc.title]);
            bar1Button1 = flockGetString(this.SB, "blog.bar1.button1");
            bar2Label = flockGetString(this.SB, "blog.bar2.label", [svc.title]);
            bar2Links = [{
              label: flockGetString(this.SB, "blog.bar2.link1"),
              callback: function () {
                flock_blogNewPost();
              }
            }];

          } else if (svc instanceof Components.interfaces.flockIMediaWebService) {
            bar1Label = flockGetString(this.SB, "media.bar1.label", [appName, svc.title]);
            bar1Button1 = flockGetString(this.SB, "media.bar1.button1");
            bar2Label = flockGetString(this.SB, "media.bar2.label", [appName, svc.title]);

          } else {
            bar1Label = "We need to add a message for your " + svc.title +
                        "account here. Please tell the developers.";
            bar1Button1 = "Remember this account";
          }

          var favicon = null;
          try { favicon = document.getElementById("page-proxy-favicon").src; }
          catch (ex) { };

          var successNotie = function () {
            nBox.appendUniqueNotification( bar2Label,
                                           "account-setup-success",
                                           favicon,
                                           nBox.FLOCK_PRIORITY_MEDIUM,
                                           null,
                                           bar2Links );
          };

          if (accountIsTransient) {
            // First we want to show a message bar giving the user a chance to
            // remember the account.
            var bar1Buttons = [{
              label: bar1Button1,
              popup: null,
              callback: function () {
                aAccount.keep();
                var metrics = Components.classes['@flock.com/metrics-service;1']
                  .getService(Components.interfaces.flockIMetricsService);
                metrics.reportCount("keep "+svc.shortName+" account");
                successNotie();
              }
            }];
            nBox.appendUniqueNotification( bar1Label,
                                           "accounts.account-setup-info-"+svc.shortName,
                                           favicon,
                                           nBox.FLOCK_PRIORITY_MEDIUM,
                                           bar1Buttons,
                                           bar1Links );
          } else {
            // The account has already been remembered (probably automatically)
            // so just show the feedback notie
            successNotie();
          }
          break;
        }
      }
    },

    onAccountPasswordMissing : function (aService, aDocument) {
      // Find the tab that contains aDocument
      var tabbrowser = getBrowser();
      var browsers = tabbrowser.mPanelContainer.childNodes;
      var window = aDocument.defaultView;
      while (window.parent != window) { window = window.parent; }
      var i = 0;
      for (; i < browsers.length; i++) {
        if (tabbrowser.getBrowserAtIndex(i).contentWindow == window) {
          // Pop the message bar
          var browser = tabbrowser.getBrowserAtIndex(i);
          var notificationBox = gBrowser.getNotificationBox(browser);
          var message =  flockGetString(this.SB,"account.logout", [aService.title,aService.title]);
          var buttons = [{
            label: "Log out",
            popup: null,
            callback: function () {
              aService.logout();
              browser.reload();
            }
          }];
          notificationBox.appendNotification( message,
            "account-setup-info",
            (document.getElementById("page-proxy-favicon"))?document.getElementById("page-proxy-favicon").src:null,
            notificationBox.PRIORITY_INFO_MEDIUM, buttons );
          break;
        }
      }
    },

    onAccountRemoved : function (aAccount) {
    },

    onAccountAuthChange : function (aAccount) {
    },
    // END flockIWebServiceAccountListener


    /**
     * showNotification - Actually displays the alertWindow with the nsIAlertsService
     *
     * @param aIconUrl URL to the icon we are going to display on the left
     * @param aAlertTitle Title of the alert
     * @param aAlertText Message to display (possibly clickable)
     * @param gotoUrl URL to load when aAlertText is clicked
     * @param aListener Listener object to callback to.
     */
    showNotification : function (aIconUrl, aAlertTitle, aAlertText, gotoUrl, aListener) {
      if (Components.classes["@mozilla.org/alerts-service;1"]) {
        // Only works for GTK and Window Toolkits?
        var alertsService = Components.classes["@mozilla.org/alerts-service;1"]
                                    .getService(Components.interfaces.nsIAlertsService);
        alertsService.showAlertNotification(aIconUrl,
                                          aAlertTitle,
                                          aAlertText,
                                          true,
                                          gotoUrl,
                                          aListener);
      } else {
        // STEVO: Hack to display alerts on OS X, nsIAlertsService is not compiled on OSX?
        //        We have to find a way to get nsIAlertsService to compile on OSX (it has something to do with the toolkit)
        window.openDialog("chrome://global/content/alerts/alert.xul", "_blank", "chrome,dialog=yes,titlebar=no,popup=yes",
                        aIconUrl,
                        aAlertTitle,
                        aAlertText,
                        true,
                        gotoUrl,
                        aListener);
      }
    },

    /**
     * handleEvent - Handles events attached to this object only onload/onunload are currently handled.
     *
     * @param aEvent Event object (.type holds the event that was triggered)
     */
    handleEvent: function(aEvent) {
      this.debug("handleEvent - " + aEvent.type);
      switch (aEvent.type) {
        case 'load':
          this.init();
          break;
        case 'unload':
          this.uninit();
          break;
        default:
          this.debug("unhandled event - " + aEvent.type);
          break;
      }
    }

};

// We attache load/unload listeners here so that we can initialize when the window loads
// since we are an overlay we don't have our own onload/onunload events.
window.addEventListener('load', notificationOverlay, false);
window.addEventListener('unload', notificationOverlay, false);
