/**
 * BEGIN FLOCK GPL
 * 
 * Copyright Flock Inc. 2005-2007
 * http://flock.com
 * 
 * This file may be used under the terms of of the
 * GNU General Public License Version 2 or later (the "GPL"),
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 * 
 * END FLOCK GPL
 */

var Cc = Components.classes;
var Ci = Components.interfaces;
var Cr = Components.results;

const ELTS_PER_PAGE = 20;
var gStreamController;

const FEED_FAVICON = "chrome://browser/skin/flock/feeds/feed.png";

var flockData = Cc["@mozilla.org/rdf/datasource;1?name=flock-favorites"].getService(Ci.nsIRDFDataSource);
var prefService = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
var faves_coop = Cc["@flock.com/singleton;1"].getService(Ci.flockISingleton).getSingleton("chrome://browser/content/flock/common/load-faves-coop.js").wrappedJSObject;
var sbs = Cc["@mozilla.org/intl/stringbundle;1"].getService(Ci.nsIStringBundleService);
var feedsBundle = sbs.createBundle("chrome://flock/locale/feeds/feeds.properties");
var feedManager = Cc["@flock.com/feed-manager;1"].getService(Ci.flockIFeedManager);
var dateFormatter = Cc["@flock.com/date-formatter;1"].getService(Ci.flockIDateFormatter);

var autoRead;
var autoReadExclude = {};

var previewMode;

var prefObserver = {
  observe: function(subject, topic, data) {
    autoRead = prefService.getBoolPref("flock.feeds.auto_mark_as_read");
  }
}


function onLoad() {
  onResizeContentArea();
  
  selectItemStyle(prefService.getCharPref("flock.feeds.default_view_style"));
  selectColumnStyle(prefService.getCharPref("flock.feeds.default_column_style"));
  
  autoRead = prefService.getBoolPref("flock.feeds.auto_mark_as_read");
  
  prefService.QueryInterface(Ci.nsIPrefBranch2);
  prefService.addObserver("flock.feeds.auto_mark_as_read", prefObserver, false);
  prefService.QueryInterface(Ci.nsIPrefBranch);

  var url = Cc["@mozilla.org/network/standard-url;1"].createInstance(Ci.nsIStandardURL);
  url.init(Ci.nsIStandardURL.URLTYPE_STANDARD, -1, window.location.toString(), null, null);
  url.QueryInterface(Ci.nsIURL);
  
  switch (url.scheme) {
    case "flock":
      previewMode = false;
      var resource = decodeURIComponent(url.path.substring(1));
      if (resource != "") {
        loadNode(resource);
      }
      _getMostRecentWindow().newsSidebarOpen();
      var message = feedsBundle.GetStringFromName("flock.feed.streamReader.automark.label");
      var nBox = _getNotificationBox();
      var buttons = [{
        label: feedsBundle.GetStringFromName("flock.feed.streamReader.automark.button"),
        popup: null,
        callback: function() { 
          turnOffAutoRead(); 
          var msg = feedsBundle.GetStringFromName("flock.feed.streamReader.automark.confirmation.label");
          var nBox = _getNotificationBox();
          var links = [{
             label: feedsBundle.GetStringFromName("flock.feed.streamReader.automark.confirmation.link"),
             callback: function() {  openPreferences("paneFeeds"); }
          }];
          nBox.appendUniqueNotification(msg,
                                        "feeds-subscribed-confirmed-info",
                                        FEED_FAVICON,
                                        nBox.thisFLOCK_PRIORITY_LOW,
                                        null,
                                        links);
          }
      }];
      nBox.appendUniqueNotification(message,
                                    "feeds-subscribed",
                                    FEED_FAVICON,
                                    nBox.FLOCK_PRIORITY_LOW,
                                    buttons,
                                    null, "feeds-subscribed");
      break;
    default:
      previewMode = true;
      var feedURL = url.spec;
      if (feedURL != "") {
        loadNode("urn:flock:feed:" + feedURL);
        showSubscribe(feedURL);
        _getMostRecentWindow().newsSidebarOpen();
      }
      break;
  }
  
  if (autoRead && !previewMode) {
    window.setTimeout(markVisibleRead, 2000);
  }
}

function onUnload() {
  prefService.QueryInterface(Ci.nsIPrefBranch2);
  prefService.removeObserver("flock.feeds.auto_mark_as_read", prefObserver);
}

function turnOffAutoRead() {
  prefService.setBoolPref("flock.feeds.auto_mark_as_read", false);
}

function onResizeContentArea() {
  document.getElementById("contentArea").setAttribute("style", "height: " + (window.innerHeight - 102) + "px");
}

function onContentScroll() {
  var contentArea = document.getElementById("contentArea");
  //dump("scrollHeight: " + contentArea.scrollHeight + " scrollTop: " + contentArea.scrollTop + " clientHeight: " + contentArea.clientHeight + "\n");
  if (autoRead && !previewMode) {
    markVisibleRead();
  }
  if ((contentArea.scrollTop + contentArea.clientHeight) == contentArea.scrollHeight) {
    //dump("onContentScroll: loading next page\n");
    gStreamController.loadNextPage();
  }
}

var _windowMediator = null;
function _getWindowMediator() {
  if (!_windowMediator) {
    _windowMediator = Cc["@mozilla.org/appshell/window-mediator;1"].
                      getService(Ci.nsIWindowMediator);
  }
  return _windowMediator;
}

function _getMostRecentWindow(aWindowType) {
  var windowType = aWindowType;
  if (!windowType) {
    windowType = "navigator:browser";
  }
  return _getWindowMediator().getMostRecentWindow(windowType);
}

function _getNotificationBox() {
  var recent = _getMostRecentWindow();
  var nBox = recent.gBrowser.getNotificationBox();
  return nBox;
}

function markVisibleRead() {
  var contentArea = document.getElementById("contentArea");
  var streamsDiv = document.getElementById("content");
  var children = streamsDiv.childNodes;
  for (var i = 0; i < children.length; i++) {
    var item = children[i];
    if (autoReadExclude[item.obj.id()]) continue;
    if ((item.offsetTop - 75) > contentArea.scrollTop && (((item.offsetTop - 75) + item.clientHeight) < (contentArea.scrollTop + contentArea.clientHeight))) {
      //dump("item " + i + ": offsetTop: " + item.offsetTop + " clientHeight: " + item.clientHeight + "\n");
      setViewed(item, true);
    }
  }
}

function markAllRead(aObject) {
  var queue = [aObject];
  while (queue.length>0) {
    aObject = queue.pop();
    if (!aObject) continue;

    // Mark the current object/stream as read
    if (aObject.isInstanceOf(faves_coop.Favorite)) { // Leaf
      aObject.unseen = false;
    }

    if (aObject.children) { // Container
      var children = aObject.children.enumerate();
      while (children.hasMoreElements())
        queue.push(children.getNext());
    }
  }
}

function markRead(div) {
  var obj = div.obj;
  if (!obj.unseen) return; // Already read
  obj.unseen = false;
}

function markUnread(div) {
  var obj = div.obj;
  if (obj.unseen) return; // Already unread
  obj.unseen = true;
}

function toggleViewed(div) {
  autoReadExclude[div.obj.id()] = true;
  if (hasClass(div, "read")) {
    setViewed(div, false);
  } else {
    setViewed(div, true)
  }
}

function setViewed(div, state) {
  if (state) {
    markRead(div);
    addClass(div, "read");
  } else {
    markUnread(div);
    removeClass(div, "read");
  }
}

function toggleSaved(div) {
  if (hasClass(div, "saved")) {
    setSaved(div, false);
  } else {
    setSaved(div, true)
  }
}

function setSaved(div, state) {
  if (state) {
    feedManager.getFeedItem(div.obj.id()).setFlagged(true);
    addClass(div, "saved");
  } else {
    feedManager.getFeedItem(div.obj.id()).setFlagged(false);
    removeClass(div, "saved");
  }
}

function makeDiv(aFavorite) {
  var newElement = document.createElement("div");
  if (aFavorite.unseen) {
    newElement.setAttribute("class", "post");
  } else {
    newElement.setAttribute("class", "post read");
  }

  var header = document.createElement("h3");
  header.setAttribute("class", "postHeader");
  newElement.appendChild(header);

  // create a link if URL exists
  if(aFavorite.URL) {
    var titleObj = document.createElement("a");
    titleObj.href = aFavorite.URL;
  } else {
    var titleObj = document.createElement("span");
  }

  // if a title exists use it, otherwise use the name field
  if(aFavorite.title) {
    titleObj.textContent = aFavorite.title;
  } else {
    titleObj.textContent = aFavorite.name;
  }

  header.appendChild(titleObj);
  
  var itemDetails = document.createElement("div");
  itemDetails.setAttribute("class", "itemDetails");
  newElement.appendChild(itemDetails);

  var date = document.createElement("span");
  date.setAttribute("class", "date");
  date.textContent = dateFormatter.getDateTimeString(new Date(aFavorite.datevalue).getTime());
  itemDetails.appendChild(date);

  switch(aFavorite.flockType) {
    case "photo":
      var content = document.createElement("div");
      content.setAttribute("class", "postContent postbody");
      var link = document.createElement("a");
      link.href = aFavorite.URL;
      var img = document.createElement("img");
      img.src = aFavorite.thumbnail;
      link.appendChild(img);
      content.appendChild(link);
      newElement.appendChild(content);
      break;
    case "feeditem":
      var feedItem = feedManager.getFeedItem(aFavorite.id());
      var unescapeHTMLSvc = Cc["@mozilla.org/feed-unescapehtml;1"].getService(Ci.nsIScriptableUnescapeHTML);
      
      if (feedItem.isFlagged()) {
        addClass(newElement, "saved");
      }

      var source = document.createElement("span");
      source.setAttribute("class", "source");
      
      if (feedItem.getFeed().getFavicon()) {
        var faviconURL = feedItem.getFeed().getFavicon().spec;
      } else {
        var faviconURL = "chrome://browser/skin/flock/feeds/feed-subscribed.png";
      }
      
      var link = document.createElement("a");
      link.href = feedItem.getFeed().getLink().spec;
      var img = document.createElement("img");
      img.src = faviconURL;
      link.appendChild(img);
      source.appendChild(link);
      
      link = document.createElement("a");
      link.href = feedItem.getFeed().getLink().spec;
      link.textContent = feedItem.getFeed().getTitle();
      source.appendChild(link);
      
      itemDetails.insertBefore(source, date);

      var content = document.createElement("div");
      content.setAttribute("class", "postContent postbody");
      var frag = unescapeHTMLSvc.parseFragment(feedItem.getContent(), false, null, content);
      content.appendChild(frag);
      newElement.appendChild(content);

      var excerpt = document.createElement("div");
      excerpt.setAttribute("class", "postContent postexcerpt");
      var frag = unescapeHTMLSvc.parseFragment(feedItem.getExcerpt(), false, null, excerpt);
      excerpt.appendChild(frag);
      newElement.appendChild(excerpt);
      break;
    default:
      var content = document.createElement("div");
      content.setAttribute("class", "postContent postbody");
      content.innerHTML = aFavorite.description;
      newElement.appendChild(content);
  }

  var footer = document.createElement("div");
  footer.setAttribute("class", "postFooter");
  newElement.appendChild(footer);
  
  var readunread = document.createElement("span");
  readunread.setAttribute("class", "readUnread");
  var readUnreadLink = document.createElement("a");
  readUnreadLink.href = "#";
  readUnreadLink.textContent = "Viewed";
  readUnreadLink.setAttribute("onclick", "toggleViewed(this.parentNode.parentNode.parentNode); return false;");
  readunread.appendChild(readUnreadLink);
  footer.appendChild(readunread);
  
  var saveThis = document.createElement("span");
  saveThis.setAttribute("class", "savePost");
  var saveThisLink = document.createElement("a");
  saveThisLink.href = "#";
  saveThisLink.textContent = "Save";
  saveThisLink.setAttribute("onclick", "toggleSaved(this.parentNode.parentNode.parentNode); return false;");
  saveThis.appendChild(saveThisLink);
  footer.appendChild(saveThis);
  
  var blogThis = document.createElement("span");
  blogThis.setAttribute("class", "blogThis");
  var blogThisLink = document.createElement("a");
  blogThisLink.href = "#";
  blogThisLink.textContent = "Blog";
  blogThisLink.setAttribute("onclick", "blogThis('" + aFavorite.URL + "', '" + encodeURIComponent(aFavorite.name).replace(/\'/g, "%27") + "', '" + encodeURIComponent(content.innerHTML).replace(/\'/g, "%27") + "'); return false;");
  blogThis.appendChild(blogThisLink);
  footer.appendChild(blogThis);

  return newElement;
}


function StreamController(aUrn, aDocument) {
  this.mNode = faves_coop.get(aUrn);
  this.mUrn = aUrn;
  this.doc = aDocument;

  // Make the header
  this.doc.title = this.mNode.name;
  
  if (previewMode) {
    addClass(this.doc.body, "preview");
  }

  switch (this.mNode.flockType) {
    case "person":
      this.doc.getElementById("pageTitle").textContent = this.mNode.name;
      this.doc.getElementById("feedLogo").src = this.mNode.avatar;
      break;
    case "folder":
      this.doc.getElementById("pageTitle").textContent = this.mNode.name;
      this.doc.getElementById("feedLogo").src = "chrome://browser/skin/flock/common/folder.png";
      this.doc.getElementById("itemCount").textContent = this.mNode.count;
      var element = document.getElementById("feed");
      addClass(element, "multi-stream");
      break;
    case "feed":
      var link = document.createElement("a");
      link.href = feedManager.getFeedFolderItem(aUrn).getLink().spec;
      link.textContent = this.mNode.name;
      this.doc.getElementById("pageTitle").appendChild(link);
      if (this.mNode.image) {
        this.doc.getElementById("feedLogo").src = this.mNode.image;
        this.doc.getElementById("feedLogo").removeAttribute("style");
      } else {
        this.doc.getElementById("feedLogo").setAttribute("style", "display: none;");
      }
      this.doc.getElementById("itemCount").textContent = this.mNode.count;
      break;
    case "stream":
      this.doc.getElementById("pageTitle").textContent =
        feedsBundle.GetStringFromName("flock.feed.streamReader.StreamController.stream");
      this.doc.getElementById("feedLogo").src = "chrome://browser/skin/flock/common/folder.png";
      this.doc.getElementById("itemCount").textContent = this.mNode.count;
      var element = document.getElementById("feed");
      addClass(element, "multi-stream");
      break;
  }
}

StreamController.prototype.clearItems = function() {
  var streamsDiv = this.doc.getElementById("content");
  var children = streamsDiv.childNodes;
  for (var i = children.length-1; i>=0; i--) {
    var child = children[i];
    streamsDiv.removeChild(child);
  }
}

StreamController.prototype.loadPage = function(aPageNum) {
  const RDFS = Cc["@mozilla.org/rdf/rdf-service;1"].getService(Ci.nsIRDFService);

  flockData.QueryInterface(Ci.nsIRDFRemoteDataSource);

  var rdfStreamNodes = streamWalker(flockData, RDFS.GetResource(this.mUrn));
  
  var aggregation = aggStreams(rdfStreamNodes, faves_coop);
  
  var streamsDiv = this.doc.getElementById("content");
  var cnt = 0;

  if (!aggregation.hasMoreElements()) {
    showEmpty();
  }

  while (cnt < ELTS_PER_PAGE * (aPageNum - 1) && aggregation.hasMoreElements ()) {
    aggregation.getNext();
    cnt++;
  }

  while (cnt < ELTS_PER_PAGE * aPageNum && aggregation.hasMoreElements()) {
    cnt++;
    var nextItem = aggregation.getNext();
    var div = makeDiv(nextItem);
    if (cnt % 2 == 1) {
      addClass(div, "odd");
    }
    streamsDiv.appendChild(div);
    div.obj = nextItem;
  }
  
  this.currentPage = aPageNum;
}

StreamController.prototype.loadNextPage = function() {
  if (Math.ceil(this.mNode.count/ELTS_PER_PAGE) > this.currentPage) {
    this.loadPage(this.currentPage + 1);
  }
}

StreamController.prototype.markAllAsRead = function() {
  // Update the UI
  var streamsDiv = document.getElementById("content");
  var children = streamsDiv.childNodes;
  for (var i = 0; i < children.length; i++) {
    setViewed(children[i], true);
  }
  // Mark everything as read in the stream
  markAllRead(this.mNode);
}


function loadNode(aUrn) {
  clearSubscribe();
  gStreamController = new StreamController(aUrn, document);
  gStreamController.loadPage(1);
}

function showEmpty() {
  var container = document.getElementById("dialog");
  var header = document.createElement("h3");
  header.textContent = feedsBundle.GetStringFromName("flock.feed.streamReader.showEmpty.header");
  container.appendChild(header);
  var description = document.createElement("p");
  description.textContent = feedsBundle.GetStringFromName("flock.feed.streamReader.showEmpty.desc");
  container.appendChild(description);
}

function showSubscribe(aURL) {
  var container = document.getElementById("dialog");
  var subscription = document.createElement("div");
  subscription.setAttribute("id", "subscription");
  subscription.setAttribute("class", "unsubscribed");

  var message = document.createElement("div");
  message.setAttribute("id", "message");
  message.innerHTML = feedsBundle.GetStringFromName("flock.feed.temp.preview.info");
  subscription.appendChild(message);
  
  /*
  var changeMessage = document.createElement("div");
  changeMessage.setAttribute("id", "changeReader");
  changeMessage.textContent = "You can subscribe with an external feed reader by changing your ";
  var changeLink = document.createElement("a");
  changeLink.setAttribute("href", "#");
  changeLink.setAttribute("onclick", "openPreferences("paneFeeds");");
  changeLink.textContent = "Feed Preferences";
  changeMessage.appendChild(changeLink);
  subscription.appendChild(changeMessage);
  */

  var action = document.createElement("div");
  action.setAttribute("id", "action");
  var subscribe = document.createElement("a");
  subscribe.setAttribute("class", "subscribe");
  subscribe.setAttribute("href", aURL);
  subscribe.setAttribute("onclick", "subscribeFeed('" + aURL + "'); return false;");
  subscribe.textContent = feedsBundle.GetStringFromName("flock.feed.temp.subscribe");
  action.appendChild(subscribe);
  subscription.appendChild(action);
  container.appendChild(subscription);
}

function clearSubscribe() {
  var container = document.getElementById("dialog");
  var subscription = document.getElementById("subscription");
  if (subscription) {
    container.removeChild(subscription);
  }
}

function subscribeFeed(aURL) {
  clearSubscribe();
  var feedManager = Cc["@flock.com/feed-manager;1"].getService(Ci.flockIFeedManager);
  var newsContext = feedManager.getFeedContext("news");
  var newsRoot = newsContext.getRoot();
  newsRoot.subscribeURL(aURL, null);
  
  document.location.reload();

  var message = feedsBundle.GetStringFromName("flock.feed.streamReader.feeds-subscribed");
  var nBox = _getNotificationBox();
  nBox.appendUniqueNotification(message,
                                "feeds-subscribed",
                                FEED_FAVICON,
                                nBox.FLOCK_PRIORITY_MEDIUM,
                                null,
                                null);
}

function hasClass(element, classStr) {
  var classes = element.className.split(" ");
  for (var c = 0; c < classes.length; c++) {
    if (classes[c] == classStr) {
      return true;
    }
  }
  return false;
}

function addClass(element, classStr) {
  if (!hasClass(element, classStr)) {
    var classes = element.className.split(" ");
    classes.push(classStr);
    element.className = classes.join(" ");
  }
}

function removeClass(element, classStr) {
  var classes = element.className.split(" ");
  for (var c = 0; c < classes.length; c++) {
    if (classes[c] == classStr) {
      classes.splice(c, 1);
      break;
    }
  }
  element.className = classes.join(" ");
}

function selectColumnStyle(colStyle) {
  prefService.setCharPref("flock.feeds.default_column_style", colStyle);
  var element = document.getElementById("feed");
  var classes = element.className.split(" ");
  for (var c = 0; c < classes.length; c++) {
    if (classes[c] == "double" || classes[c] == "single") {
      classes[c] = colStyle;
      break;
    }
  }
  element.className = classes.join(" ");
}

function selectItemStyle(itemStyle) {
  prefService.setCharPref("flock.feeds.default_view_style", itemStyle);
  var element = document.getElementById("feed");
  var classes = element.className.split(" ");
  for (var c = 0; c < classes.length; c++) {
    if (classes[c] == "full" || classes[c] == "excerpts" || classes[c] == "headlines") {
      classes[c] = itemStyle;
      break;
    }
  }
  element.className = classes.join(" ");
}

// New post, open post 
function blogThis(url, title, content) {
  var blogService = Cc["@flock.com/flock-blog;1"].getService(Ci.flockIBlogService);
  blogService.openEditor(decodeURIComponent(title), "<p>" + decodeURIComponent(content) + "</p>", null);
}
