//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2007
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

const RDFCU = Components.classes['@mozilla.org/rdf/container-utils;1'].getService (Components.interfaces.nsIRDFContainerUtils);
const RDFS = Components.classes['@mozilla.org/rdf/rdf-service;1'].getService (Components.interfaces.nsIRDFService);
const RDF_TYPE = RDFS.GetResource ('http://www.w3.org/1999/02/22-rdf-syntax-ns#type');
const RDF_FOLDER = RDFS.GetResource ('http://home.netscape.com/NC-rdf#Folder');
const RDF_BOOKMARK = RDFS.GetResource ('http://home.netscape.com/NC-rdf#Bookmark');
const RDF_LIVEMARK = RDFS.GetResource ('http://home.netscape.com/NC-rdf#Livemark');
const RDF_ICON = RDFS.GetResource ('http://home.netscape.com/NC-rdf#Icon');
const RDF_NAME = RDFS.GetResource ('http://home.netscape.com/NC-rdf#Name');
const RDF_DESCRIPTION = RDFS.GetResource ('http://home.netscape.com/NC-rdf#Description');
const RDF_URL = RDFS.GetResource ('http://home.netscape.com/NC-rdf#URL');
const RDF_FEEDURL = RDFS.GetResource ('http://home.netscape.com/NC-rdf#FeedURL');
const RDF_TOOLBAR = RDFS.GetResource ('http://home.netscape.com/NC-rdf#BookmarksToolbarFolder');
const RDF_ADDDATE = RDFS.GetResource ('http://home.netscape.com/NC-rdf#BookmarkAddDate');

const nsIRDFLiteral = Components.interfaces.nsIRDFLiteral;
const nsIRDFDate = Components.interfaces.nsIRDFDate;

var favSvc;
var migrationOverlayCoop;

function walkBookmarks (aDataSource, aNode, aFolder) {
    var name, description, url, favicon, isFeed;
    var children;
    var child;
    var node_type;
    var skip=false;
    var collection;
    var toolbar = false;
    var adddate = null;

    // dump("hello i am " + aNode + '\n');
    aNode.QueryInterface( Components.interfaces.nsIRDFResource );
    try {
        node_type = aDataSource.GetTarget( aNode, RDF_TYPE, true ).QueryInterface( Components.interfaces.nsIRDFResource ).ValueUTF8;
    } catch (e) { return; }

    if (node_type == 'http://home.netscape.com/NC-rdf#Folder') {
        dump('  --- decided it was a folder \n');
        if (!RDFCU.IsSeq (aDataSource, aNode)) {
            // its not a rdf seq - ignore it
            return;
        }
        name = null;
        try {
            name = aDataSource.GetTarget (aNode, RDF_NAME, true).
                QueryInterface (Components.interfaces.nsIRDFLiteral).Value;
        } catch (e) {
            // hmm - that sucks
        }
        
        

        var newFolder = new migrationOverlayCoop.Folder ({name: name});
        aFolder.children.add(newFolder);

        children = null;
        try {
            children = RDFCU.MakeSeq (aDataSource, aNode).GetElements ();
        } catch (e) {
            // oh well
        }

        var hasChildren = false;
        while (children && children.hasMoreElements ()) {
            child = children.getNext ();
            walkBookmarks (aDataSource, child, newFolder);
            hasChildren = true;
        }

        // if there were children...
        if (hasChildren) {
            try {
                toolbar = aDataSource.GetTarget (aNode, RDF_TOOLBAR, true).
                    QueryInterface (Components.interfaces.nsIRDFLiteral).Value;
            } catch (e) {
                // hmm - that sucks
            }
            if (toolbar) {
                migrationOverlayCoop.toolbar.folder = newFolder;
            }
        }
        
    } else if (node_type == 'http://home.netscape.com/NC-rdf#Bookmark' || 
               node_type == 'http://home.netscape.com/NC-rdf#Livemark') {
        try { name = aDataSource.GetTarget (aNode, RDF_NAME, true).QueryInterface (nsIRDFLiteral).Value; }
        catch (e) { name = ''; dump(e + '\n' ); }
        try { description = aDataSource.GetTarget (aNode, RDF_DESCRIPTION, true).QueryInterface (nsIRDFLiteral).Value; }
        catch (e) { description = ''; }
        try { url = aDataSource.GetTarget (aNode, RDF_URL, true).QueryInterface (nsIRDFLiteral).Value; }
        catch (e) { url = null; }
        try { favicon = aDataSource.GetTarget (aNode, RDF_ICON, true).QueryInterface (nsIRDFLiteral).Value; }
        catch (e) { favicon = null; }
        try { adddate = aDataSource.GetTarget (aNode, RDF_ADDDATE, true).QueryInterface (nsIRDFDate).Value / 1000; }
        catch (e) { adddate = null; }

        if (adddate == null) {
            // we had no date - let's make it now
            adddate = (new Date()).getTime();
        }

        if (node_type == 'http://home.netscape.com/NC-rdf#Livemark') {
          try {
            feedurl = aDataSource.GetTarget (aNode, RDF_FEEDURL, true).QueryInterface (nsIRDFLiteral).Value;
          }
          catch (e) { // missing feed url
            return;
          }
          var feedService = Components.classes["@flock.com/feed-manager;1"].getService(Components.interfaces.flockIFeedManager);
          var livemarkContext = feedService.getFeedContext("livemarks");
          feed = livemarkContext.getRoot().subscribeURL(feedurl, name);
          fav = migrationOverlayCoop.get(feed.id());
        } else {
          if (url == null) // missing url
            return;
          fav = new migrationOverlayCoop.Bookmark (url, {name: name, favicon: favicon, URL: url})
        }

        aFolder.children.add(fav);

    } else {
        dump('migration - not sure what to do with this node of type:\n\t');
        dump(node_type+'\n\n');
    }
    
}

function loadSubScript(spec) {
    var loader = Components.classes['@mozilla.org/moz/jssubscript-loader;1']
        .getService(Components.interfaces.mozIJSSubScriptLoader);
    var context = {};
    loader.loadSubScript(spec, context);
    return context;
}

// Migration
    
function doTheImportDeed() {
    var prefSvc = Components.classes['@mozilla.org/preferences-service;1']
                            .getService(Components.interfaces.nsIPrefBranch);

    favSvc = Components.classes['@mozilla.org/rdf/datasource;1?name=flock-favorites']
                       .getService(Components.interfaces.nsIRDFDataSource);

    var Coop = loadSubScript('chrome://browser/content/flock/common/coop.js').Coop;
    migrationOverlayCoop = Components.classes['@flock.com/singleton;1'].getService(Components.interfaces.flockISingleton).getSingleton('chrome://browser/content/flock/common/load-faves-coop.js').wrappedJSObject;

    walkBookmarks(RDFS.GetDataSource('rdf:bookmarks'), RDFS.GetResource('NC:BookmarksRoot'), migrationOverlayCoop.bookmarks_root);
    prefSvc.setCharPref('browser.search.selectedEngine', 'Yahoo'); // XXX this should be in branding
    prefSvc.setBoolPref("browser.didImport", true);

    favSvc.QueryInterface( Components.interfaces.nsIRDFRemoteDataSource );

    favSvc.Flush(); // this is needed as after import we shutdown then restart
}

