// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2007
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

// This is a library containing various functions useful for the online favorites back-end
// It is supposed to be called from the service itself, as it is accessing members
// not exposed in the interface (the coop instance, the logger...)

function stringify (foo) {
  if (foo==null) {
    return '';
  } else {
    return foo;
  }
}

function dateify (foo) {
  if (foo == null) {
    return 0;
  } else {
    return foo.getTime ();
  }
}

function sanitize_tags (tags) {
  return tags?tags.split(/[\s,]/).sort().join(','):'';
}

function bookmarks_match (bm1, bm2) {
  return (bm1.private == bm2.private && /* booleans - nice and easy */
          stringify (bm1.name) == stringify (bm2.name) &&
          stringify (bm1.description) == stringify (bm2.description) &&
          sanitize_tags(bm1.tags) == sanitize_tags(bm2.tags) &&
          dateify (bm1.datevalue) == dateify (bm2.datevalue));
}

var onlineFavoritesBackend = {
  getCompTK: function () {
    if (!gCompTK) {
      gCompTK = Cc["@flock.com/singleton;1"].getService(Ci.flockISingleton)
        .getSingleton("chrome://browser/content/flock/services/common/load-compTK.js")
        .wrappedJSObject;
    }
    return gCompTK;
  },

  createAccount: function (aService, aAccountID, aIsTransient) {
    var account_urn = aService.urn + ":" + aAccountID;
    var account;
    if (!aService._coop.Account.exists (account_urn)) {
      account = new aService._coop.Account (account_urn, {
        name: aAccountID,
        serviceId: CONTRACT_ID,
        service: aService._service,
        accountId: aAccountID,
        favicon: aService._ctk.favicon,
        URL: aService.getUserUrl(aAccountID),
        isTransient: aIsTransient,
        showInSidebar: false
      });
      aService._coop.accounts_root.children.addOnce (account);
    } else {
      account = aService._coop.get (account_urn);
    }

    // Create a "stream" for the online bookmarks
    var bookmarks_urn = account_urn+":bookmarks";
    if (!aService._coop.OnlineBookmarksStream.exists(bookmarks_urn)) {
      var bookmarks = new aService._coop.OnlineBookmarksStream(bookmarks_urn, {
        name: aAccountID+ ' on '+aService.title,
        userid: aAccountID,
        serviceId: aService._ctk.contractID,
        favicon: aService._ctk.favicon,
        isTransient: aIsTransient,
        refreshInterval: 5*60 // update every 5 minutes
      });
      aService._coop.onlinebookmarks_root.children.addOnce(bookmarks);
    }
    var acct = aService.getAccount(account_urn);
    return acct;
  },

  removeAccount: function (aService, aAccountUrn) {
    var acctCoopObj = aService._coop.get(aAccountUrn);
    // Next delete the bookmarks that are solely associated with this account
    var bookmarks_urn = aAccountUrn+':bookmarks';
    var bmStream = aService._coop.get(bookmarks_urn);
    if (bmStream) {
      var bmChildren= bmStream.children.enumerate();
      while (bmChildren.hasMoreElements()) {
        var bmChild = bmChildren.getNext();
        bmChild.destroy();
      }
    
      // Remove the Delicious Bookmarks node from its parents
      var parents = bmStream.getParents();
      for (var i = 0; i < parents.length; i++) {
        parents[i].children.remove(bmStream);
      }
      bmStream.destroy();
    
    } else {
      // this._logger.debug("  DID NOT find bookmark stream "+bookmarks_urn); 
    }
    // Next remove the account from any parents it might have
    var parents = acctCoopObj.getParents();
    for (var i = 0; i < parents.length; i++) {
      // this._logger.debug("   removing from parent "+i);
      parents[i].children.remove(acctCoopObj);
    }
    // Finally, kill the account node
    // this._logger.debug("  destroying account "+aUrn);
    acctCoopObj.destroy();
  },

  updateBookmark: function (aService, aAccountUrn, aServerBookmark, aPrivate) {
    var bookmark_urn = aAccountUrn+":"+aServerBookmark.URL;
    var localBookmark = null;
    var tagList;

    // Create/update the bookmark itself
    if (aService._coop.Bookmark.exists (bookmark_urn)) {
      // Need to check if any modification is needed before we assert anything
      var localBookmark = aService._coop.get (bookmark_urn);
      if (bookmarks_match(aServerBookmark, localBookmark)) {
        aService._logger.debug(aServerBookmark.URL+" is unchanged. Do nothing.");
        return;
      }
      aService._logger.debug(aServerBookmark.URL+" has been modified on the server. Updating.");
      if (stringify (aServerBookmark.name) != stringify (localBookmark.name))
        localBookmark.name = aServerBookmark.name;
      if (stringify (aServerBookmark.description) != stringify (localBookmark.description))
        localBookmark.description = aServerBookmark.description;
      if (sanitize_tags(aServerBookmark.tags) != sanitize_tags(localBookmark.tags)) {
        localBookmark.tags = sanitize_tags(aServerBookmark.tags)
        var tagenum = localBookmark.tag.enumerate();
        while (tagenum.hasMoreElements()) {
          var tag = tagenum.getNext();
          localBookmark.tag.remove(tag);
        }
       }
       var date  =  new Date();
       date = date.toLocaleString();
       localBookmark.LastModifiedDate = date;
       localBookmark.private = aPrivate;
       localBookmark.save();
    }
    else {
      // New bookmark
      aService._logger.debug(aServerBookmark.URL+" is a new bookmark. Creating.");
      localBookmark = new aService._coop.Bookmark (bookmark_urn, {
        URL: aServerBookmark.URL,
        name: aServerBookmark.name,
        description: aServerBookmark.description,
        tags: aServerBookmark.tags,
        datevalue: aServerBookmark.datevalue,
        hash: aServerBookmark.hash,
        flockType: "bookmark",
        private: aServerBookmark.shared=='no'?true:false,
        isPollable: false
      });
      // bookmarks.children.add(localBookmark);
    }
    var tagList = aServerBookmark.tags.split(/[\s,]/);
    for (i = 0; i < tagList.length; i++) {
      localBookmark.tag.addOnce(tagList[i]);
    }
    var bookmarks = aService._coop.get(aAccountUrn + ":bookmarks");
    onlineFavoritesBackend.classifyBookmark (aService, bookmarks, localBookmark, tagList);
  },

  classifyBookmark: function (aService, aBookmarks, aLocalBookmark, aTagList) {
     /*dump("Classify "+aLocalBookmark.URL+" with tags ");
     for (i = 0; i < aTagList.length; i++)
     dump(aTagList[i]+", ");
    dump("in "+aBookmarks.id()+" and length " +aTagList[0].length + "\n");
    */
    // Delete the bookmark from the tag folders
    var parents = aLocalBookmark.getParents();
    while (parents.length > 0) {
      var parent_ = parents.pop();
      parent_.children.remove(aLocalBookmark);
    }

    var tagFolder;
    // Add the bookmark to the tag folders it should be
    if ((aTagList.length == 1 && aTagList[0].length ==0) || (aTagList.length == 0) || (aTagList == [""])) {
      var notagUrn = aBookmarks.id()+":notag";
      if (aService._coop.Folder.exists(notagUrn)) {
        tagFolder = aService._coop.get(notagUrn);
      }
      else {
        //dump("  Create the tag folder for notag\n");
        tagFolder = new aService._coop.Folder(notagUrn,
          {
            name: "No Tag"
          }
        );
        aBookmarks.children.add(tagFolder);
      }
      tagFolder.children.add(aLocalBookmark);
    }
    else for (i = 0; i < aTagList.length; i++) {
      var tagUrn = aBookmarks.id() + ":tag:" + aTagList[i];
      if (aService._coop.Folder.exists(tagUrn)) {
        var tagFolder = aService._coop.get(tagUrn);
      }
      else {
        //dump("  Create the tag folder for "+aTagList[i]+": "+tagUrn+"\n");
        tagFolder = new aService._coop.Folder(tagUrn, 
          {
            name: aTagList[i]
          }
        );
        //dump("    => add it to "+aBookmarks.id()+"\n");
        aBookmarks.children.add(tagFolder);
      }
      tagFolder.children.add(aLocalBookmark);
    }
  },

  destroyBookmark: function (aService, aAccountId, aUrl) {
    var bookmark_urn = aService.urn+":"+aAccountId+":"+aUrl;

    if (aService._coop.Bookmark.exists (bookmark_urn)) {
      var bookmarks = aService._coop.get(aService.urn+":"+aAccountId+":bookmarks");
      var bookmark = aService._coop.get(bookmark_urn);

      bookmarks.children.remove(bookmark);
      bookmark.destroy();
    }
  },

  updateLocal: function (aService, aPostsList, aAccountUrn) {
    // Update the last update time for the bookmark stream
    var localBookmarks = aService._coop.get(aAccountUrn+':bookmarks');
    localBookmarks.last_update_time = (new Date());

    var synchronize = function (should_yield) {
      var urlHash = {}; // Useful to optimize deletion

      // Create new bookmarks, or update existing bookmarks
      for (var i=0; i < aPostsList.length; i++) {
        if (aPostsList[i].tags == "system:unfiled")
          aPostsList[i].tags = "";
        // To optimize deletion: put the URLs in a hash
        urlHash[aPostsList[i].URL] = true;
        onlineFavoritesBackend.updateBookmark (aService, aAccountUrn, aPostsList[i]);
        if (i % 10 == 0 && should_yield ())
          yield;
      }

      // Remove locally bookmarks deleted on the server
      var localLength = 0;
      // *TODO* Optimization: use direct RDF rather than coop to avoid useless GetTarget

      var i = 0;
      var _enum = localBookmarks.children.enumerate();
      while (_enum.hasMoreElements()) {
        var localTag = _enum.getNext();
        var _enum2 = localTag.children.enumerate();
        while (_enum2.hasMoreElements()) {
          var localBookmark = _enum2.getNext();
          if (!urlHash[localBookmark.URL]) {
            aService._logger.debug(localBookmark.URL+" has been deleted on the server. Deleting locally.");
            localBookmarks.children.remove(localBookmark);
            localBookmark.destroy();
          }
        }
        if (++i % 5 == 0 && should_yield ())
          yield;
      }
      // TODO: Clean empty tags
    }
    
    getCompTK().schedule(gTimers, 0.4, 40, synchronize);
  },

  updateTags: function (aService, aAccountId, aTags) {
    var localBookmarks = aService._coop.get(aService.urn+':'+aAccountId+':bookmarks');
    tagenum = localBookmarks.tag.enumerate();
    // Remove the existing tags
    while (tagenum.hasMoreElements())
      localBookmarks.tag.remove(tagenum.getNext());
    // Put the new tags
    for (var i=0; i<aTags.length; i++)
      localBookmarks.tag.addOnce(aTags[i]);
  },

  showNotification: function (aMessage) {
    var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                     .getService(Components.interfaces.nsIWindowMediator);
    var topNavWindow = wm.getMostRecentWindow("navigator:browser");
    
    var notificationBox = topNavWindow.gBrowser.getNotificationBox();
    var notification = notificationBox.getNotificationWithValue("favorite-error");
    if (notification) {
      notification.label = aMessage;
    }
    else {
      notificationBox.appendNotification(aMessage,
        "favorite-error",
        "chrome://browser/skin/Info.png",
        notificationBox.FLOCK_PRIORITY_HIGH, null);
    }
  }

}
