// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2007
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

var gProfiler = Components.classes["@flock.com/profiler;1"]
                          .getService(Components.interfaces.flockIProfiler);
var gMBProfEvt = gProfiler.profileEventStart("mediabar");

var $ = function(aId) {
  return document.getElementById(aId);
}

// The mediaOverlay uses this to pass in an initial query or person to load.
var initialMedia;

const FLNS = "http://flock.com/rdf#";
const RECENT_ROOT = "urn:media:recent";
const FAV_CHANNELS_ROOT = "urn:media:favorites";
const PERSON_QUERY_START = "user:";
const PER_PAGE = 50;
const MAX_RECENT = 20; // The maximum length of the recent channels list
const FLOCK_DEFAULT_MEDIA_SEARCH_PREF = "flock.media.search.currentEngine";
const FLOCK_MEDIA_ADD_FAV_DLG_PREF = "flock.media.addFavDlg.show";

const height_small = 130;
const height_small_meta = 147;
const height_large = 250;

var gPhotoDrawer;

var photoUploadSvc = Components.classes["@mozilla.org/rdf/datasource;1?name=flock-photo-uploads"]
                               .getService(Components.interfaces.flockIPhotoUploadService);

// The photo drawer controller

var PhotoDrawer = function(aNode, aSlideybox) {
  this.mSlideybox = aSlideybox;
  this.mGroupbarBox = this.mSlideybox.groupbarBox;
  this.pref_service = Components.classes["@mozilla.org/preferences-service;1"]
                                .getService(Components.interfaces.nsIPrefBranch);
  this.acUtils = Components.classes["@flock.com/account-utils;1"]
                           .getService(Components.interfaces.flockIAccountUtils);
  this.metrics = Components.classes["@flock.com/metrics-service;1"]
                           .getService(Components.interfaces.flockIMetricsService);
  this.mediaObserverService = Components.classes["@mozilla.org/observer-service;1"]
                                  .getService( Components.interfaces.nsIObserverService );
  this.mFinishedTasks = 0;
  this.init();
  this.loadInitialMedia();
  this.mediaObserver = 
    { 
      observe: function (aSubject, aTopic, aData) 
      { 
        //there are 2 tasks we are relying on: searchPerson() and loadQuery()
        if(gPhotoDrawer.mFinishedTasks == 2 )
        {
    
            if(gPhotoDrawer.isEmpty && gPhotoDrawer.hasPerson) {
              gPhotoDrawer.showActionMessage("Person found for search, however no media");
            }else if(gPhotoDrawer.isEmpty) {
                gPhotoDrawer.showActionMessage("Sorry, no media found for search");
              }

        }
      } 
    };
  this.mediaObserverService.addObserver(this.mediaObserver, 'mediaFinished', false);
}

PhotoDrawer.prototype = {
  init: function pdInit() {
    this.metrics.reportCount("mediabar open");

    var inst = this;
    this.mFinishedTasks = 0;
    this.mDnDInProgress = false;
    // Status of the UI
    this.mFilter = "";
    this.mPage = 0;
    this.isEmpty = true;
    this.isPersonQuery = false;
    this.objSynch = false;
    this.mControlBoxList = [];
    this.hasPerson = false;
    
    // persist which photo size was last selected
    if ($("mediabar").getAttribute("photoSizeLarge") == "true") {
      this.setPhotoSize("large");
    } else {
      this.setPhotoSize("small");
      if (this.getMode() == "consume") {
        top.flock_topbarSize(height_small);
      } else {
        top.flock_topbarSize(height_small_meta);
      }
    }

    // hide group bar in consume mode
    if (this.getMode() == "consume") {
      this.mGroupbarBox.setAttribute("hidden", "true");
    }

    // Information about the current channel
    this.mApi = null;
    this.mQuery = new queryHelper();
    this.mName = "";
    this.mAlbumLabel = "";
    this.mAlreadyLoaded = false;
    this.mPerson = null;
    this.mChannel = null;
    this.mHasPrivateMedia = false;

    this.mRDFService = Components.classes["@mozilla.org/rdf/rdf-service;1"]
                                 .getService(Components.interfaces.nsIRDFService);
    this.mFavsDS = this.mRDFService.GetDataSource("rdf:flock-favorites");
    this.mFavsDS.QueryInterface (Components.interfaces.flockIRDFObservable);

    this.unseenItemsPredicate = this.mRDFService.GetResource(FLNS + "hasUnseenItems");
    this.mFavsDS.addArcObserver(Components.interfaces.flockIRDFObserver.TYPE_CHANGE,
                                null, this.unseenItemsPredicate, null,
                                unseenItemsObs);

    // Coop stuff
    this._coop = Components.classes["@flock.com/singleton;1"]
                           .getService(Components.interfaces.flockISingleton)
                           .getSingleton("chrome://browser/content/flock/common/load-faves-coop.js")
                           .wrappedJSObject;
    this.recentRoot = this._coop.get(RECENT_ROOT);
    if (!this.recentRoot) {
      this.recentRoot = new this._coop.Folder(RECENT_ROOT);
      this._coop.favorites_root.children.add(this.recentRoot);
    }
    this.favChannelsRoot = this._coop.get(FAV_CHANNELS_ROOT);
    if (!this.favChannelsRoot ) {
      this.favChannelsRoot = new this._coop.Folder(FAV_CHANNELS_ROOT);
      this._coop.favorites_root.children.add(this.favChannelsRoot);
    }

    // UI stuff
    this.slideyBoxListener = {
      onDrop: function onDrop(aSelection, aHoverChild) {
      },
      onScrollLeft: function onScrollLeft() {
      },
      onScrollRight: function onScrollRight() {
        var pagesLeft = inst.mSlideybox.pagesLeft;
        if (pagesLeft < 3.3) {
          gPhotoDrawer.loadMore();
        }
      },
      onRemoveSelectionLeft: function onRemoveSelectionLeft() {
      },
      onRemoveSelection: function onRemoveSelection(aSelection) {
      }
    }

    this.mSlideybox.setListener(this.slideyBoxListener);

    // add the list of default channels to the popup and populate the search dropdown
    var channelsPopup = $("mediabar-channels-popup");
    var channelsSeperator = $("mediabar-featured-seperator")
    var searchPopup = $("mediabar-search");

    this.mApiMgr = Components.classes["@flock.com/photo-api-manager;1?"]
                             .getService(Components.interfaces.flockIPhotoAPIManager);
    var svcs = this.mApiMgr.services;

    // var searchpersonmenu = document.createElement("menu");
    // searchpersonmenu.setAttribute( "label", "View Person on...");
    // var searchpersonpopup = document.createElement("menupopup");
    // searchpersonmenu.appendChild(searchpersonpopup);

    while (svcs.hasMoreElements()) {
      var svc = svcs.getNext();

      var menu = document.createElement("menu");
      menu.setAttribute("label", svc.serviceName);
      menu.setAttribute("image", svc.iconUrl);
      menu.setAttribute("class", "menu-iconic");
      var menupopup = document.createElement("menupopup");
      menu.appendChild(menupopup);

      var channels = svc.channels;
      var count = 0;
      if (channels && channels.hasMoreElements()) {
        while (channels.hasMoreElements()) {
          count++;
          var channel = channels.getNext();
          channel.QueryInterface(Components.interfaces.flockIMediaChannel);
          var channelmenuitem = document.createElement("menuitem");
          channelmenuitem.setAttribute("label", channel.title );
          channelmenuitem.setAttribute("value", channel.id );
          channelmenuitem.setAttribute("oncommand",
                                       " gPhotoDrawer.mFilter = ''; gPhotoDrawer.loadQuery('" +
                                       svc.shortName +
                                       "', this.value, this.label); gPhotoDrawer.emptySearchbox();" +
                                       "gPhotoDrawer.metrics.reportCount('mediabar channel "+svc.shortName+" '+this.value);");
          menupopup.appendChild(channelmenuitem);
        }
      }
      if (count > 0) {
        channelsPopup.insertBefore(menu, channelsSeperator);
      }

      var searchmenuitem = document.createElement("menuitem");
      searchmenuitem.setAttribute("label", "Search " + svc.serviceName);
      searchmenuitem.setAttribute("image", svc.iconUrl);
      searchmenuitem.setAttribute("class", "menuitem-iconic");
      searchmenuitem.setAttribute("oncommand", "gPhotoDrawer.searchService('" +
                                  svc.shortName +
                                  "'); gPhotoDrawer.searchbarFocus();");
      $("media-search-menupopup").appendChild(searchmenuitem);
    }
    var defaultMediaSearchPref = "flickr";
    this.pref_service.getPrefType(FLOCK_DEFAULT_MEDIA_SEARCH_PREF) && (defaultMediaSearchPref =
      this.pref_service.getCharPref(FLOCK_DEFAULT_MEDIA_SEARCH_PREF));

    this.searchService(defaultMediaSearchPref);

    this.updateFavoritesChannelUI();

    this.mediabarForwardButton = $("mediabar-stream-forward");
    this.mediabarBackButton = $("mediabar-stream-back");

    // if there are no media accounts configured, pop message bar
    var mediaAccts = this.acUtils.getAccountsByInterface("flockIMediaWebService");
    if (!mediaAccts.hasMoreElements()) {
      var message = "You can view your own photos and videos in the Media Bar. Open the Accounts sidebar, choose your media service, and log in as usual.";
      var nBox = top.gBrowser.getNotificationBox();
      var links = [{
        label: "Get your media account",
        callback: function() {
          toggleSidebar("flock_AccountsSidebarBroadcaster", true);
        }
      }];
      nBox.appendUniqueNotification(message,
                                    "account-needed",
                                    null,
                                    nBox.FLOCK_PRIORITY_LOW,
                                    null, links, "accounts");
    }
    gProfiler.profileEventEnd(gMBProfEvt, "");
  },


  uninit: function pdUninit() {
    this.metrics.reportCount("mediabar close");

    // var unseenItemsPredicate = this.mRDFService.GetResource(FLNS + "hasUnseenItems");
    if (updateTimer) {
      clearTimeout(updateTimer);
    }
    updateTimer = null;
    this.mFavsDS.removeArcObserver(Components.interfaces.flockIRDFObserver.TYPE_CHANGE,
                                   null, this.unseenItemsPredicate, null,
                                   unseenItemsObs);
   this.mediaObserverService.removeObserver( this.mediaObserver, 'mediaFinished' );
    // unseenItemsObs.rdfChanged = null;
    // unseenItemsObs = null;
  },

  loadInitialMedia: function pdLoadInitialMedia() {
    // if there is new content, set initial media to be Preview New stream
    if (!initialMedia && this.mApiMgr.hasNewMedia) {
      initialMedia = {
        query: {
          service: "preview",
          query: "preview",
          name: "Preview New"
        }
      }
    }

    if (initialMedia && initialMedia.query) {
      this.loadQuery(initialMedia.query.service,
                     initialMedia.query.query,
                     initialMedia.query.name);
      if (this.recentRoot.children.enumerate().hasMoreElements()) {
        this._disableMediabarBackButton(false);
        this._disableMediabarForwardButton(true);
      }
      return;
    }

    if (initialMedia && initialMedia.person) {
      this.loadPhotoPerson(initialMedia.person.urn, initialMedia.person.svc);
      if (this.recentRoot.children.enumerate().hasMoreElements()) {
        this._disableMediabarBackButton(false);
        this._disableMediabarForwardButton(true);
      }
      return;
    }

    var nonPreviewStreamFound = false;
    var theEnum = this.recentRoot.children.enumerate();
    // look for the first non-preview stream in recent history to load
    while (theEnum.hasMoreElements()) {
      var recentItem = theEnum.getNext();
      if (recentItem.service != "preview") {
        nonPreviewStreamFound = true;
        this.loadFromId(recentItem.id());
        break;
      }
    }

    // if there are no non-preview streams in recent history OR
    // if recent history is empty, load default media stream
    if (!nonPreviewStreamFound) {
      var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                            .getService(Components.interfaces.nsIPrefService)
                            .getBranch("flock.media.defaultquery.");
      var service = prefs.getCharPref("service");
      var query = prefs.getCharPref("query");
      var label = prefs.getCharPref("label");
      this.loadQuery(service, query, label);
    }
    this.onSearchboxBlur();
    this._disableMediabarBackButton(!theEnum.hasMoreElements());
    this._disableMediabarForwardButton(true);
  },

  searchPerson: function pdSearchPerson(svcShortname, queryString) {
    var svc = Components.classes["@flock.com/photo-api-manager;1?"]
                        .getService(Components.interfaces.flockIPhotoAPIManager)
                        .getAPIFromShortname(svcShortname);
    if (svc instanceof Components.interfaces.flockISocialWebService) {
      var str = queryString;
      if (!str) {
        str = $("mediabar-search-textbox").value;
      }
      if (str == "" || $("mediabar-search-textbox").hasAttribute("empty")) {
        return;
      }
      var inst = this;
      var findUserListener = {
        onFindByUsernameResult: function onFindByUsernameResult(aPerson) {
          inst.hasPerson = true;
          var query = new queryHelper();
          query.user = aPerson.id;
          query.username = str;
          var name = str;
          inst.clearSnowman();
          inst.showSnowman(aPerson, svcShortname, query, name);
          inst.mFinishedTasks++;
          inst.mediaObserverService.notifyObservers(null, 'mediaFinished', null);
        },
        onError: function onError(aError) {
         inst.mFinishedTasks++;
         inst.mediaObserverService.notifyObservers(null, 'mediaFinished', null);
        }
      };
      svc.findByUsername(findUserListener, str);
    }
  },

  showSnowman: function pdShowSnowman(aPerson, svcShortname, query, name) {
    var SNOWMAN_ICON = "chrome://browser/skin/flock/services/common/no_avatar.png";
    if (aPerson.avatarUrl) {
      SNOWMAN_ICON = aPerson.avatarUrl;
    }
    var snowmanLabel = document.createElement("label");
    snowmanLabel.setAttribute("value", "Person found:");
    var snowman = document.createElement("image");
    snowman.setAttribute("src", SNOWMAN_ICON);
    snowman.setAttribute("style", "cursor: pointer;");
    snowman.setAttribute("query", query.stringVal);
    snowman.setAttribute("username", name);
    snowman.setAttribute("onclick", "gPhotoDrawer.loadQuery( '" +
                                    svcShortname + "', this.getAttribute('query'), this.getAttribute('username'));" +
                                    "gPhotoDrawer.metrics.reportCount('mediabar snowman click');");
    snowman.setAttribute("tooltiptext", "View " + aPerson.fullname + "'s stream");
    var label = document.createElement("label");
    label.setAttribute("value", aPerson.fullname);
    label.setAttribute("crop", "end");
    label.setAttribute("style", "font-weight: bold; max-width: 75px; cursor: pointer;");
    label.setAttribute("query", query.stringVal);
    label.setAttribute("username", name);
    label.setAttribute("onclick", "gPhotoDrawer.loadQuery('" + svcShortname +
                                  "', this.getAttribute('query'), this.getAttribute('username'));");
    label.setAttribute("tooltiptext", "View " + aPerson.fullname + "'s stream");
    $("snowman").appendChild(snowmanLabel);
    $("snowman").appendChild(snowman);
    $("snowman").appendChild(label);
    // display user photo count. see flockBug 7105
    if (aPerson.photoCount > -1) {
      var photoCountLabel = document.createElement("label");
      photoCountLabel.setAttribute("value", aPerson.photoCount + " photos");
      $("snowman").appendChild(photoCountLabel);
    }
    $("snowman").setAttribute("empty", "false");
  },

  clearSnowman: function pdClearSnowman() {
    $("snowman").setAttribute("empty", "true");
    while ($("snowman").lastChild) {
      $("snowman").removeChild($("snowman").lastChild);
    }
  },

  close: function pdClose() {
    top.flock_topbarHide();
  },

  move: function pdMove(ignoreLocation) {
    var location = top.flock_topbarMove(ignoreLocation);
    $("mediabar").setAttribute("location", location);
  },

  clear: function pdClear() {
    while(this.mSlideybox.hasChildNodes()) {
      this.mSlideybox.removeChild(this.mSlideybox.lastChild);
    }
    while(this.mGroupbarBox.childNodes.length > 0) {
      this.mGroupbarBox.removeChild(this.mGroupbarBox.lastChild);
    }
    this.mFinishedTasks = 0;
    this.mPage = 0;
    this.isEmpty = true;
    this.hasPerson = false;
    this.mHasPrivateMedia = false;
    this.mAlreadyLoaded = false;
    this.mControlBoxList = [];
    this.mSlideybox.scrollHome();
    this.clearSnowman();
  },

  refreshView: function pdRefreshView() {
    this.addRecent();
    this.loadFirst();
  },

  createGroupBar: function pdCreateGroupBar(aSecondPhoto, aFirstPhoto) {
    var newBar = document.createElement("hbox");
    newBar.id = "mediabar-groupbar-" + aFirstPhoto.id;
    newBar.setAttribute("hidden", true);

    this.mGroupbarBox.appendChild(newBar);
    aFirstPhoto.setAttribute("groupbar", newBar.id);
    if (aSecondPhoto) aSecondPhoto.setAttribute("groupbar", newBar.id);
    newBar.mSize = this.mSize;
    newBar.setAttribute("align", "center");
    newBar.setAttribute("class", "groupbar");

    var newLabel = document.createElement("label");
    newLabel.setAttribute("flex", "1");

    if (this.groupMethod(aSecondPhoto, aFirstPhoto) == "user") {
      newLabel.setAttribute("value", aFirstPhoto.username);
      newLabel.setAttribute("class", "authorLink");
      newLabel.setAttribute("querystring", "user:" + aFirstPhoto.userid + "|username:" + aFirstPhoto.username);
      newLabel.setAttribute("username", aFirstPhoto.username);
      newLabel.setAttribute("onclick", "gPhotoDrawer.loadQuery('" + aFirstPhoto.svc +"', "
         + " this.getAttribute('querystring'),  this.getAttribute('username'))");
    } else {
      var formatter = Components.classes["@flock.com/date-formatter;1"]
                                .getService(Components.interfaces.flockIDateFormatter);
      newLabel.setAttribute("value",
                            formatter.getShortDateTimeString(new Date(Number(aFirstPhoto.mPhoto.uploadDate))));
    }
    newBar.appendChild(newLabel);
    newBar.photoCount = (aSecondPhoto) ? 2 : 1;
    newBar.setupBar = function setupBar() {
      this.removeAttribute("hidden");
    };
    newBar.changeSize = function changeSize(sizeDiff) {
      var newWidth = Number(this.getAttribute("photowidth")) + sizeDiff;
      this.setAttribute("photowidth", newWidth);
      if (newWidth < 0) {
        newWidth = 0;
      }
      this.setAttribute("width", newWidth);
      this.setAttribute("style", "width: "+this.getAttribute("width")+"px;");
    };

    this.processPhotoSize(aFirstPhoto);
    if (aSecondPhoto) {
      this.processPhotoSize(aSecondPhoto);
    }
  },

  extendGroupBar: function pdExtendGroupBar(aGroupBar, aPhoto) {
    aPhoto.setAttribute("groupbar", aGroupBar.id);
    aGroupBar.photoCount = aGroupBar.photoCount + 1;
    this.processPhotoSize(aPhoto);
  },

  groupMethod: function pdGroupMethod(secondPhoto, firstPhoto) {
    if (!this.isPersonQuery &&
        (!secondPhoto || secondPhoto.userid == firstPhoto.userid))
    {
      return "user";
    }

    // Photos are uploaded within x minutes of each other...
    if (!secondPhoto ||
        (Number(firstPhoto.mPhoto.uploadDate) - Number(secondPhoto.mPhoto.uploadDate)) < 300000)
    {
      return "date";
    }
    return;
  },

  shouldGroup: function pdShouldGroup(secondPhoto, firstPhoto) {
    if (this.groupMethod(secondPhoto, firstPhoto)) {
      return true;
    }
    return false;
  },

  groupBarAround: function pdGroupBarAround(secondPhoto, firstPhoto) {
    if (firstPhoto.getAttribute("groupbar")) {
      this.extendGroupBar($(firstPhoto.getAttribute("groupbar")), secondPhoto);
    } else {
      this.createGroupBar(secondPhoto, firstPhoto);
    }
  },

  appendPhoto: function pdAppendPhoto(aPhoto) {
    var newPhoto = document.createElement("photo");
    newPhoto.setAttribute("photoControlBoxOpen", "false");
    newPhoto.setAttribute("style", "-moz-opacity: 0;");
    this.mSlideybox.appendChild(newPhoto);

    newPhoto.size = this.mSize;
    newPhoto.setAttribute("onload",
                          "gPhotoDrawer.fadeInPhoto(this); event.stopPropagation();");
    newPhoto.photo = aPhoto;

    var prevPhoto = newPhoto.previousSibling;
    if (prevPhoto && this.shouldGroup(newPhoto, prevPhoto)) {
      this.groupBarAround(newPhoto, prevPhoto);
    } else if (prevPhoto && !(prevPhoto.getAttribute("groupbar"))) {
      this.createGroupBar(null, prevPhoto);
    }

    if (newPhoto.is_public == "false") {
      this.mHasPrivateMedia = true;
    }
    newPhoto.tooltipId = "mediabar-tooltip";
    if (this.mQuery.hasKey("user")) {
      newPhoto.is_single = "true";
    } else {
      // JMC - Evil special case for the preview view
      if (this.mApi == "preview") {
       newPhoto.svc = aPhoto.svcShortName;
      } else {
        newPhoto.svc = this.mApi; // this is a string
      }
    }
    newPhoto.setupMetaData();
    this.isEmpty = false;
  },

  fadeInPhoto: function pdFadeInPhoto(aPhoto) {
    this.processPhotoSize(aPhoto);
    gDomFx.fadeIn(aPhoto, null);
  },

  loadFirst: function pdLoadFirst(aForce) {
    this.clear();
    this.setupSearch();
    this.setupAlbumMenu();
    this.loadMore();
  },

  onError: function pdOnError(aError) {
    document.getElementById("mediabar-statusbox").error(aError.errorString);
  },

  onSearchResult: function pdOnSearchResult(aBaseEnum) {
    document.getElementById("mediabar-statusbox").hidden = true;
    if (!this.mFilter) {
        this.onSearchboxBlur();
    }
    var biggestSeq = 0;
    var photo;

    function enumWrapper(aEnum) {
      this.innerEnum = [];
      while (aEnum.hasMoreElements()) {
        this.innerEnum.push(aEnum.getNext());
      }
      this.enumIndex = 0;
    }
    enumWrapper.prototype.rewind = function rewind(){
      this.enumIndex = 0;
    }
    enumWrapper.prototype.hasMoreElements = function hasMoreElements() {
      return this.innerEnum.length > this.enumIndex;
    }
    enumWrapper.prototype.getNext = function getNext() {
      return this.innerEnum[this.enumIndex++];
    }

    var aEnum = new enumWrapper(aBaseEnum);

    while (aEnum.hasMoreElements()) {
      photo = aEnum.getNext().QueryInterface(Components.interfaces.flockIPhoto);
      this.appendPhoto(photo);
      if (photo.id > biggestSeq) {
        biggestSeq = photo.id;
      }
    }
    if (photo && $(photo.id) && $(photo.id).getAttribute("groupbar")) {
      $($(photo.id).getAttribute("groupbar")).changeSize(-2);
      if ($(photo.id).previousSibling &&
          $($(photo.id).previousSibling.getAttribute("groupbar")))
      {
        $($(photo.id).previousSibling.getAttribute("groupbar")).setupBar();
      }
    }

    if (this.isPersonQuery && this.isEmpty) {
      this.hasPerson = true;
      this.isEmpty = false;
      var svc = Components.classes["@flock.com/photo-api-manager;1?"]
                          .getService(Components.interfaces.flockIPhotoAPIManager)
                          .getAPIFromShortname(this.mApi);
      this.showActionMessage("There is no media available in this " +
                             svc.serviceName + " account");
      !this.bHistoryNav && this.addRecent();
      this.updateWatch();
      this.disableWatch(false);
      this.mAlreadyLoaded = true;
      
    }else if(!this.mAlreadyLoaded) {
      !this.bHistoryNav && this.addRecent();
      this.updateWatch();
      this.disableWatch(false);
      this.mAlreadyLoaded = true;
    }

    this.latestSeq = biggestSeq;
    if (this.currentCoopStream &&
        biggestSeq > this.currentCoopStream.latestSeq)
    {
      this.currentCoopStream.latestSeq = biggestSeq;
    }
    aEnum.rewind();
    var photoAPIManager = Components.classes["@flock.com/photo-api-manager;1?"]
                        .getService(Components.interfaces.flockIPhotoAPIManager);
    var aStreamUrn = FAV_CHANNELS_ROOT + ":" + this.mApi + ":" +
                     this.mQuery.canonicalVal;
    photoAPIManager.processPhotoStream(aStreamUrn, aEnum);
    this.mFinishedTasks++;
    this.mediaObserverService.notifyObservers(null, 'mediaFinished', null);
  },

  loadMore: function pdLoadMore() {
    var svc = Components.classes["@flock.com/photo-api-manager;1?"]
                        .getService(Components.interfaces.flockIPhotoAPIManager)
                        .getAPIFromShortname(this.mApi);
    var query = new queryHelper(this.mQuery.stringVal);
    if (this.mFilter != "") {
      query.search = this.mFilter;
    }

    this.mPage++;
    svc.search(this, query.stringVal, PER_PAGE, this.mPage);
  },

  searchService: function pdSearchService(svcShortname) {
    this.metrics.reportCount("mediabar service: "+svcShortname);
    var svc = Components.classes["@flock.com/photo-api-manager;1?"]
                          .getService(Components.interfaces.flockIPhotoAPIManager)
                          .getAPIFromShortname( svcShortname );
    this.activeSearchSVC = svc;
    if (!this.activeSearchSVC) {
      svc = {svcShortname: "filter",
             iconUrl: "chrome://browser/skin/flock/common/lookingGlass.png"};
    }

    var FLOCK_DEFAULT_MEDIA_SEARCH_PREF = "flock.media.search.currentEngine";
    this.pref_service.setCharPref(FLOCK_DEFAULT_MEDIA_SEARCH_PREF, svcShortname);
    $("mediabar-search-button").setAttribute("style",
                                             "list-style-image: url(" + svc.iconUrl + ");");
  },

  searchbarFocus: function pdSearchbarFocus() {
    $("mediabar-search-textbox").focus();
    $("mediabar-search-textbox").select();
  },

  performSearch: function pdPerformSearch() {
    $("mediabar-search-textbox").blur();

    var str = $("mediabar-search-textbox").value;
    if ($("mediabar-search-textbox").hasAttribute("empty")) {
      str = "";
    }

    if (!this.activeSearchSVC) {
      if (this.supportsSearch) {
        this.mFilter = str;
        this.mQuery.search = this.mFilter;
        this.refreshView();
        return;
      } else {
        this.searchService( this.mApi );
      }
    }
    if (str == "") {
      return;
    }
    var svcShortname = this.activeSearchSVC.shortName;
    var query = new queryHelper();
    query.search = str; // perhaps we should strip this?
    var name = '"' + str + '"';
    var inst = this;
    setTimeout(function() { inst.searchPerson(svcShortname, str); } ,300);
    this.loadQuery(svcShortname, query.stringVal, name);
  },

  keyup: function pdKeyUp(aEvent) {
    switch (aEvent.keyCode) {
      case aEvent.DOM_VK_RETURN:
        this.performSearch();
        break;

      case aEvent.DOM_VK_DOWN:
        $("media-search-menupopup").showPopup($("media-search-menupopup"),
                                              -1, -1, "popup", "bottomleft",
                                              "topleft");
        break;
    }
  },

  loadAuthPerson: function pdLoadAuthPerson(aService) {
    var coopService = this._coop.get(aService.urn);
    var accounts = this._coop.Account.find({service: coopService});
    if (accounts.length > 0) {
      this.loadPhotoPerson(accounts[0].id(), aService.shortName);
    }
  },

  loadPhotoPerson: function pdLoadPhotoPerson(aPersonURN, aService) {
    this.mFilter = "";
    var identityTarget = this._coop.get(aPersonURN);
    var query = new queryHelper();
    query.user = identityTarget.accountId;
    query.username = identityTarget.name;
    this.loadQuery(aService, query.stringVal, identityTarget.name);
  },

  checkFavoritesMenu: function pdCheckFavoritesMenu() {
    var lastChild = $("flock-favorites-channels-dropdown").lastChild;
    if (!(lastChild.localName == "menuitem")) {
      var channelmenuitem = document.createElement("menuitem");
      channelmenuitem.setAttribute("label", "Empty");
      channelmenuitem.setAttribute("disabled", true)
      $("flock-favorites-channels-dropdown").appendChild(channelmenuitem);
    }
  },

  clearEmptyFavoritesMenu: function pdClearEmptyFavoritesMenu() {
    var lastChild = $("flock-favorites-channels-dropdown").lastChild;
    if(lastChild.label == "Empty") {
      $("flock-favorites-channels-dropdown").removeChild(lastChild);
    }
  },

  populateAccountsMenu: function pdPopulateAccountsMenu() {
    var mediaAccts = this.acUtils.getAccountsByInterface("flockIMediaWebService");
    var acctsMenu = $("flock-media-accounts-dropdown");
    while (acctsMenu.hasChildNodes()) {
      acctsMenu.removeChild(acctsMenu.lastChild);
    }
    if(!mediaAccts.hasMoreElements()) {
      var menuitem = document.createElement("menuitem");
      menuitem.setAttribute("disabled", "true");
      menuitem.setAttribute("label", "No media accounts currently configured");
      acctsMenu.appendChild(menuitem);
    }
    while (mediaAccts.hasMoreElements()) {
      var acct = mediaAccts.getNext()
                           .QueryInterface(Components.interfaces.flockIWebServiceAccount);
      var c_acct = this._coop.get(acct.urn);
      var svc = Components.classes[c_acct.serviceId]
                          .getService(Components.interfaces.flockIWebService);
      var menuitem = document.createElement("menuitem");
      menuitem.setAttribute("class", "menuitem-iconic");
      menuitem.setAttribute("image", c_acct.favicon);
      menuitem.setAttribute("label", c_acct.name);
      menuitem.setAttribute("username", c_acct.name);
      menuitem.setAttribute("querystring", "user:" + c_acct.accountId + "|username:" + c_acct.name);
      menuitem.setAttribute("oncommand", "gPhotoDrawer.mFilter = ''; gPhotoDrawer.loadQuery('" + svc.shortName +
                            "', this.getAttribute('querystring'), this.getAttribute('username')); gPhotoDrawer.emptySearchbox(); ");
      acctsMenu.appendChild(menuitem);
    }
  },

  onSettingsMenu: function pdOnSettingsMenu() {
    if ($("mediabar").getAttribute("advControls") == "on") {
      $("toggle-advanced-controls").setAttribute("label", "Hide Advanced Controls");
    } else {
      $("toggle-advanced-controls").setAttribute("label", "Show Advanced Controls");
    }

    if ($("mediabar").getAttribute("location") == "top") {
      $("toggle-location").setAttribute("label", "Move to bottom");
    } else {
      $("toggle-location").setAttribute("label", "Move to top");
    }
  },

  toggleAdvControls: function pdToggleAdvControls() {
    if ($("mediabar").getAttribute("advControls") == "on") {
      $("mediabar").setAttribute("advControls", "off");
    } else {
      $("mediabar").setAttribute("advControls", "on");
    }
  },

  toggleFavorite: function pdToggleFavorite() {
    this.metrics.reportCount("mediabar toggleFavorite");
    var id = FAV_CHANNELS_ROOT + ":" + this.mApi + ":" + this.mQuery.stringVal;
    if (this._coop.get(id)) {
      this.removeFavorite(id);
      top.updateMediaButton();
      this.updateWatch();
    } else {
      this.addFavorite();
    }
  },

  toggleSize: function pdToggleSize() {
    if ($("mediabar").getAttribute("photoSizeLarge") == "true") {
      this.setPhotoSize("small");
      this.metrics.reportCount("mediabar photosize small");
    } else {
      this.setPhotoSize("large");
      this.metrics.reportCount("mediabar photosize large");
    }
    this._resizePhotos();
  },

  toggleMode: function pdToggleMode() {
    if ($("mediabar").getAttribute("mode") == "consume") {
      this.interactMode();
      this.mGroupbarBox.removeAttribute("hidden");
    } else {
      this.consumeMode();
      this.mGroupbarBox.setAttribute("hidden", "true");
    }
  },

  setPhotoSize: function pdSetPhotoSize(aSize) {
    if (aSize == "large") {
      $("photo-drawer-size").setAttribute("checked", "true");
      this.mSize = "medium";
      top.flock_topbarSize(height_large);
      $("mediabar").setAttribute("photoSizeLarge", true);
    } else {
      $("photo-drawer-size").setAttribute("checked", "false");
      this.mSize = "thumb";
      if (this.getMode() == "consume") {
        top.flock_topbarSize(height_small);
      } else {
        top.flock_topbarSize(height_small_meta);
      }
      $("mediabar").removeAttribute("photoSizeLarge");
    }
  },

  consumeMode: function pdConsumeMode() {
    $("mediabar").setAttribute("mode", "consume");
    if ($("mediabar").getAttribute("photoSizeLarge") != "true") {
      top.flock_topbarSize(height_small);
    }
  },

  interactMode: function pdInteractMode() {
    $("mediabar").setAttribute("mode", "interact");
    if ($("mediabar").getAttribute("photoSizeLarge") != "true") {
      top.flock_topbarSize(height_small_meta);
    }
  },

  getMode: function pdGetMode() {
    return $("mediabar").getAttribute("mode");
  },

  _resizePhotos: function pd_resizePhotos() {

    for (var i = 0; i < this.mGroupbarBox.childNodes.length; i++) {
      this.mGroupbarBox.childNodes[i].setAttribute("width", 0);
      this.mGroupbarBox.childNodes[i].setAttribute("photowidth", 0);
      this.mGroupbarBox.childNodes[i].setAttribute("hidden", true);
    }

    var photos = this.mSlideybox.getElementsByTagName("photo");
    for (var i = 0; i < photos.length; i++) {
      photos[i].sizeProcessed = null;
      photos[i].size = this.mSize;
    }
  },

  processPhotoSize: function pdProcessPhotoSize(aPhoto) {
    // if the photo loads before the groupbar exists, then
    // bail and don't do this twice
    if (aPhoto.sizeProcessed == true || aPhoto.boxObject.width == 0) {
      return;
    }

    var padding = (aPhoto.mSize == "medium") ? 4 : 4;
    // if the photo is part of a groupbar, and it's the first photo,
    // add its size, plus left AND right edges

    if (aPhoto.getAttribute("groupbar")) {
      if (!$(aPhoto.getAttribute("groupbar"))) {
        return;
      }
      $(aPhoto.getAttribute("groupbar")).changeSize(aPhoto.boxObject.width + padding);
    }

    // if it doesn't have the same groupbar setting as the last one,
    // Close off the previous groupbar size
    if (aPhoto.previousSibling
        && aPhoto.previousSibling.getAttribute("groupbar")
        && aPhoto.previousSibling.getAttribute("groupbar") != aPhoto.getAttribute("groupbar"))
    {
        $(aPhoto.previousSibling.getAttribute("groupbar")).changeSize(-2);
        $(aPhoto.previousSibling.getAttribute("groupbar")).setupBar();
    }
    aPhoto.sizeProcessed = true;
  },

  _loadQuery: function pd_loadQuery(aService, aQuery, aName) {
    this.currentCoopStream = null;
    this.mApi = aService;
    if (aService == "preview") {
      $("mediabar").setAttribute("newMediaPreview", "true");
    } else {
      $("mediabar").removeAttribute("newMediaPreview");
    }
    this.mQuery = aQuery;
    if (this.mQuery.search && this.mQuery.user) {
      this.mFilter = this.mQuery.search;
    }
    this.mName = aName;
    var svc = Components.classes["@flock.com/photo-api-manager;1?"]
                        .getService(Components.interfaces.flockIPhotoAPIManager)
                        .getAPIFromShortname(aService);

    if (svc instanceof Components.interfaces.flockISocialWebService) {
      $("mediabar").setAttribute("social", "true");
    } else {
      $("mediabar").removeAttribute("social");
    }
    this.disableWatch(true);
    this.loadFirst(true);

    // If it's pollable, clear the new flag
    // How to get the higest seq out of the query?
    // I guess (ugly) we register a callback to appendPhoto
    var uri = FAV_CHANNELS_ROOT + ":" + aService + ":" + aQuery.canonicalVal;
    var item = this._coop.get(uri);
    if (item && item.isPollable) {
      // JMC - Bug #6733
      // If the unseenItems flag is getting cleared,
      // the template will remove the item from one rule,
      // but won't rerun against the other run
      // so we have to manually call rebuild
      var rebuilding = false;
      if (item.hasUnseenItems) rebuilding = true;
      item.hasUnseenItems = false;
      if (rebuilding) {
         $("flock-favorites-channels-dropdown").builder.rebuild();
      }
      // JMC - If this has a performance impact, we'll have to defer it for later
    }
    this.currentCoopStream = item;
  },

  clearAllNew: function pdClearAllNew() {
    // JMC - Should this only clear all new that's reflected in the current view?
    // For now, it clears all new mediaqueries no matter what
    var mediaQueries = this._coop.MediaQuery.find({hasUnseenItems: true});
    for (x in mediaQueries) {
      mediaQueries[x].hasUnseenItems = false;
    }

    // DP - Bug #8082
    // if we are on the preview stream and user clears new, load
    // the most recent non-preview stream
    if (this.mApi == "preview") {
      initialMedia = null;
      this.loadInitialMedia();
    }
    $("flock-favorites-channels-dropdown").builder.rebuild();
  },

  loadQuery: function pdLoadQuery(aService, aQueryString, aName, bIsHistoryNav) {
    this.metrics.reportCount("mediabar loadQuery "+aService);
    this.bHistoryNav = (bIsHistoryNav == true);
    this.mFilter = "";
    var aQuery = new queryHelper(aQueryString);
    this.isPersonQuery = aQuery.hasKey("user");

    if ($("mediabar-slideybox").leftmostChild) {
      this.mostRecentSeq = $("mediabar-slideybox").leftmostChild.mPhoto.id;
    } else {
      this.mostRecentSeq = null;
    }

    // we are doing a setTimeout because of a focus issue that happens
    // see flockBug 5827 --- ja
    var inst = this;
    setTimeout(function() { inst._loadQuery(aService, aQuery, aName); } ,0);
  },

  loadFromId: function pdLoadFromId(aId, bIsHistoryNav) {
    this.mFilter = "";
    this.emptySearchbox();
    var item = this._coop.get(aId);
    this.currentItem = item;
    // load it
    this.loadQuery(item.service, item.query, item.name, bIsHistoryNav);
    
    if (bIsHistoryNav) {
      // update the back/forward buttons
      var forwardItem = $(aId).previousSibling;
      var backItem = $(aId).nextSibling;
      this._disableMediabarBackButton((backItem == null));
      this._disableMediabarForwardButton((forwardItem == null));
    }
    
    var aQuery = new queryHelper(item.query);
    if (aQuery.search) {
      this.searchPerson(item.service, aQuery.search);
    }
    
  },

  /**
   * Navigation via the back and forward arrows.
   * @param aBackwards  boolean: true == go backwards (ex: back arrow clicked)
   */
  navigate: function pdNavigate(aBackwards) {
    //var currentItem = this._coop.get(RECENT_ROOT + ":" + this.mApi + ":" +
    //                                 this.mQuery.stringVal);
    var kids = this._coop.get(RECENT_ROOT).children;
    var recentEnum;
    if (aBackwards) {
      // Yes, this is unfortunately both unintuitive and correct.
      recentEnum = kids.enumerate();
    } else {
      recentEnum = kids.enumerateBackwards();
    }
    var item;
    var itemId;
    while (recentEnum.hasMoreElements() && itemId != this.currentItem.id()) {
      item = recentEnum.getNext();
      itemId = item.id();
    }
    if (recentEnum.hasMoreElements()) {
      this.loadFromId(recentEnum.getNext().id(), true);
    }
     // Now see if we need to enable or disable the arrows.
    if (recentEnum.hasMoreElements()) {
      // We're no longer at the end.
      if (aBackwards) {
        this._disableMediabarForwardButton(false);
      } else {
        this._disableMediabarBackButton(false);
      }
    } else {
      // We're at one end of the history.
      if (aBackwards) {
        this._disableMediabarBackButton(true);
        this._disableMediabarForwardButton(false);
      } else {
        this._disableMediabarForwardButton(true);
        this._disableMediabarBackButton(false);
      }
    }
  },

  /**
   * Helper for _disableMediabarBackButton and _disableMediabarForwardButton.
   * @param aElement  button to act upon
   * @param aState    true to disable, false to enable
   */
  _disableMediabarNavButton:
  function pd_disableMediabarNavButton (aButton, aState)
  {
    if (aState) {
      aButton.setAttribute("disabled", aState);
    } else {
      if (aButton.hasAttribute("disabled")) {
        aButton.removeAttribute("disabled");
      }
    }
  },

  /**
   * Disables or enables media bar back arrow button
   * @param aState  true to disable, false to enable
   */
  _disableMediabarBackButton:
  function pd_disableMediabarBackButton (aState)
  {
    this._disableMediabarNavButton(this.mediabarBackButton, aState);
  },

  /**
   * Disables or enables media bar forward arrow button
   * @param aState  true to disable, false to enable
   */
  _disableMediabarForwardButton:
  function pd_disableMediabarForwardButton (aState)
  {
    this._disableMediabarNavButton(this.mediabarForwardButton, aState);
  },

  addRecent: function pdAddRecent() {
    var svc = Components.classes["@flock.com/photo-api-manager;1?"]
                        .getService(Components.interfaces.flockIPhotoAPIManager)
                        .getAPIFromShortname(this.mApi);
    // XXX TODO: This is not very robust, and it doesn't allow favorite channels
    // to be recent channels without duplicating the mediaquery
    // Also, the split, below, looks fragile  - similar queries may clobber?

    var detailedQuery = new queryHelper(this.mQuery.stringVal);
   /*
    if (this.mostRecentSeq) {
      detailedQuery.leftmost = this.mostRecentSeq;
    }
    */
    
    // Don't double up at the top.
    if ($('mediabar-recent-channels-popup').childNodes[1] &&
    this._coop.get($('mediabar-recent-channels-popup').childNodes[1].id).query == detailedQuery.stringVal) {
      return;
    }
    
    var growing;
    /*
    if (item) {
      item.query = detailedQuery.stringVal;
      this.recentRoot.children.remove(item);
      growing = false;
    } else {
      */
     var item = new this._coop.MediaQuery({
        name: this.mLabel,
        query: detailedQuery.stringVal,
        service: this.mApi,
        favicon: svc.iconUrl
      });
      growing = true;
    // }
    if (this.recentRoot.children.enumerate().hasMoreElements()) {
      // We have history.  Enable "back".
      this._disableMediabarBackButton(false);
    }
    this.recentRoot.children.insertAt(item, 1);
    if (growing) {
      this.cleanRecentList();
    }
    this.currentItem = item;
  },

  addFavorite: function pdAddFavorite() {
    var photoAPI = Components.classes["@flock.com/photo-api-manager;1?"]
                        .getService(Components.interfaces.flockIPhotoAPIManager);
    var svc = photoAPI.getAPIFromShortname(this.mApi);

    var id = FAV_CHANNELS_ROOT + ":" + this.mApi + ":" + this.mQuery.canonicalVal;

    var item = new this._coop.MediaQuery(id, {
      name: this.mLabel,
      query: this.mQuery.canonicalVal,
      service: this.mApi,
      favicon: svc.iconUrl,
      isPollable: true,
      hasUnseenItems: false,
      latestSeq: this.latestSeq
    });

    //XXX TODO: Should this be addOnce?
    this.favChannelsRoot.children.add(item);
    // Call refresh manually to force a no-newness refresh
    this.updateWatch();
    var lister = {
      onResult: function (a, b) {},
      onError: function (a) {}
    }
    photoAPI.refresh(id, lister);

    if (!this.pref_service.getPrefType(FLOCK_MEDIA_ADD_FAV_DLG_PREF) ||
        (this.pref_service.getBoolPref(FLOCK_MEDIA_ADD_FAV_DLG_PREF) == true))
    {
      var url = "chrome://browser/content/flock/photo/addFavoriteDlg.xul";
      window.openDialog(url, "_blank", "centerscreen,modal");
    }
  },

  removeFavorite: function pdRemoteFavorite(aId) {
    var item = this._coop.get(aId);
    this.favChannelsRoot.children.remove(item);
    item.destroy();
    this.updateWatch();
  },

  showActionMessage: function pdShowActionMessage(aMessage) {
    $("mediabar-bannermessage").message = aMessage;
    $("mediabar-bannermessage").show();
  },

  hideActionMessage: function pdHideActionMessage() {
    $("mediabar-bannermessage").hide();
  },

  updateWatch: function pdUpdateWatch() {
    var id = FAV_CHANNELS_ROOT + ":" + this.mApi + ":" +
             this.mQuery.canonicalVal;
    // XXX: JMC - I have changed the above to allow favoriting of searches
    // within a user. Does this make sense?
    $("mediabar-watch").setAttribute("checked", this._coop.get(id) != null);
  },

  disableWatch: function pdDisableWatch(aState) {
    $("mediabar-watch").setAttribute("disabled", aState);
  },

  cleanRecentList: function pdCleanRecentList() {
    var count = 0;
    var theEnum = this.recentRoot.children.enumerate();
    while (theEnum.hasMoreElements()) {
      theEnum.getNext(); count++;
    }
    // Normally count can not be > 7 (only one element to remove), but you never know
    if (count > MAX_RECENT) {
      var fat = count - MAX_RECENT;
      var enumBack = this.recentRoot.children.enumerateBackwards();
      for (i = 0; i < fat; i++) {
        var item = enumBack.getNext();
        this.recentRoot.children.remove(item);
        item.destroy();
      }
    }
  },

  emptySearchbox: function pdEmptySearchBox() {
    $("mediabar-search-textbox").value = "";
    $("mediabar-search-textbox").setAttribute("empty", "true");
    // this.onSearchboxBlur();
    $("mediabar-search-textbox").blur();
  },

  onSearchboxBlur: function pdOnSearchboxBlur(aForce) {
    if ($("mediabar-search-textbox").value.length == 0 || aForce) {
 /*     if (this.supportsSearch) {
        $("mediabar-search-textbox").value = "Search " + this.mName;
      } else { */
        var mediaSearchApi = this.pref_service
                                 .getCharPref(FLOCK_DEFAULT_MEDIA_SEARCH_PREF);
        var txt;
        if (mediaSearchApi == "filter") {
          txt = "Filter this stream";
        } else {
          txt = "Search " + mediaSearchApi;
        }
        $("mediabar-search-textbox").value = txt;
  //    }
      $("mediabar-search-textbox").setAttribute("empty", true);
    }
  },

  onSearchboxFocus: function pdOnSearchboxFocus() {
    if ($("mediabar-search-textbox").hasAttribute("empty")) {
      $("mediabar-search-textbox").removeAttribute("empty");
      $("mediabar-search-textbox").value = "";
    }
  },

  /* this function will hopefully go away when we work in
     our solution to display the # of new items
     */
  updateFavoritesChannelUI: function pdUpdateFavoritesChannelUI() {
    if (this.mApiMgr.hasNewMedia) {
      $("mediabar").setAttribute("hasUnseenMedia", true);
    } else {
      $("mediabar").removeAttribute("hasUnseenMedia");
    }
  },

  lock: function pdLock() {
    if (this.objSynch) {
      // if already locked return false; could not lock
      return false;
    }
    this.objSynch = true;
    return true;
  },

  unlock: function pdUnlock() {
    setTimeout(function() { gPhotoDrawer.objSynch = false; }, 500);
  }
}

PhotoDrawer.prototype.__defineGetter__("mLabel",  function () {
  var str = "";
  var prefix = "";
  if (!this.mQuery) return "";
  if (this.mQuery.album) str += "'" + this.mQuery.albumlabel + "' by ";
  if (this.mQuery.username) str += this.mQuery.username;  
  if (this.mQuery.special || this.mApi == "preview") str += this.mName;
  if (this.mQuery.search) {
    if (str) str = " in " + str;
    str = "\"" + this.mQuery.search + "\"" + str;
  }
  return str;
})

PhotoDrawer.prototype.onMediabarChannelsPopupShowing = function() {
  var id = FAV_CHANNELS_ROOT + ":" + this.mApi + ":" + this.mQuery.stringVal;
  if (this._coop.get(id)) {
    $("favorite-this-stream-menuitem").setAttribute("label",
                                                    "Remove this stream from Favorites");
  } else {
    $("favorite-this-stream-menuitem").setAttribute("label",
                                                    "Add this stream to Favorites");
  }

  // Disable "Add this stream" menuitem if on Preview New
  if (this.mApi  == "preview") {
    $("favorite-this-stream-menuitem").setAttribute("disabled", "true");
  } else {
    $("favorite-this-stream-menuitem").removeAttribute("disabled");
  }

  // disable the Favorite Channels menu if empty
  var thisenum = this.favChannelsRoot.children.enumerate();
  $("favChannelMenu").setAttribute("disabled", "true");
  $("allNewMedia").setAttribute("disabled", "true");
  $("clearNew").setAttribute("disabled", "true");
  while (thisenum.hasMoreElements()) {
    $("favChannelMenu").removeAttribute("disabled");
    if (thisenum.getNext().hasUnseenItems) {
      $("allNewMedia").removeAttribute("disabled");
      $("clearNew").removeAttribute("disabled");
    }
  }

  if (this.mApiMgr.hasNewMedia) {
    $("favChannelMenu").setAttribute("label", "Favorite Streams (NEW)");
  } else {
    $("favChannelMenu").setAttribute("label", "Favorite Streams");
  }
}

PhotoDrawer.prototype.setAlbumName = function(albumId, albumLabel) {
  this.mQuery.album = albumId;
  this.mQuery.albumlabel = albumLabel;
  this.mAlbumLabel = albumLabel;
  var label = "Viewing: " + this.mLabel;

  $("current-stream-label").setAttribute("value", label);
  $("current-stream-menu").setAttribute("label", label);
  $("current-stream-menuitem").setAttribute("label", label);
}

PhotoDrawer.prototype.browseAlbum = function pd(eventTarget) {
  if (eventTarget.target.localName != "menuitem") {
    return;
  }

  var albumId = eventTarget.target.getAttribute("value");
  var albumLabel = eventTarget.target.getAttribute("label")

  this.clear();
  this.mFilter = "";
  this.emptySearchbox();
  this.setAlbumName(albumId, albumLabel);
  this.setupSearch();
  this.loadMore();
}

PhotoDrawer.prototype.setupSearch = function() {
  var svc = Components.classes["@flock.com/photo-api-manager;1?"]
                      .getService(Components.interfaces.flockIPhotoAPIManager)
                      .getAPIFromShortname(this.mApi);
  $("mediabar-statusbox").busy("Searching " + svc.serviceName);
  this.supportsSearch = svc.supportsSearch(this.mQuery.stringVal);
  if (this.supportsSearch) {
    $('media-search-filter-menuitem').removeAttribute('disabled');
  } else {
    $('media-search-filter-menuitem').setAttribute('disabled', true);
    var mediaSearchApi = this.pref_service
                             .getCharPref(FLOCK_DEFAULT_MEDIA_SEARCH_PREF);
    if (mediaSearchApi == 'filter') {
      $('media-search-filter-menuitem').nextSibling.doCommand();
    }
  }
}

function onLoad_photo(aEvt) {
  gPhotoDrawer = new PhotoDrawer($("mediabar"), $("mediabar-slideybox"));
  $("mediabar-refresh").addEventListener("command", onMediaRefresh, false);
  gPhotoDrawer.move(true);
}

function onUnload_photo(aEvt) {
  gDomFx.uninit();
  gPhotoDrawer.uninit();
}

function onClick_photo(aEvt) {
  $("mediabar-bannermessage").hide();
}

function onMediaRefresh(aEvt) {
  if(gPhotoDrawer.lock()) {
    gPhotoDrawer.refreshView();
    gPhotoDrawer.unlock();
  }
}

var updateTimer;
var unseenItemsObs = {
  rdfChanged: function (ds, type, source, predicate, target, oldtarget) {
    // JMC - Just set a dirty flag and update the UI on a timer
    // Otherwise this will really hurt when changing a lot of streams.
    // XXX TODO: Fix this for multi window bugs
    try {
      if (updateTimer) {
        clearTimeout(updateTimer);
      }
      updateTimer = setTimeout("gPhotoDrawer.updateFavoritesChannelUI();", 1);
    } catch(e) { dump(e); }
  }
}

function PhotoDropProcessor(aList, aHoverChild)  {
  this.mList = aList;
  this.mHoverChild = aHoverChild;
}

PhotoDropProcessor.prototype = {
  mList: null,
  start: function pdpStart() {
    var str = "";
    if (this.mHoverChild) {
      var newThing = Components.classes["@flock.com/photo-upload;1"]
                               .createInstance(Components.interfaces.flockIPhotoUpload);
      newThing.id = this.mHoverChild.getAttribute("id");
      this.mHoverChild = newThing;
    }
    photoUploadSvc.addListener(this);
    this.next();
  },

  next: function pdpNext() {
    if(this.mList.length==0) {
      photoUploadSvc.startImportQueue(this);
      return;
    }
    var url = this.mList.shift();
    if (url.split(":")[0] == "file") {
      photoUploadSvc.addMediaToImportQueue(url, this);
      this.next();
    } else {
      // dnding from a remote source
      var URIFixup = Components.classes["@mozilla.org/docshell/urifixup;1"]
                               .getService(Components.interfaces.nsIURIFixup);
      var uri = URIFixup.createFixupURI(url, URIFixup.FIXUP_FLAG_NONE);

      var IOService = Components.classes["@mozilla.org/network/io-service;1"]
                                .getService(Components.interfaces.nsIIOService);
      var channel = IOService.newChannelFromURI(uri);

      var self = this;

      var streamListener = {
        processor: self,
        sinkFile: null,
        sinkBufferStream: null,

        onStartRequest: function onStartRequest(request, context) {
          var extension;
          switch (channel.contentType) {
            case "image/jpeg":
              extension = "jpg";
              break;
            case "image/png":
              extension = "png";
              break;
            case "image/gif":
              extension = "gif";
              break;
            default:
              extension = "jpg";
              break;
          }
          var sinkFileName = url.split("/").pop();

          this.sinkFile = Components.classes["@mozilla.org/file/directory_service;1"]
                                    .getService(Components.interfaces.nsIProperties)
                                    .get("ProfD", Components.interfaces.nsIFile);
          this.sinkFile.append("flock_imagecache");
          if (!this.sinkFile.exists()) {
            this.sinkFile.createUnique(this.sinkFile.DIRECTORY_TYPE, 0700);
          }
          if (sinkFileName) {
            this.sinkFile.append(sinkFileName);
          }

          var sinkFileStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
                                         .createInstance(Components.interfaces.nsIFileOutputStream);
          sinkFileStream.init(this.sinkFile, 0x02 | 0x08 | 0x20, 0664, 0);
          this.sinkBufferStream = Components.classes["@mozilla.org/network/buffered-output-stream;1"]
                                            .createInstance(Components.interfaces.nsIBufferedOutputStream);
          this.sinkBufferStream.init(sinkFileStream, 65536 * 4);
        },

        onStopRequest: function onStopRequest(request, context, statusCode) {
          var ios = Components.classes["@mozilla.org/network/io-service;1"]
                              .getService(Components.interfaces.nsIIOService);
          var fileHandler = ios.getProtocolHandler("file")
                               .QueryInterface(Components.interfaces.nsIFileProtocolHandler);
          this.sinkBufferStream.close();
          photoUploadSvc.addMediaToImportQueue(fileHandler.getURLSpecFromFile(this.sinkFile), this.processor);
          this.processor.next();
        },

        onDataAvailable:
        function onDataAvailable(request, context, inputStream, offset, count) {
          this.sinkBufferStream.writeFrom(inputStream, count);
        }
      }

      channel.asyncOpen(streamListener, null);
    }
  },

  onMediaBatchImportStart: function pdpOnMediaBatchImportStart(aNumImport) {
  },

  onMediaBatchImportFinish: function pdpOnMediaBatchImportFinish() {
    photoUploadSvc.removeListener(this);
  },

  onMediaImportStart: function onMediaImportStart(aFilename, aNum) {
  },

  onMediaImportFinish: function onMediaImportFinish(aPhotoUpload) {
    photoUploadSvc.addPhotoUpload((this.mHoverChild?this.mHoverChild:null),aPhotoUpload);
  },

  onMediaImportError: function onMediaImportError(aFile) {
    var stringBundle = flock_photo.util.getStringBundle();
    alert(stringBundle.GetStringFromName("flock.photo.upload.error.invalid").replace("%1",aFile));
    this.next();
  },

  onUploadAdd: function pdpOnUploadAdd() {
  },

  onUploadRemove: function pdpOnUploadRemove() {
  },

  onError: function pdpOnError(aError) {
    alert(aError.errorString);
    this.next();
  },

  cancel: function pdpCancel() {
    // not impl
  }
}

var photoUploadDNDObserver = {
  canDrop: function (aEvent, aDragSession) {
    return !photoUploadSvc.isImporting && !photoUploadSvc.isUploading;
  },

  canHandleMultipleItems: true,

  onDragOver: function(aEvent, aFlavor, aSession) {
  },

  onDragEnter: function(aEvent, aSession) {
  },

  onDragExit: function(aEvent, aSession) {
  },

  onDrop: function(aEvent, aDropData, aSession) {
    aEvent.stopPropagation();
    var photoURLs = new Array();

    for (var c = 0; c < aDropData.dataList.length; c++) {
      var supports = aDropData.dataList[c].dataList[0].supports;
      var contentType = aDropData.dataList[c].dataList[0].flavour.contentType;
      var url;

      if (aSession.sourceNode && aSession.sourceNode.localName == "IMG") {
        contentType = null;
      }

      switch (contentType) {
        case "application/x-moz-file":
          try {
            var file = supports.QueryInterface(Components.interfaces.nsIFile);
            handleFile(file, photoURLs);
          } catch (ex) {
          }
          break;
        case "flock/shelficon":
          url = aSession.sourceNode.url;
          photoURLs.push(url);
          break;
        default:
          if (aSession.sourceNode && aSession.sourceNode.localName == "IMG") {
            url = aSession.sourceNode.getAttribute("src");
            var baseURI = aSession.sourceDocument.baseURI;
            var URIFixup = Components.classes["@mozilla.org/docshell/urifixup;1"]
                                     .getService(Components.interfaces.nsIURIFixup);
            url = URIFixup.createFixupURI(baseURI, URIFixup.FIXUP_FLAG_NONE).resolve(url);
            photoURLs.push(url);
          } else {
            url = supports.QueryInterface(Components.interfaces.nsISupportsString).data;
            url = url.split("\n")[0];
            if (isAPhoto(url)) {
              photoURLs.push(url);
            } else {
              if (url.indexOf("file") == 0) {
                var ios = Components.classes["@mozilla.org/network/io-service;1"]
                                    .getService(Components.interfaces.nsIIOService);
                var fileHandler = ios.getProtocolHandler("file")
                                     .QueryInterface(Components.interfaces.nsIFileProtocolHandler);
                var sourceSpec = fileHandler.getFileFromURLSpec(url);
                if (sourceSpec.exists()) {
                  handleFile(sourceSpec, photoURLs);
                }
              }
            }
          }
          break;
      }
    }

    var hoverNode = null;
    if (aEvent.target.nodeName == "slideyitem") {
      hoverNode = aEvent.target.parentNode.hoverChild.firstChild;
    }
    if (aEvent.target.nodeName == "photoupload") {
      var slideyboxHover = aEvent.target.parentNode.parentNode.hoverChild;
      if (slideyboxHover) {
        hoverNode = slideyboxHover.firstChild;
      }
    }

    if (photoURLs.length) {
      photoUploadSvc.launchUploader();
      var dropProcessor = new PhotoDropProcessor(photoURLs, hoverNode);
      window.setTimeout(function foo() { dropProcessor.start(); }, 400);
    }

    function handleFile(file, photoURLs) {
      if (file.isDirectory()) {
        var theEnum = file.directoryEntries;
        while (theEnum.hasMoreElements()) {
          var child = theEnum.getNext().QueryInterface(Components.interfaces.nsIFile);
          handleFile(child, photoURLs);
        }
      } else {
        url = flock_photo.util.getURLFromFile(file);
        if (isAPhoto(url)) {
          photoURLs.push(url);
        } else {
          var stringBundle = flock_photo.util.getStringBundle();
          alert(stringBundle.GetStringFromName("flock.photo.upload.error.unsupported").replace("%1",file.leafName));
        }
      }
    }

    function isAPhoto(aURL) {
      if (aURL.indexOf("file") != 0) {
         return false;
      }
      var lc = aURL.toLowerCase();
      if (lc.match(/.+jpg$/) || lc.match(/.+jpeg$/) || lc.match(/.+png$/) ||
          lc.match(/.+gif$/) || lc.match(/.+mpg$/)  || lc.match(/.+bmp$/))
      {
        return true;
      } else {
        return false;
      }
    }
  },

  getSupportedFlavours: function() {
    var flavors = new FlavourSet();
    flavors.appendFlavour("application/x-moz-file", "nsIFile");
    flavors.appendFlavour("text/x-moz-url");
    flavors.appendFlavour("flock/shelficon");
    flavors.appendFlavour("application/x-moz-url");
    return flavors;
  }
}

PhotoDrawer.prototype.setupAlbumMenu = function() {
  var inst = this;
  var albumsMenu = {
    popup: $("current-stream-popup"),
    numAlbums: 0,

    init: function() {
      $("current-stream-menu").setAttribute("hidden", "true");
      $("current-stream-menuitem").removeAttribute("hidden");

      var numItems = this.popup.childNodes.length;

      for (var i = 0; i < numItems; i++) {
        if (!this.popup.lastChild.hasAttribute("dontDelete")) {
          this.popup.removeChild(this.popup.lastChild);
        }
      }
    },

    refresh: function(albumEnumerator) {
      this.init();
      var albumId;
      var albumTitle;
      this.numAlbums = 0;
      while(albumEnumerator.hasMoreElements()) {
        $("current-stream-menuitem").setAttribute("hidden", "true");
        $("current-stream-menu").removeAttribute("hidden");

        var album = albumEnumerator.getNext();
        album.QueryInterface(Components.interfaces.flockIPhotoAlbum);
        var menuitem = document.createElement("menuitem");
        this.popup.appendChild(menuitem);
        if (this.album == album.id) {
          inst.setAlbumName(album.id, album.title);
        }
        menuitem.setAttribute("value", album.id);
        menuitem.setAttribute("label", album.title);
        this.numAlbums++;
      }
    }
  }

  var albumsGetterListener = {
   onGetAlbumsResult: function(aEnumerator) {
     albumsMenu.refresh(aEnumerator);
   },

    onError: function(aError) {
      dump("albumsGetterListener error - " + aError.errorString + "\n");
    }
  }

  $("current-stream-label").setAttribute("value", "Viewing: " + this.mLabel);
  $("current-stream-menu").setAttribute("label", "Viewing: " + this.mLabel);
  $("current-stream-menuitem").setAttribute("label", "Viewing: " + this.mLabel);
  albumsMenu.init();

  var userId = this.mQuery.getKey("user");

  if (userId && userId.length) {
    var svc = Components.classes["@flock.com/photo-api-manager;1?"]
                        .getService(Components.interfaces.flockIPhotoAPIManager)
                        .getAPIFromShortname(this.mApi);
    try {
      svc.getAlbums(albumsGetterListener, userId);
    } catch (ex) {
      // Usually happens when albums are not supported (YouTube...)
    }
  }
}

PhotoDrawer.prototype.onMediaTooltipShowing = function() {
  // if DnD in progress don't show the rich photo tooltip
  // see https://bugzilla.flock.com/show_bug.cgi?id=8559
  if (this.mDnDInProgress) {
    return false;
  }
}

window.addEventListener("load", onLoad_photo, false);
window.addEventListener("unload", onUnload_photo, false);
window.addEventListener("click", onClick_photo, false);
