// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2007
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//
const ENABLE_DEBUG = true; // switch to turn off slow debug code for production
function DEBUG(x) { if (ENABLE_DEBUG) dump("photoUploader.js: "+x+"\n"); }

const PREF_UPLOADER = "flock.photo.uploader.";
const INITIAL_FILEPATH = "initialPath";
const PU_INITIAL = 1;
const PU_SUCCESS = 2;
const PU_HAS_PHOTOS = 3;
const PU_IMPORTING = 4;

const Cc = Components.classes;
const Ci = Components.interfaces;

var photoUploader = {};
photoUploader.SVC = Cc["@mozilla.org/rdf/datasource;1?name=flock-photo-uploads"].getService(Ci.flockIPhotoUploadService);

photoUploader.mState = PU_INITIAL;
photoUploader.mApiListener = null;
photoUploader.totalNumImportedPhotos = null;
photoUploader.favsCoop = Cc['@flock.com/singleton;1'].getService(Ci.flockISingleton)
                           .getSingleton('chrome://browser/content/flock/common/load-faves-coop.js')
                           .wrappedJSObject,

dropEventHandler = {
  handleEvent: function(event) {
    nsDragAndDrop.drop(event, photoUploadDNDObserver);
  }
}

photoUploader.accountListener = {
  mMenulist: null,
  onStart: function(aSubject, aTopic) {
    $("cmd_doUpload").setAttribute("disabled", true);
  },
  onSuccess: function (aSubject, aTopic) {
    DEBUG("accountListener: onSuccess('"+aTopic+"')");
    photoUploader.setupUI();
    var acct = aSubject.QueryInterface(Ci.flockIWebServiceAccount);
    var urn = acct.urn;
    // Select the menu list item representing the account that was just
    // authenticated...
    var items = this.mMenulist.firstChild.childNodes;
    for (var i = 0; i < items.length; i++) {
      var item = items.item(i);
      if (item.value == urn) {
        this.mMenulist.selectedIndex = i;
        break;
      }
    }
    photoProperties.setupUIFields(urn);
    if (photoUploader.getPhotoCount())
      $("cmd_doUpload").removeAttribute("disabled");
    
  },
  onError: function (aSubject, aTopic, aError) {
    DEBUG("accountListener: onError('"+aTopic+"')");
    flock_photo.util.populateServicesPopup(this.mMenulist.firstChild, this);
    if (photoUploader.getPhotoCount())
      $("cmd_doUpload").removeAttribute("disabled");
  }
};

photoUploader.load = function() {
  DEBUG("load()");

  var metrics = Components.classes['@flock.com/metrics-service;1']
    .getService(Components.interfaces.flockIMetricsService);
  metrics.reportCount('uploader open');

  this.SVC.addListener(this.uploadListener);
  $("flock-photo-upload-slideybox").setListener(this.scrolleyboxListener);

  if ($("batch_is_public").value == "public") {
    $('batch_is_friend').disabled = true;
    $('batch_is_family').disabled = true;
  } else {
    $('batch_is_friend').disabled = false;
    $('batch_is_family').disabled = false;
  }
  
  this.totalNumImportedPhotos = 0;
  // this.mApiMgr.addListener(this.mApiListener);
  this.updateCommandState();
  var servicesMenulist = $("servicesMenuList");
  this.accountListener.mMenulist = servicesMenulist;
  this.setupUI();
  window.addEventListener("dragdrop", dropEventHandler, true);
}

photoUploader.updatePermInfo = function() {
  var result;
  if ($("batch_is_public").value == "public") {
    result = "(public)";
  } else {
    result = "(private";
    if ($('batch_is_friend').checked)
      result += ", friends";
    if ($('batch_is_family').checked)
      result += ", family";
    result += ")";
  }
  $("batch-permissions-info").value = result;
}

photoUploader.updateTags = function() {
  // Hack to get that executed after the key is processed, not before
  var sync = {
    notify: function (timer) {
      if ($('batch_tags').value != "") {
        var textNode = document.createTextNode("Will also be applied: " + $('batch_tags').value.replace(/,/g, ", "));
        if ($("global-tags-desc").firstChild)
          $("global-tags-desc").removeChild($("global-tags-desc").firstChild);
        $("global-tags-desc").appendChild(textNode);
        $("global-tags-desc").setAttribute("hidden", "false");
        window.sizeToContent();
      }
      else
        $("global-tags-desc").setAttribute("hidden", "true");
    }
  }
  this.timer = Components.classes['@mozilla.org/timer;1'].createInstance(Components.interfaces.nsITimer);
  this.timer.initWithCallback(sync, 0, Ci.nsITimer.TYPE_ONE_SHOT);
}

photoUploader.setupUI = function() {
  DEBUG(".setupUI()");
  var servicesMenulist = this.accountListener.mMenulist;
  var hasAuthAccounts =
    flock_photo.util.populateServicesPopup( servicesMenulist.firstChild,
                                            this.accountListener );
  if (hasAuthAccounts) {
    if (!servicesMenulist.selectedItem)
      servicesMenulist.selectedIndex = 0;
    
    photoProperties.setupUIFields(servicesMenulist.value);
  } else {
    servicesMenulist.selectedIndex = -1;
    var stringBundle = flock_photo.util.getStringBundle();
    servicesMenulist.value = stringBundle.GetStringFromName("flock.photo.service.select");
  }

  if ($("batch_is_public") && $("batch_is_public").value == "public")
    $("batch-public").setAttribute("selected", "true");
  else
    $("batch-private").setAttribute("selected", "true");
}

photoUploader.updatePrivacy = function() {
  var ispublic = ($("batch_is_public").value == "public");
  $('batch_is_friend').disabled = ispublic;
  $('batch_is_family').disabled = ispublic;

  photoUploader.updatePermInfo();
}

photoUploader.unload = function() {
  DEBUG("upload()");

  var metrics = Components.classes['@flock.com/metrics-service;1']
    .getService(Components.interfaces.flockIMetricsService);
  metrics.reportCount('uploader close');

  // The attribute holds the default value while the property holds the real value.
  // We set the attribute to have it saved in the localStore (using persist).
  $("batch-title").setAttribute("value", $("batch-title").value);
  $("batch-description").setAttribute("value", $("batch-description").value);
  $("batch_is_public").setAttribute("value", $("batch_is_public").value);

  this.SVC.removeListener(this.uploadListener);
}

photoUploader.onClose = function() {
  var promptService = Cc["@mozilla.org/embedcomp/prompt-service;1"].
                           getService(Ci.nsIPromptService);
  var localeService = Cc["@mozilla.org/intl/nslocaleservice;1"].getService(Ci.nsILocaleService);
  var stringBundleService = Cc["@mozilla.org/intl/stringbundle;1"].getService(Ci.nsIStringBundleService);
  var stringBundle = stringBundleService.createBundle("chrome://flock/locale/photo/photo.properties", localeService.getApplicationLocale());
    
  var dialogMessage = "";
  var titleMessage = "";
    
  if (this.SVC.isUploading) {
    titleMessage = stringBundle.GetStringFromName("flock.photo.upload.er.title");
    dialogMessage = stringBundle.GetStringFromName("flock.photo.upload.er.msg");

    var rv = promptService.confirm(null, titleMessage, dialogMessage);
    if (rv) {
      this.SVC.cancelUpload();
      return true;
    } else {
      return false;
    }
  } else if (this.SVC.isImporting) {
    titleMessage = stringBundle.GetStringFromName("flock.photo.upload.er.title");
    dialogMessage = stringBundle.GetStringFromName("flock.photo.import.er.msg");
    var rv = promptService.confirm(null, titleMessage, dialogMessage);
    if (rv) {
      this.SVC.cancelImport();
      return true;
    } else {
      return false;
    }
  } else 
    return true;
}


photoUploader.cancelUpload = function() {
  DEBUG("cancelUpload()");
  this.mRecentlyCancelled = true;
  this.SVC.cancelUpload();
}

photoUploader.updateCommandState = function(aDisable) {
  DEBUG("updateCommandState("+aDisable+")");
  //first, see if we have uploads
  if (this.mState == PU_IMPORTING) {
    $('flock-photo-import-deck').selectedIndex = 1;
    $("flock-photo-upload-actions").setAttribute("hidden", true);
    $("flock-photo-cancel-actions").setAttribute("hidden", true);
    $("cmd_removePhoto").setAttribute("disabled", true);
    $("cmd_doUpload").setAttribute("disabled", true);
    $("cmd_addFiles").setAttribute("disabled", true);
    $("cmd_launchSettings").setAttribute("disabled", true);
    photoUploader.disableSlideybox();
    this.updateMessage();
    return;
  } else {
    $('flock-photo-import-deck').selectedIndex = 0;
  }
 
  if (photoUploader.SVC.isUploading || aDisable) {
    photoUploader.disableSlideybox();
    photoProperties.clearSelection();
    $("flock-photo-upload-actions").setAttribute("hidden", true);
    $("flock-photo-cancel-actions").removeAttribute("hidden");
    $("cmd_removePhoto").setAttribute("disabled", true);
    $("cmd_doUpload").setAttribute("disabled", true);
    $("cmd_addFiles").setAttribute("disabled", true);
    $("cmd_launchSettings").setAttribute("disabled", true);
    return;
  }
  $("flock-photo-upload-actions").removeAttribute("hidden");
  $("flock-photo-cancel-actions").setAttribute("hidden", true);

  $("cmd_addFiles").removeAttribute("disabled");
  $("cmd_launchSettings").removeAttribute("disabled");

  var count = this.getPhotoCount();
  var selectionCount = this.getSelectionCount();


  if (count == 0) {
    // disable props, upload
    // setup message
    this.mState = PU_INITIAL;
    $("cmd_doUpload").setAttribute("disabled", true);
  }
  else {
    this.mState = PU_HAS_PHOTOS;
    $("cmd_doUpload").removeAttribute("disabled");
  }

  if (selectionCount == 0) {
    $("cmd_removePhoto").setAttribute("disabled", true);
  }
  else {
    $("cmd_removePhoto").removeAttribute("disabled");
  }
  this.updateMessage();
}

photoUploader.updateMessage = function(aOverride) {
  DEBUG("updateMessage("+aOverride+")");
  if (aOverride) {
    this.mState = aOverride;
  }
  var message = "";
  switch(this.mState) {
    case PU_INITIAL:
      $("flock-photo-instructions").removeAttribute("hidden");
      message = $("flock-photo-instructions-message").getAttribute("msgInitial");
    break;
    case PU_SUCCESS:
      $("flock-photo-instructions").removeAttribute("hidden");
      message = $("flock-photo-instructions-message").getAttribute("msgSuccess");
    break;
    case PU_HAS_PHOTOS:
      $("flock-photo-instructions").setAttribute("hidden", true);
      message = "";
    break;
    case PU_IMPORTING:
      $("flock-photo-instructions").setAttribute("hidden", true);
      message = "";
    break;
  }
  DEBUG(message + "<<<<<<MESSAGE");
  $("flock-photo-instructions-message").value = message;
}

photoUploader.getPhotoCount = function() {
  return ($("flock-photo-upload-slideybox").getElementsByTagName('slideyitem').length - 1);
}

photoUploader.getSelectionCount = function() {
  var selection = $("flock-photo-upload-slideybox").getSelection();
  return selection.length;
}

photoUploader.getAllPhotoUploads = function() {
  var photouploads = document.getElementsByTagName('photoupload');
  var uploadsArray = Array();
  for (var i = 1; i < photouploads.length; i++) {
    uploadsArray.push(photouploads[i]);
  }
  return uploadsArray;
}

photoUploader.addPhotoFiles = function() {
  DEBUG("addPhotoFiles()");
  var prefs = Cc["@mozilla.org/preferences-service;1"]
          .getService(Ci.nsIPrefService)
          .getBranch(PREF_UPLOADER);

  const nsIFilePicker = Ci.nsIFilePicker;
  var init_dir = Cc["@mozilla.org/file/local;1"].
      createInstance(Ci.nsILocalFile);
  var init_path_pref = getUnicodePref(INITIAL_FILEPATH, prefs);
  var photoUrls = Array();

  if (!init_path_pref) {
    var dirService = Cc["@mozilla.org/file/directory_service;1"]
         .getService(Ci.nsIProperties);

// platform specificness
//@line 358 "/home/build/tinderbuild_blue/src/flock/mozilla/browser/components/flock/photo/content/photoUploader.js"
    init_path_pref = dirService.get("Home", Ci.nsIFile).path;
//@line 360 "/home/build/tinderbuild_blue/src/flock/mozilla/browser/components/flock/photo/content/photoUploader.js"

    init_dir.initWithPath(init_path_pref);

    if (init_dir.path && init_dir.path.length) {
      init_path_pref = init_dir.path;
    }

    setUnicodePref(INITIAL_FILEPATH, prefs, init_path_pref);

  } else {
    init_dir.initWithPath(init_path_pref);
  }

  var fp = Cc["@mozilla.org/filepicker;1"]
                     .createInstance(nsIFilePicker);

  var localeService = Cc["@mozilla.org/intl/nslocaleservice;1"].getService(Ci.nsILocaleService);
  var stringBundleService = Cc["@mozilla.org/intl/stringbundle;1"].getService(Ci.nsIStringBundleService);
  var stringBundle = stringBundleService.createBundle("chrome://flock/locale/photo/photo.properties", localeService.getApplicationLocale());

  fp.init(window, stringBundle.GetStringFromName("flock.photo.upload.add"), nsIFilePicker.modeOpenMultiple);

  fp.appendFilters(nsIFilePicker.filterAll);
  
  fp.appendFilter(stringBundle.GetStringFromName("flock.photo.upload.imagefilter"), "*.jpg; *.gif; *.png; *.bmp");
  
  fp.filterIndex = 1;

  var homeDir = null;
  var mypicsPath = null;

  fp.displayDirectory = init_dir;

  if (fp.show() == nsIFilePicker.returnOK && fp.files) {
    var enum = fp.files;
    var item = null;
    var file = null;
    var path = null;
    while (enum.hasMoreElements()) {
      item = enum.getNext();
      file = item.QueryInterface(Ci.nsIFile);
      path = file.path.toLowerCase();
      if ( path.match(/\.jpg$/) ||
           path.match(/\.jpeg$/) ||
           path.match(/\.png$/) ||
           path.match(/\.gif$/) ||
           path.match(/\.mpg$/) ||
           path.match(/\.bmp$/) ) {
        photoUrls.push(flock_getURLFromFile(file));
      }
    }
    var dropProcessor = new PhotoDropProcessor(photoUrls, null);
    dropProcessor.start();
    var lastSelectedDir = file.path.substring(0, file.path.indexOf(file.leafName));
    setUnicodePref(INITIAL_FILEPATH, prefs, lastSelectedDir);
    $('flock-photo-instructions').setAttribute('hidden', true);
  }
}


photoUploader.enableSlideybox = function() {
  $("flock-photo-disable").setAttribute("hidden", true);
}

photoUploader.disableSlideybox = function() {
  $("flock-photo-disable").removeAttribute("hidden");
}

photoUploader.enableGlobal = function(aEnable) {
  if (!$("batch-title"))
    return; // Side-uploader
  $("flock-batch-disable").setAttribute("hidden", aEnable?"true":"false");
}

photoUploader.clearBatch = function() {
  if (!$("batch-title"))
    return; // Side-uploader
  $("batch-title").value = "";
  $("batch-description").value = "";
}

// this may move to slideybox
photoUploader.selectFirstPhoto = function() {
  $("flock-photo-upload-slideybox").firstChild.nextSibling.selected = true;
  $("flock-photo-upload-slideybox").firstChild.nextSibling.focus();
}

photoUploader.uploadListener = {
  mErrorOnUpload: false,
  mWindow: window,
  importBatchSize: 0,
  mNewAlbum: false,
  onUploadStart: function (aCurrentUpload, aCount, aMax) {
    DEBUG("uploadListener: onUploadStart()");
    // clear properties selection
    photoUploader.mRecentlyCancelled = false;
    photoUploader.uploadListener.mErrorOnUpload = false;
    photoUploader.updateCommandState();
    photoUploader.enableGlobal(false);

    var localeService = Cc["@mozilla.org/intl/nslocaleservice;1"].getService(Ci.nsILocaleService);
    var stringBundleService = Cc["@mozilla.org/intl/stringbundle;1"].getService(Ci.nsIStringBundleService);
    var stringBundle = stringBundleService.createBundle("chrome://flock/locale/photo/photo.properties", localeService.getApplicationLocale());

    var msg = stringBundle.GetStringFromName("flock.photo.upload.progress").replace(/%count/, aCount).replace(/%max/, aMax).replace(/%title/,aCurrentUpload.title);

     // -1 because the current photo is not uploaded yet
    var percent = ((aCount - 1)*100) / aMax;
    $('uploading-image').setAttribute("src", aCurrentUpload.thumbFileSpec);
    $('uploading-title').setAttribute("value", msg);
    $('upload-progressmeter').setAttribute("value", percent.toString()+"%");
  },
  onUploadProgress: function (aCurrentProgress) {
    // Why is this now not updating the progress bar?
    DEBUG("uploadListener: onUploadProgress(" + aCurrentProgress + ")");
    $('upload-progressmeter').setAttribute("value", aCurrentProgress.toString()+"%");
  },
  onUpload: function (aCurrentUpload, aCount, aMax) {
  },
  onUploadAdd: function (aCurrentUpload) {
    DEBUG("uploadListener: onUploadAdd()");
    if (!aCurrentUpload) {
      photoProperties.synch();
      photoProperties.invalidate();
    }
    else {
      if (photoProperties.updateSelectedItem(aCurrentUpload)) {
        $(aCurrentUpload.id).parentNode.selected = true;
      }
      photoUploader.updateCommandState();
      if (photoUploader.getPhotoCount() == 1) {
        photoUploader.selectFirstPhoto();
      }
    }
  },
  onUploadRemove: function () {
    DEBUG("uploadListener: onUploadRemove()");
    if (photoUploader.SVC.isUploading) return;
    photoUploader.updateCommandState();
  },
  onUploadComplete: function () {
    DEBUG("uploadListener: onUploadComplete()");
    $('flock-photo-upload-deck').selectedIndex = 0;
    photoUploader.enableSlideybox();
    photoUploader.updateCommandState();
    photoUploader.enableGlobal(true);
    photoUploader.clearBatch();
    // Delete the temporary directory, unless the single photo uploader is open

    if (!this.mErrorOnUpload && !photoUploader.mRecentlyCancelled)
    {
      var window = this.mWindow;
      if (!singleUploadWindow) {
        window.setTimeout("photoUploader.SVC.cleanPhotoUploadCache();", 1000);
      }
      var showUploadedPics = flock_getBoolPref('flock.photo.uploader.showPhotosOnUpload');
      if (showUploadedPics) {
        var account = photoUploader.favsCoop.get($('servicesMenuList').selectedItem.value);

        // if we are uploading a to a new album in flickr, increase the refresh delay
        // see bug #4765
        var delayTime;
        if (account.service.name == 'flickr' && this.mNewAlbum) {
          delayTime = 10000;
        } else {
          delayTime = 5000;
        }
        var service = $('servicesMenuList').selectedItem.svc;
        //var authPerson = service.getAuthPerson();

        var wm = Cc["@mozilla.org/appshell/window-mediator;1"]
          .getService(Ci.nsIWindowMediator);
        var win = wm.getMostRecentWindow('navigator:browser');
        if (win) {
          win.mediabar.showActionMessage("The Media Stream will refresh once the service has processed the upload");
        }
        
        setTimeout( function() {
          var wm = Cc["@mozilla.org/appshell/window-mediator;1"]
            .getService(Ci.nsIWindowMediator);
          var win = wm.getMostRecentWindow('navigator:browser');
          if (win) {
            win.mediabar.hideActionMessage();
            win.mediabar.loadQuery(account.service.name, 'user:' + account.accountId, account.name);
          }
        }, delayTime );
      }
      var windowMediator = Cc["@mozilla.org/appshell/window-mediator;1"]
        .getService(Ci.nsIWindowMediator);
      var singleUploadWindow = false;
      var allWindows = windowMediator.getEnumerator(null);
      while (allWindows.hasMoreElements()) {
        var win = allWindows.getNext();
        if (win.name == "singlePhotoUploading")
          singleUploadWindow = true;
      }
      window.setTimeout("photoUploader.updateMessage(PU_SUCCESS);", 0);//force override
    }
  },
  onUploadError: function (aUpload, aCount, aMax, aError) {
    DEBUG("uploadListener: onUploadError()");
    var errorMessage = aError.errorString;
    // if it's an unknow error display any addition info the server might return
    if (aError.errorCode == aError.PHOTOSERVICE_UNKNOWN_ERROR) {
      errorMessage += "\n\nAdditionally, the server returned the message " + aError.serviceErrorString;
    }
    alert(aError.errorString);
    photoUploader.enableSlideybox();
    photoUploader.enableGlobal(true);
    this.mErrorOnUpload = true;
  },
  onAuthStateChange: function (aState) {
  },
  onMediaImportError: function() {
  },
  onError: function(aError) {
  },
  onUploadCreateError: function(aFile) {
    var localeService = Cc["@mozilla.org/intl/nslocaleservice;1"].getService(Ci.nsILocaleService);
    var stringBundleService = Cc["@mozilla.org/intl/stringbundle;1"].getService(Ci.nsIStringBundleService);
    var stringBundle = stringBundleService.createBundle("chrome://flock/locale/photo/photo.properties", localeService.getApplicationLocale());

    DEBUG("*** onUploadCreateError in photoUploader.js");
    alert(stringBundle.GetStringFromName("flock.photo.upload.error.invalid"));
    photoUploader.updateCommandState();
  },
  onMediaBatchImportStart: function(aBatchSizeNum) {
    this.importBatchSize = aBatchSizeNum;
    $('import-current-number').setAttribute('value', 0);
    $('import-total-number').setAttribute('value', aBatchSizeNum);
    $('import-progressmeter').removeAttribute('value');
    photoUploader.mState = PU_IMPORTING;
    photoUploader.updateCommandState();
  },  
  onMediaBatchImportFinish: function() {
    photoUploader.mState = PU_INITIAL;
    photoUploader.enableSlideybox();
    var firstPhoto = $("flock-photo-upload-slideybox").childNodes.item(1);
    if (firstPhoto) {
      firstPhoto.selected = true;
      firstPhoto.focus();
    }
    photoUploader.updateCommandState();
  },
  onMediaImportFinish: function(aMediaUpload, aCount) {
    var progress = (aCount/this.importBatchSize)*100;
    $('import-progressmeter').setAttribute('value', progress + '%');
  },
  onMediaImportStart: function (aFilename, aNumInBatch) {
    $('import-current-photo-title').setAttribute('value', aFilename);
    $('import-current-number').setAttribute('value', aNumInBatch);
    photoUploader.mState = PU_IMPORTING;
    photoUploader.updateCommandState();
  },
}

photoUploader.scrolleyboxListener = {
  onDrop: function(aHoverChild, aSelection) {
    try {
      photoProperties.synch();
      var svc = photoUploader.SVC;
      var hoverChild = aHoverChild;
      if (hoverChild) hoverChild = svc.getPhotoUpload(hoverChild.firstChild.id);
      var ids = new Array();
      for (var i = 0; i < aSelection.length; ++i) {
        ids[i] = aSelection[i].firstChild.id;
      }
      var hoverID = null;
      if (aHoverChild) hoverID = aHoverChild.firstChild.id;

      for (var i = 0; i < ids.length; ++i) {
        if (hoverID && hoverID == ids[i]) continue;
        svc.addPhotoUpload(hoverChild, svc.getPhotoUpload(ids[i]));
      }
      for (var i = 0; i < ids.length; ++i) {
        $(ids[i]).parentNode.selected = true;
      }
    }
    catch(e) {
      alert(e);
    }
  },
  onScrollRight: function() {
  },
  onScrollLeft: function() {
  },
  onSelect: function(aSelection) {
    if (photoUploader.SVC.isUploading) return;
    photoUploader.updateCommandState();
    var uploadItems = Array();
    for (var i = 0; i < aSelection.length; i++) {
      uploadItems.push(aSelection[i].firstChild);
    }
    photoProperties.setSelection(uploadItems, true);
  },
  onRemoveSelection: function(aSelection) {
    var svc = photoUploader.SVC;
    photoUploader.removeSelection(aSelection);
  },
}

photoUploader.cancelImport = function() {
  this.SVC.cancelImport();
}

photoUploader.doUpload = function() {
  DEBUG("doUpload()");
  var acUtils = Cc["@flock.com/account-utils;1"]
                          .getService(Ci.flockIAccountUtils);
  var params = {
    serviceId: acUtils.getServiceIDForAccountURN($('servicesMenuList').selectedItem.value),
    accountURN: $('servicesMenuList').selectedItem.value,
    globalTags: $('batch_tags').value,                            
  };

  // JMC - Extending params to contain all the global settings as well, 
  // They'll get munged into the individual photouploads in the next step
  
  DEBUG("doUpload(): Get the photos");
  params.photoUploads = this.getAllPhotoUploads();
  var rv = flock_photo.util.launchUploadOptions(params);

  this.uploadListener.mNewAlbum = params.newAlbum;

  if (params.launchUploader) {
    $('flock-photo-upload-deck').selectedIndex = 1;
  }

  // Merging tags
  function mergeTags(listA, itemsB) {
    var itemsA = listA.split(",");
    if (listA == "" || listA == ",") {
      itemsA = [];
    }
    for (x in itemsB) {
      if (itemsA.indexOf(x) == -1) {
        itemsA.push(itemsB[x]);
      }
    }
    return itemsA.join(",");
  }

  var globalTagList = params.globalTags.split(",");
  if (params.globalTags == "" || params.globalTags == ",") globalTagList = [];
  var globalPub = ($('batch_is_public').value == "public"); 
  var globalIsFriend = $('batch_is_friend').checked; 
  var globalIsFamily = $('batch_is_family').checked; 
  // set the album id on the photo uploads
  // this should probably be an attribute on the service
  // not each particular photo upload object
  var uploads = this.getAllPhotoUploads();
  for (var i = 0; i < uploads.length; i++) {
    // Apply the global title & description if the image doesn't have one
    if (uploads[i].title == "")
      uploads[i].title = $('batch-title').value;
    if (uploads[i].description == "")
      uploads[i].description = $('batch-description').value;

    uploads[i].album = params.albumId;
    uploads[i].tags = mergeTags(uploads[i].tags, globalTagList);
    if (uploads[i].privacy_use_batch == "true") {
      uploads[i].is_public = globalPub;
      if (globalPub) {
        uploads[i].is_friend = false;
        uploads[i].is_family = false;
      } else {
        uploads[i].is_friend = globalIsFriend;
        uploads[i].is_family = globalIsFamily;
      }
    } else if (uploads[i].is_public == "true") {
      uploads[i].is_friend = false;
      uploads[i].is_family = false;
    }
    this.SVC.savePhotoUpload(uploads[i]);
  }

  if (params.apiName) {
    window.setTimeout(this.reallyDoUpload, 0, params.serviceId);
    this.updateCommandState(true);
  }
}

photoUploader.reallyDoUpload = function(aServiceId) {
  DEBUG("reallyDoUpload('"+aServiceId+"')");
  var api = Cc[aServiceId].getService(Ci.flockIPhotoAPI);
  photoUploader.SVC.upload(api);
}

photoUploader.removeSelection = function(aSelection) {
  DEBUG("removeSelection()");
  if (!aSelection) {
    aSelection = $("flock-photo-upload-slideybox").getSelection();
  }

  photoProperties.clearSelection();
  var inst = this;
  var domFxListener = {
    onComplete: function(aElement) {
      var spacer = document.createElement('spacer');
      spacer.setAttribute('width', '75');
      aElement.parentNode.parentNode.insertBefore(spacer, aElement.parentNode);
      
      collapseListener = {
        onComplete: function(aElement) {
          aElement.parentNode.removeChild(aElement);
        },
      }
      gDomFx.collapse(spacer, collapseListener, "horizontal", "75");
      inst.SVC.removePhotoUpload(aElement);
    },
  }
  
  for (var i = 0; i < aSelection.length; ++i) {
    gDomFx.fadeOut(aSelection[i].firstChild, domFxListener);  
    aSelection[i].selected = false;
  }
  
  var newSelected = aSelection[aSelection.length-1].nextSibling || aSelection[0].previousSibling;
  if (newSelected) {
    newSelected.selected = true;
    newSelected.focus();
    // todo -- make this smoothly scroll to this new element
    $("flock-photo-upload-slideybox").ensureElementIsVisible(newSelected);
  }
  return;
  
}

photoUploader.startUploadFlow = function() {
  DEBUG("startUploadFlow()");
  var inst = this;
  var accountURN = $("servicesMenuList").value;
  var service = null;
  if (accountURN) {
    var acUtils = Cc["@flock.com/account-utils;1"]
                            .getService(Ci.flockIAccountUtils);
    var serviceId = acUtils.getServiceIDForAccountURN(accountURN);
    service = Cc[serviceId]
                        .getService(Ci.flockIPhotoAPI);
  }
  if (service) {
    DEBUG("startUploadFlow(): using service: "+service.title);
    if (service.isLoggedIn) {
      DEBUG("startUploadFlow(): service "+service.title+" is logged in");
      this.doUpload();
    } else {
      DEBUG("startUploadFlow(): service "+service.title+" is logged out");
      var loginListener = {
        onAuth: function() {
          DEBUG("startUploadFlow(): loginListener: onAuth()");
          if (service.isLoggedIn) {
            inst.doUpload();
          }
        },
        onError: function(aError) {
          DEBUG("startUploadFlow(): loginListener: onError(): "+aError.errorString);
        },
      }
      var localeService = Cc["@mozilla.org/intl/nslocaleservice;1"].getService(Ci.nsILocaleService);
      var stringBundleService = Cc["@mozilla.org/intl/stringbundle;1"].getService(Ci.nsIStringBundleService);
      var stringBundle = stringBundleService.createBundle("chrome://flock/locale/photo/photo.properties", localeService.getApplicationLocale());

      // there is a bug in the apimanager which causes multiple state changes
      // which prevents us to use it -- for now call the login on the
      // photoservice directly
      DEBUG("startUploadFlow(): logging in to service "+service.title);
      service.login(accountURN, loginListener);
    }
  } else {
    var params = {};
    params.hasValidSvc = false;
    rv = flock_photo.util.launchSettings(params);
    if (params.hasValidSvc) {
      this.doUpload();
    }
  }
}

function getUnicodePref(prefName, prefBranch) {
  if (prefBranch.prefHasUserValue(prefName)) {
    return prefBranch.getComplexValue(prefName, Ci.nsISupportsString).data;
  }
  return null;
}

function setUnicodePref(prefName, prefBranch, prefValue) {
  var sString = Cc["@mozilla.org/supports-string;1"]
                          .createInstance(Ci.nsISupportsString);
  sString.data = prefValue;
  prefBranch.setComplexValue(prefName, Ci.nsISupportsString, sString);
}

function openHelp() {
  window.open('http://help.flock.com/content.php?title=Uploading___organizing_online_photos', 'FlockHelp', 'width=555,height=545,resizable,scrollbars');
}
