//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2007
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

const SELECTED_SEARCH_ENGINE_PREF = "browser.search.selectedEngine";
const SEARCH_ELSEWHERE_PREF = "flock.search.excludedEngines";

var searchFlyout = {

  init: function() {
    this._logger = Cc["@flock.com/logger;1"].createInstance(Ci.flockILogger);
    this._logger.init("searchflyout");
    this.mFlyout = document.getElementById("searchFlyout");
    this.mIsOpen = false;
    this.mSearchbar = document.getElementById("searchbar");       
    this.mSearchMgrSvc = Cc["@flock.com/search-manager;1?"].getService(Ci.flockISearchManager);
    this.mPrefSvc = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch2);
    this.mPrefSvc.addObserver("flock.liveSearch.enabledServices", this, false);
  },
  
  uninit: function() {
    this.mPrefSvc.removeObserver("flock.liveSearch.enabledServices", this);
  },
  
  focus: function() {
    this.mFlyout.focus();
  },
  
  hide: function() {
    this._logger.debug("hiding the flyout");
    this.mFlyout.hidePopup();
  },
  
  show: function() {
    if (this.isOpen()) {
      return;
    }
    this._logger.debug("showing the flyout");
    searchbarController.updateSearchElsewhere();
    searchbarController.updateNavigatableResults();
    searchbarController.clearSelection();
    this.mFlyout.showPopup(this.mSearchbar, -1, -1, "popup", "bottomleft", "topleft");
  },
  
  toggle: function() {
    if (this.isOpen()) {
      this.hide();
    } else {
      searchbarController.focus();
      this.show();
    }
  },
  
  isOpen: function() {
    return this.mIsOpen;
  },
  
  observe: function(aSubject, aTopic, aData) {
    if (aData == "flock.liveSearch.enabledServices") {
      searchbarController.init();
    }
  },
  
  handleEvent: function(aEvent) {
    switch (aEvent.type) {
      case "load":
        this.init();
        break;
      case "unload":
        this.uninit();
        break;
    }
  }
};

var searchbarController = {

  debugLog: function(debugMessage) {
    if ("_logger" in this) {
      this._logger.debug(debugMessage);
    }
  },

  mSearchTimer: null,
  mCurrentIndex: null,
  mResults: null,
  mSelectedItem: null,
  mSearchbar: null,
  mFavsDS: null,
  mHistoryDS: null,
  mFlyoutXULContainer: null,
  mPrefService: null,
  mSearchMgrService: null,
  mSearchElsewhereXulContainer: null,

  init: function() {
    this._logger = Cc["@flock.com/logger;1"].createInstance(Ci.flockILogger);
    this._logger.init("searchbarcontroller");
    this.debugLog("init");
    this.mSearchbar = document.getElementById("searchbar");
    this.mFlyout = document.getElementById("searchFlyout");
    this.addEventListeners();
    this.mCurrentIndex = -1;
    this.mResults = []; // used for keyboard navigation of results rows
    this.mFlyoutXULContainer = document.getElementById("flyoutContainer");
    this.mSearchElsewhereXulContainer = document.getElementById("searchElsewhereContainer");
    this.mSearchMgrService = Cc["@flock.com/search-manager;1?"].getService(Ci.flockISearchManager);

    this.mSearchTimer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
    this.mInitTimer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
    this.mPrefService = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService);
    this.mPrefBranch = this.mPrefService.QueryInterface (Ci.nsIPrefBranch2);
    this.mPrefBranch.addObserver(SELECTED_SEARCH_ENGINE_PREF, this, false);
    this.mPrefBranch.addObserver(SEARCH_ELSEWHERE_PREF, this, false);

    // reference rdf services
    this.mRDFService = Cc["@mozilla.org/rdf/rdf-service;1"].getService(Ci.nsIRDFService);
    this.mRdfContainerUtils = Cc["@mozilla.org/rdf/container-utils;1"].getService(Ci.nsIRDFContainerUtils);

    this.mFavsDS = this.mRDFService.GetDataSource("rdf:flock-favorites");
    this.mHistoryDS = this.mRDFService.GetDataSource("rdf:history");
    this.doLiveSearch();
    this.updateNavigatableResults();
    this.mInitTimer.initWithCallback(this, 50, Ci.nsITimer.TYPE_ONE_SHOT);
  },
  
  uninit: function() {
    this.debugLog("uninit");
    this.mPrefService.removeObserver(SELECTED_SEARCH_ENGINE_PREF, this);
    this.mPrefBranch.removeObserver(SEARCH_ELSEWHERE_PREF, this);
    this.removeListeners();
  },
  
  handleEvent: function(aEvent) {
    this.debugLog("handleEvent: type = " + aEvent.type);
    switch (aEvent.type) {
      case "load":
        this.init();
        break;
      case "unload":
        this.uninit();
        break;
      case "blur":
        searchFlyout.hide();
        break;
      case "input":
        this.mSearchTimer.cancel();
        this.mSearchTimer.initWithCallback(this, 500, Ci.nsITimer.TYPE_ONE_SHOT);
        searchFlyout.show();
        break;
      case "keypress":
        this.handleKeyInput(aEvent);
        break;
      case "select":
        // this event is for tab selection change
        searchFlyout.hide();
        break;
      case "focus":
        //this.mSearchbar.select();
        break;
    }
  },

  doLiveSearch: function(aSearchText) {
    var inst = this;
    // needs to be a local var as it is passed through all the enabled services
    var resultsDS = Cc["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
      .createInstance(Ci.nsIRDFDataSource);
    

    var root = this.mRdfContainerUtils.MakeBag(resultsDS,
      this.mRDFService.GetResource("urn:flock:livesearch:root"));

    var theEnum = this.mSearchMgrService.enabledServices;
    while (theEnum.hasMoreElements()) {
      var service = theEnum.getNext();

      var subject = this.mRDFService.GetResource(service.shortName);

      var predicate = this.mRDFService.GetResource("http://home.netscape.com/NC-rdf#Name");
      var name = this.mRDFService.GetLiteral(service.serviceName);
      resultsDS.Assert(subject, predicate, name, true);

      predicate = this.mRDFService.GetResource("http://home.netscape.com/NC-rdf#ref");
      name = this.mRDFService.GetLiteral(service.ref);
      resultsDS.Assert(subject, predicate, name, true);

      // make this a container
      var enginesContainer = this.mRdfContainerUtils.MakeSeq(resultsDS, subject);
      root.AppendElement(subject);
    }

    // start the live search
    // remove any datasources before trying to add the new search results
    var sources = this.mFlyoutXULContainer.database.GetDataSources();
    while (sources.hasMoreElements()) {
      var source = sources.getNext();
      this.mFlyoutXULContainer.database.RemoveDataSource(source);
    }

    this.mFlyoutXULContainer.database.AddDataSource(resultsDS);
    this.mSearchMgrService.search(aSearchText, 4, searchCallback, resultsDS)
    this.mCurrentIndex = -1;
    this.mFlyoutXULContainer.builder.rebuild();
  },

  handleKeyInput: function(aEvent) {
    this.debugLog("handleKeyInput: keyCode = " + aEvent.keyCode + " charCode = " + aEvent.charCode);
    if (aEvent.metaKey || aEvent.ctrlKey || aEvent.altKey) {
      return;
    }
    if (aEvent.keyCode == KeyEvent.DOM_VK_ESCAPE) {
      searchFlyout.hide();
      aEvent.stopPropagation();
      aEvent.preventDefault();
      return;
    }
    if (aEvent.charCode != 0 ||
        aEvent.keyCode == KeyEvent.DOM_VK_BACK_SPACE ||
        aEvent.keyCode == KeyEvent.DOM_VK_DELETE)
    {
      this.mSearchTimer.cancel();
      this.mSearchTimer.initWithCallback(this, 500, Ci.nsITimer.TYPE_ONE_SHOT);
      searchFlyout.show();
    } else if (aEvent.keyCode == KeyEvent.DOM_VK_TAB) {
      searchFlyout.hide();
    } else if (aEvent.keyCode == KeyEvent.DOM_VK_DOWN) {
      if (searchFlyout.isOpen()) {
        this.selectNextItem();
      } else {
        searchFlyout.show();
      }
      aEvent.stopPropagation();
      aEvent.preventDefault();
    } else if (aEvent.keyCode == KeyEvent.DOM_VK_UP) {
      if (searchFlyout.isOpen()) {
        this.selectPreviousItem();
      }
      aEvent.stopPropagation();
      aEvent.preventDefault();
    } else if (aEvent.keyCode == KeyEvent.DOM_VK_RETURN ||
               aEvent.keyCode == KeyEvent.DOM_VK_ENTER)
    {
      // check if our live results has a selected item from key navigation
      var selectedItem = this.getItemAtIndex(this.mCurrentIndex)
      if (selectedItem && selectedItem.id) {
        if (selectedItem.id == "prefs") {
          openPreferences("paneSearching");
        } else {
          openUILinkIn(selectedItem.getAttribute("url"),
                       getBoolPref("browser.search.openintab") ? "tab" : "current" );
        }
        aEvent.stopPropagation();
      }
      searchFlyout.hide();
    } else {
      this.debugLog("handleKeyInput: Unhandled keyCode = " + aEvent.keyCode);
    }
  },

  selectNextItem: function() {
    if (this.mCurrentIndex == this.mResults.length - 1) {
      this.mCurrentIndex = 0;
    } else {
      this.mCurrentIndex++;
    }
    this.selectItemAtIndex(this.mCurrentIndex);
  },

  selectPreviousItem: function() {
    if (this.mCurrentIndex == -1) {
      this.mSelectedItem = null;
      return;
    }
    if (this.mCurrentIndex == 0) {
      this.mCurrentIndex = this.mResults.length - 1;
    } else {
      this.mCurrentIndex--;
    }
    this.selectItemAtIndex(this.mCurrentIndex);
  },

  selectItemAtIndex: function(aIndex) {
    if (this.mSelectedItem) {
      this.mSelectedItem.removeAttribute("selected");
    }

    var selectedItem = this.getItemAtIndex(aIndex);
    if (selectedItem) {
      this.mSelectedItem = selectedItem;
      this.mSelectedItem.setAttribute("selected", true);
    }
    this._logger.debug("selectItemAtIndex: aIndex = " + aIndex + " mSelectedItem = " + this.mSelectedItem.getAttribute("url"));
  },
  
  selectItem: function(aItem) {
    var itemIndex;
    for (var c = 0; c < this.mResults.length; c++) {
      if (this.mResults[c] == aItem) {
        itemIndex = c;
        break;
      }
    }
    if (this.mSelectedItem) {
      this.mSelectedItem.removeAttribute("selected");
    }
    this.mSelectedItem = aItem;
    this.mCurrentIndex = itemIndex;
    this.mSelectedItem.setAttribute("selected", true);
    this._logger.debug("selectItem: aItem = " + this.mSelectedItem + " mSelectedItem = " + this.mSelectedItem.getAttribute("url"));
  },
  
  clearSelection: function() {
    if (this.mSelectedItem) {
      this.mSelectedItem.removeAttribute("selected");
      this.mSelectedItem = null;
      this.mCurrentIndex = -1;
    }
  },

  getItemAtIndex: function(aIndex) {
    return this.mResults[aIndex];
  },
  
  focus: function() {
    this.mSearchbar.focus();
  },

  updateNavigatableResults: function() {
    var results = this.mFlyout.getElementsByAttribute("result", "true");
    var items = [];
    for (var j = 1; j < results.length; j++) {
      items.push(results[j]);
    }
    this.mResults = items;
  },

  addEventListeners: function() {
    this.debugLog("addEventListeners");
    this.mSearchbar.addEventListener("keypress", this, true);
    this.mSearchbar.addEventListener("focus", this, true);
    this.mSearchbar.addEventListener("input", this, true);
    this.mSearchbar.addEventListener("blur", this, true);
    document.getElementById("content").tabContainer.addEventListener("select", this, false);
    //window.addEventListener("click", this, true);
  },

  removeListeners: function() {
    this.debugLog("removeListeners");
    this.mSearchbar.removeEventListener("keypress", this, true);
    this.mSearchbar.removeEventListener("focus", this, true);
    this.mSearchbar.removeEventListener("input", this, true);
    this.mSearchbar.removeEventListener("blur", this, true);
    document.getElementById("content").tabContainer.removeEventListener("select", this, false);
    //window.removeEventListener("click", this, true);
  },

  updateSearchElsewhere: function() {
    this.debugLog("updateSearchElsewhere");
    var theEnum = this.mSearchMgrService.searchElsewhereList;

    var kids = this.mSearchElsewhereXulContainer.childNodes;
    for (var i = kids.length - 1; i >= 0; i--) {
      if (!(kids[i].getAttribute("dontDelete"))) {
        this.mSearchElsewhereXulContainer.removeChild(kids[i]);
      }
    }

    if (theEnum.hasMoreElements()) {
      document.getElementById("engineResultsContainer").removeAttribute("hidden");
    } else {
      document.getElementById("engineResultsContainer").setAttribute("hidden", "true");
    }

    while (theEnum.hasMoreElements()) {
      var engineItem = theEnum.getNext();
      var hbox = document.createElement("hbox");
      var label = document.createElement("label");
      var icondiv = document.createElementNS("http://www.w3.org/1999/xhtml", "div");
      var icon = document.createElementNS("http://www.w3.org/1999/xhtml", "img");
      icondiv.appendChild(icon);
      var url = this.getSearchElsewhereQString(engineItem.name);
      this.mSearchElsewhereXulContainer.appendChild(hbox);
      hbox.setAttribute("result", true);
      hbox.setAttribute("class", "flock-result");
      hbox.appendChild(icondiv);
      hbox.appendChild(label);
      hbox.setAttribute("onmouseover", "searchbarController.selectItem(this);");
      hbox.setAttribute("onmouseout", "searchbarController.clearSelection();");
      hbox.setAttribute("onclick",
                        "var metrics = Components.classes['@flock.com/metrics-service;1'].getService(Components.interfaces.flockIMetricsService);" +
                        "metrics.reportCount('search "+engineItem.name+"');" +
                        "var url = searchbarController.getSearchElsewhereQString('"+engineItem.name+"');"
                        + "if (getBoolPref('browser.search.openintab')) {"
                        + "  openUILinkIn(url, 'tab');"
                        + "} else {"
                        + "  openUILink(url, event);"
                        + "}" );
      hbox.setAttribute("id", engineItem.name + "fullResults");
      icon.setAttribute("src", engineItem.iconURI.spec);
      label.setAttribute("value", engineItem.name);
      label.setAttribute("class", "primary-result");
    }
  },

  getSearchElsewhereQString: function(aEngineName) {
    this.debugLog("getSearchElsewhereQString: aEngineName = " + aEngineName);
    var engine = this.mSearchbar.searchService.getEngineByName(aEngineName);
    if (!engine) return;
    var query = this.mSearchbar.value;
    var searchSubmission = engine.getSubmission(query, null);
    var itemActionString = searchSubmission.uri.spec;
    return itemActionString;
  },
  
  notify: function(aTimer) {
    if (aTimer == this.mSearchTimer) {
      this.doLiveSearch(this.mSearchbar.value);
      aTimer.cancel();
    } else if (aTimer == this.mInitTimer) {
      this.updateSearchElsewhere();
    }
  },
  
  observe: function(aSubject, aTopic, aData) {
    this.debugLog("observe: aTopic = " + aTopic);
    if (aTopic == "nsPref:changed") {
      switch (aData) {
        case SELECTED_SEARCH_ENGINE_PREF:
          var newEngine;
          try {
            newEngine = this.mPrefService.getCharPref(aData);
          } catch (ex) {
            newEngine = this.mPrefService.getComplexValue("browser.search.defaultenginename",
                                                          Ci.nsIPrefLocalizedString).data;
          }
          if (newEngine && newEngine.length) {
            // string 2 nsISearchEngine conversion
            var selectedEngine = this.mSearchbar.searchService.getEngineByName(newEngine);
            if (selectedEngine) {
              this.mSearchbar.currentEngine = selectedEngine;
            }
          }
          break;

        case SEARCH_ELSEWHERE_PREF:
          this.updateSearchElsewhere();
          break;

        default:
          break;
      }
    }

    this.updateNavigatableResults();
  }
};

var searchCallback = {
  foundResults: function(aCount, aResults, aShortName, aQuery) {
    var resultBox = document.getElementById(aShortName);
    if (!resultBox) return;

    resultBox.setAttribute("empty", aCount == 0);
    resultBox.removeAttribute("busy");
    resultBox.firstChild.nextSibling.firstChild.hidden = true;

    // create the view all results item
    var hbox = document.createElement("hbox");
    var label = document.createElement("label");
    var service = searchbarController.mSearchMgrService.getServiceFromShortname(aShortName);
    // to do add a .hasFullResultsUrl in the interface
    if (service && aCount && aShortName != "flock") {
      hbox.setAttribute("result", true);
      var engineQString = service.fullResultsUrl + encodeURIComponent(aQuery);
      var itemActionString = "if (getBoolPref('browser.search.openintab', false)) {"
                           + "  openUILinkIn('"+engineQString+"', 'tab');"
                           + "} else {"
                           + "  openUILink('"+engineQString+"', event);"
                           + "}"
                           + "$('searchFlyout').hidePopup();"
                           + "event.preventDefault();";
      label.setAttribute("value", "Show all "+service.serviceName+" results ...");
      hbox.appendChild(label);
      hbox.setAttribute("onmouseover", "searchbarController.selectItem(this);");
      hbox.setAttribute("onmouseout", "searchbarController.clearSelection();");
      hbox.setAttribute("onclick", itemActionString);
      hbox.setAttribute("id", service.shortName+"FullResults");
      hbox.setAttribute("url", engineQString);
      hbox.setAttribute("class", "view-all-results flock-result");
      hbox.setAttribute("result", "true"); // so its accessible by our key nav
      resultBox.firstChild.nextSibling.appendChild(hbox);
    }
    searchbarController.updateNavigatableResults();
  },

  searchStarted: function(aShortName) {
    dump(aShortName+" search started >>> \n");
    var liveSearchElem = document.getElementById(aShortName);
    if (liveSearchElem) {
      liveSearchElem.firstChild.nextSibling.firstChild.hidden = false;
      liveSearchElem.removeAttribute("empty");
      liveSearchElem.setAttribute("busy", true);
    }
  }
};


function updateSearchDiscoveryPopup() {
  var searchButton = $("favoritesSearchButton");

  // Clear out the old menuitems
  var container = searchButton.firstChild;
  while (container.lastChild) {
    container.removeChild(container.lastChild);
  }
  
  var searchService = Cc["@mozilla.org/browser/search-service;1"].getService(Ci.nsIBrowserSearchService);
  var newMenuItem, miObj;
  var engTypes = ["engines", "hiddenEngines"];
  for (var e in engTypes) {
    var engines = gBrowser.mCurrentBrowser[engTypes[e]];
    if (!engines) continue;
    for (var c = 0; c < engines.length; c++) {
      newMenuItem = document.createElement("menuitem");
      miObj = {
        class: "menuitem-iconic",
        label: engines[c].title,
        oncommand: "addSearchEngine(this); updateSearchMessage(); event.stopPropagation();",
        src: engines[c].icon,
        uri: engines[c].uri
      };
      if (searchService.getEngineByName(engines[c].title)) {
        miObj.image = "chrome://browser/skin/flock/common/star16.png";
        miObj.disabled = "true";
      }
      for (p in miObj) {
        newMenuItem.setAttribute(p, miObj[p]);
      }
      newMenuItem.setAttribute("discovered", "false");
      searchButton.firstChild.appendChild(newMenuItem);
    }
  }
}

function updateSearchMessage() {
  var nBox = gBrowser.getNotificationBox(); 
  nBox.removeCurrentNotification("search-discovered");
  
  var message = "Search engine added. Click the search engine icon in the search widget at any time to select the search engine you want to use.";
  nBox.appendUniqueNotification(message,
                                  "search-discovered-info",
                                  (document.getElementById("page-proxy-favicon"))?document.getElementById("page-proxy-favicon").src:null,
                                  nBox.FLOCK_PRIORITY_MEDIUM,
                                  null);
}

function addSearchEngine(aTarget) {
  var searchService = Cc["@mozilla.org/browser/search-service;1"].getService(Ci.nsIBrowserSearchService);
  var type = Ci.nsISearchEngine.DATA_XML;
  searchService.addEngine(aTarget.getAttribute("uri"), type, aTarget.getAttribute("src"), false);
}


window.addEventListener("load", searchFlyout, false);
window.addEventListener("unload", searchFlyout, false);

window.addEventListener("load", searchbarController, false);
window.addEventListener("unload", searchbarController, false);
