/*
 * BEGIN FLOCK GPL
 * 
 * Copyright Flock Inc. 2005-2007
 * http://flock.com
 * 
 * This file may be used under the terms of of the
 * GNU General Public License Version 2 or later (the "GPL"),
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 * 
 * END FLOCK GPL
 */

const FOLDERICON = 'chrome://browser/skin/flock/drawers/folder.png';

var gShelf;
var gShelfContext;
var gCurrentItem = null;

var dragService = Components.classes["@mozilla.org/widget/dragservice;1"].getService(Components.interfaces.nsIDragService);

function createNotesFolder(aFolder) {
  var thumbnail = FOLDERICON;
  var title = aFolder.name;
  var folderUrn = aFolder.id();
 
  var item = document.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "media");
  item.setAttribute('type', 'person'); // Ok, this is not a person but it works anyway
  item.setAttribute('thumbnail', thumbnail);
  item.setAttribute('title', title);
  item.setAttribute('tooltiptext', title);
  item.setAttribute('url', folderUrn);
  item.setAttribute('onclick', 'goToFolder(this.url)');

  var slidey = document.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "slideyitem");
  slidey.appendChild(item);

  return slidey;
}


var faves = Components.classes['@mozilla.org/rdf/datasource;1?name=flock-favorites'].getService(Components.interfaces.flockIFavoritesService);
faves.QueryInterface(Components.interfaces.nsIRDFDataSource);

var faves_coop = Components.classes['@flock.com/singleton;1'].getService(Components.interfaces.flockISingleton).getSingleton('chrome://browser/content/flock/common/load-faves-coop.js').wrappedJSObject;


function onLoad_shelf() {
  var metrics = Components.classes['@flock.com/metrics-service;1']
    .getService(Components.interfaces.flockIMetricsService);
  metrics.reportCount('shelf open');

  window.gShelfTreeObserver = {
    canDrop: function (aDOMNode, aOrientation) {
      // The only drop we don't support is on items (on folders, or before/after is OK)
      if ((aDOMNode.tagName == "richtreeitem") && (aOrientation == "on"))
        return false;
      return true;
    },
    onDrop: function (aDOMNode, aOrientation, aDropData) {
      var dragSession = dragService.getCurrentSession();
      var data = flock_getMarkedUpSelection(dragSession);
      addObject (data, aDOMNode, aOrientation);
    }
  }

  $("notesContainer").addObserver(window.gShelfTreeObserver);

  // Add an observer to refresh the clear all button/message (empty or not)
  window.gShelfRDFObserver = {
    onAssert: function () { refreshNotesUI(); },
    onUnassert: function () { refreshNotesUI(); },
    onBeginUpdateBatch: function () { },
    onChange: function () { },
    onEndUpdateBatch: function () { },
    onMove: function () { }
  };
  faves.AddObserver(window.gShelfRDFObserver);
  refreshNotesUI();
}

function onUnload_shelf() {
  var metrics = Components.classes['@flock.com/metrics-service;1']
    .getService(Components.interfaces.flockIMetricsService);
  metrics.reportCount('shelf close');

  faves.RemoveObserver(window.gShelfRDFObserver);
  $("notesContainer").removeObserver(window.gShelfTreeObserver);
}

function refreshNotesUI() {
  // dump("FIXME: do the following only when the change applies to a shelf object\n");
  if (shelf_service.elementsCount() == 0) {
    $('clear').setAttribute("disabled", "true");
    $('shelfDeck').setAttribute("selectedIndex", 0);
  }
  else {
    $('clear').setAttribute("disabled", "false");
    $('shelfDeck').setAttribute("selectedIndex", 1);
  }
}

function openSelected(aLocation) {
  var itemId = $('notesContainer').selectedItems[0].id;
  
  var item = faves_coop.get(itemId);
  var params = {
    title: item.name,
    content: item.content
  }
  
  window.openDialog("chrome://browser/content/flock/shelf/previewSnippet.xul", "_blank",
                    "chrome,close,titlebar,resizable=yes,toolbar=no,scrollbars=yes,centerscreen",
                    params); 
}


function deleteSelected() {
  var bundle = flock_shelf.getStringBundle();
  var items = $('notesContainer').selectedItems;

  var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                    .getService(Components.interfaces.nsIPromptService);
  if (prompts.confirm(window, bundle.GetStringFromName("flock.shelf.deleteSelected.title"),
                      bundle.GetStringFromName("flock.shelf.deleteSelected"))) {
    for (i = 0; i < items.length; i++)
      SHELF_SRV.remove(items[i].id);
  }
}

function deleteOne(aId) {
  var bundle = flock_shelf.getStringBundle();
  if (!aId && !$('notesContainer').selectedItems[0])
    return;
  var id = aId?aId:$('notesContainer').selectedItems[0].id;

  var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                    .getService(Components.interfaces.nsIPromptService);
  if (prompts.confirm(window, bundle.GetStringFromName("flock.shelf.deleteOne.title"),
                      bundle.GetStringFromName("flock.shelf.deleteOne")))
    SHELF_SRV.remove(id);
}

function deleteFolder(aId) {
  var bundle = flock_shelf.getStringBundle();
  if (!aId && !$('notesContainer').selectedItems[0])
    return;
  var id = aId?aId:$('notesContainer').selectedItems[0].id;

  var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                    .getService(Components.interfaces.nsIPromptService);
  if (prompts.confirm(window, bundle.GetStringFromName("flock.shelf.deleteFolder.title"),
                      bundle.GetStringFromName("flock.shelf.deleteFolder")))
    SHELF_SRV.remove(id);
}


function deleteAll() {
  var bundle = flock_shelf.getStringBundle();
  var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                    .getService(Components.interfaces.nsIPromptService);
  if (prompts.confirm(window, bundle.GetStringFromName("flock.shelf.menu.delete.title"),
                      bundle.GetStringFromName("flock.shelf.menu.delete")))
    SHELF_SRV.clear();
}

function refreshNotesContext() {
  var cardinal = $('notesContainer').selectedItems.length;
  if (cardinal > 1) {
    $('deleteOne').setAttribute('hidden', 'true');
    $('deleteSelected').setAttribute('hidden', 'false');
  }
  else if (cardinal > 0) {
    $('deleteOne').setAttribute('hidden', 'false');
    $('deleteSelected').setAttribute('hidden', 'true');
  }
  else {
    $('deleteOne').setAttribute('hidden', 'false');
    $('deleteOne').setAttribute('disabled', 'true');
    $('deleteSelected').setAttribute('hidden', 'true');
  }
}


function _getSrc(aItem) {
  if (aItem.nodeName == "richtreefolder") {
    var src = " ";
    for (var i = 0; i < aItem.childNodes.length; i++)
      src += _getSrc(aItem.childNodes[i]);
    return src;
  }
  else {
    return ("<p>" + aItem.firstChild.getAttribute('src') + "</p>");
  }
}

function getSrc(aItem) {
  if (aItem.nodeName == "richtreefolder")
    return _getSrc(aItem);
  else
    return aItem.firstChild.getAttribute('src');
}

var notesObserver = {
  onDragStart: function(event, transferData, action){
    var items = $('notesContainer').selectedItems;
    var src;
    if (items.length == 0)
      return;
    else if (items.length == 1)
      src = getSrc(items[0]);
    else {
      src = "";
      for (i = 0; i < items.length; i++)
        src += "<p>" + getSrc(items[i]) + "</p>";
    }

    dump("Sending: "+src+"\n");
    transferData.data=new TransferData();
    transferData.data.addDataForFlavour("text/html", src);
    transferData.data.addDataForFlavour("text/unicode", src);
    transferData.data.addDataForFlavour("flock/shelficon", src);
  },

  onDragOver: function(aEvent, aFlavor, aSession) {
  },
 
  onDrop: function (evt, dropData, asession) {
    if (gCurrentItem) {
      try { gCurrentItem.removeAttribute('dragOver'); }
      catch(e) { dump(e); }
    }
    // Files from desktop are not supported
    // (for some reason, files appear as text/unicode)
    if ((dropData.flavour.contentType == "text/unicode") &&
        (dropData.data.substring(0, 7) == "file://")) {
      return false;
    }
    var data = flock_getMarkedUpSelection(asession);
    addObject(data, null, null);
    return true;
  },

  getSupportedFlavours: function() {
    flavors = new FlavourSet();
    flavors.appendFlavour('text/html');
    flavors.appendFlavour('text/unicode');
    flavors.appendFlavour('text/x-moz-url');
    return flavors;
  }
}

function addObject(aData, aDOMNode, aOrientation) {
  var data = aData;
  var title;
  var content;
  var url;
  var parentNode = faves_coop.get('urn:flock:shelfroot');
  var pos = -1;
  var type = data.type;

  if (aDOMNode) {
    var aCoopNode = faves_coop.get(aDOMNode.id);
    if (aCoopNode instanceof faves_coop.Folder) {
      parentNode = aCoopNode;
      pos = -1;
    }
    else {
      parentNode = aCoopNode.getParent();
      pos = parentNode.children.indexOf(aCoopNode);
      if (aOrientation == "after")
        pos++;
    }
  }

  if (data.dragfrom) {
    var items = $('notesContainer').selectedItems;

    for (i = 0; i < items.length; i++) {
      var item = faves_coop.get(items[i].id);
      // Remove the item from its original parent
      item.getParent().children.remove(item);
      // Add it where it should go
      if (pos < 0)
        parentNode.children.add(item);
      else
        parentNode.children.insertAt(item, pos++);
    }
  }
  else {
    SHELF_SRV.insert(data.title, data.content, data.URL, data.type, pos, parentNode.id());
  }

  // we also want to clear the selection
  // this.clearSelection();
}

function newFolder(event) {
  var bundle = flock_shelf.getStringBundle();
	var check = {value: false};
	var input = {value: ""};
  var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
	var result = promptService.prompt(null,
    bundle.GetStringFromName("flock.shelf.newFolder.title"),
    bundle.GetStringFromName("flock.shelf.newFolder"),
    input, null, check)
	if (result) {
		SHELF_SRV.createFolder(input.value);
	}
}
